/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.datacenter.dao.mappers

import com.persagy.server.datacenter.models.entities.assistant.ModelKey
import com.persagy.server.datacenter.models.entities.dict.Family
import com.persagy.server.datacenter.wanda.obj.Building
import com.persagy.server.datacenter.wanda.obj.Floor
import org.apache.ibatis.annotations.Mapper
import org.apache.ibatis.annotations.Param
import org.apache.ibatis.annotations.Select
import org.springframework.stereotype.Repository

/**
 * 方案接口
 *
 * @author 张维新
 */
@Mapper
@Repository
interface SchemeMapper {

    /**
     * 导入方案
     */
    @Select("insert into dict.property_def_project_info(first_tag,first_name,second_tag,second_name,info_point_name,info_point_code,datatype,type,flag,inputmode,unit,datasource,note,priority,visible,editable,sort,un_disabled,project_id) \n" +
            "select first_tag,first_name,second_tag,second_name,info_point_name,info_point_code,datatype,type,flag,inputmode,unit,datasource,note,priority,visible,editable,sort,un_disabled,#{projectId} from dict.datacenter_property_def_scheme WHERE scheme_id = #{schemeId}")
    fun setSchemeDict(@Param("projectId") projectId:String, @Param("schemeId") schemeId:String)

    /**
     * 自定义信息点
     */
    @Select("insert into dict.property_def_project_info(first_tag,first_name,second_tag,second_name,info_point_name,info_point_code,datatype,type,flag,inputmode,unit,datasource,note,priority,visible,editable,sort,un_disabled,project_id) \n" +
            "select first_tag,first_name,second_tag,second_name,info_point_name,info_point_code,datatype,type,flag,inputmode,unit,datasource,note,priority,visible,editable,sort,un_disabled,#{projectId} from dict.data_platform_scheme_info WHERE scheme_id = #{schemeId}")
    fun setSchemecustomDict(@Param("projectId") projectId:String, @Param("schemeId") schemeId:String)

    /**
     * 自定义信息点
     */
    @Select("insert into dict.property_def_project_info(first_tag,first_name,second_tag,second_name,info_point_name,info_point_code,datatype,type,flag,inputmode,unit,datasource,note,priority,visible,editable,sort,un_disabled,project_id) \n" +
            "select first_tag,first_name,second_tag,second_name,info_point_name,info_point_code,datatype,type,flag,inputmode,unit,datasource,note,priority,False,editable,sort,un_disabled,#{projectId} from dict.data_platform_scheme_info WHERE scheme_id = #{schemeId}")
    fun setSchemecustomDictFalse(@Param("projectId") projectId:String, @Param("schemeId") schemeId:String)

    /**
     * 导入原始字典
     */
    @Select("insert into dict.property_def_project_info(first_tag,first_name,second_tag,second_name,info_point_name,info_point_code,datatype,type,flag,inputmode,unit,datasource,note,priority,visible,editable,sort,un_disabled,project_id) \n" +
            "select first_tag,first_name,second_tag,second_name,info_point_name,info_point_code,datatype,type,flag,inputmode,unit,datasource,note,priority,visible,editable,sort,un_disabled,#{projectId} from dict.property_def_control_info ")
    fun setControlDict(@Param("projectId") projectId:String)

    /**
     * 操作自定义字典表数据
     */
    @Select("SELECT dictcustom.dict()")
    fun setDict():Boolean

    @Select("SELECT update_equip_ispace(#{projectId},#{oldBimId},#{bimId},#{floorId})")
    fun rUpDateEquipAndSapce(@Param("projectId") projectId: String,@Param("oldBimId") oldBimId: String,@Param("bimId") bimId: String,@Param("floorId") floorId: String):Boolean

    /**
     * 统计项目下有多少种设备类型
     *
     */
    @Select("SELECT category as code,(SELECT name FROM dict.class_def WHERE id = category) FROM equipment WHERE project_id = #{projectId} GROUP BY category")
    fun equipType(@Param("projectId") projectId: String):ArrayList<Family>


    /**
     * 信息点导入到项目定制化表
     */
    @Select("INSERT INTO dict.def_funcid_project (id,class_code,category,code,name,type,group_code,project_id,data_type,formater,data_source,input_mode,first_tag,second_tag,idx,priority,unit,note,can_sub,is_multiple,is_region,sub_flag,readonly,visible,editable,sort,un_disabled,object_type,key_word,last_update,create_time,first_name,second_name ) SELECT id,class_code,category,code,name,type,group_code,#{projectId},data_type,formater,data_source,input_mode,first_tag,second_tag,idx,priority,unit,note,can_sub,is_multiple,is_region,sub_flag,readonly,visible,editable,sort,un_disabled,object_type,key_word,last_update,create_time,first_name,second_name FROM dict.def_funcid WHERE group_code = #{groupId} AND class_code = #{code} and project_id = #{projectId}")
    fun dictInfo(@Param("projectId") projectId: String,@Param("groupId") groupId: String ,@Param("code") code:String )
    /**
     * 信息点导入到项目定制化表
     */
    @Select("INSERT INTO dict.def_funcid_project (id,class_code,category,code,name,type,group_code,project_id,data_type,formater,data_source,input_mode,first_tag,second_tag,idx,priority,unit,note,can_sub,is_multiple,is_region,sub_flag,readonly,visible,editable,sort,un_disabled,object_type,key_word,last_update,create_time,first_name,second_name ) SELECT id,#{propertyCode},category,code,name,type,group_code,#{projectId},data_type,formater,data_source,input_mode,first_tag,second_tag,idx,priority,unit,note,can_sub,is_multiple,is_region,sub_flag,readonly,visible,editable,sort,un_disabled,object_type,key_word,last_update,create_time,first_name,second_name FROM dict.def_funcid WHERE group_code = #{groupId} AND class_code = #{code} and project_id = #{projectId}")
    fun dictInfoProperty(@Param("projectId") projectId: String,@Param("groupId") groupId: String ,@Param("code") code:String ,@Param("propertyCode") propertyCode: String )
    /**
     * 模型下载 key
     *
     * @param projectId     项目 id
     * @param floorId       楼层 id
     * @return 模型下载 key
     */
    @Select("SELECT bim_id as bimIdPre,model_id as key FROM equipment WHERE project_id = #{projectId} AND floor_id = #{floorId} AND not bim_id ISNULL LIMIT 1")
    fun modelKey(@Param("projectId") projectId: String,@Param("floorId") floorId: String): ModelKey

    /**
     * 查询模型文件 id
     */
    @Select("select  m.id as key ,(select floor_name from revit.model_floor where current_model_id in (m.id)) as file_name,(SELECT name FROM revit.model_folder WHERE id = #{id}) AS folder_name from  revit.model_file m where id in (select current_model_id from revit.model_floor where folder_id in (#{id}))")
    fun modelId(@Param("id") id: String): ArrayList<ModelKey>


    /**
     * 统计建筑id
     */
    @Select("SELECT building_id AS id FROM public.equipment WHERE project_id = #{projectId} AND  NOT building_id IS NULL GROUP BY building_id")
    fun buildingId(@Param("projectId") projectId: String):ArrayList<Building>

    /**
     * 统计楼层id
     */
    @Select("SELECT floor_id AS id FROM public.equipment WHERE project_id = #{projectId} AND  NOT floor_id IS NULL GROUP BY floor_id")
    fun floorId(@Param("projectId") projectId: String):ArrayList<Building>

    /** 空间的建筑 id 统计 */
    @Select("SELECT building_id AS id FROM public.zone_space_base WHERE project_id = #{projectId} AND  NOT building_id IS NULL GROUP BY building_id")
    fun spBuildingId(@Param("projectId") projectId: String):ArrayList<Building>

    /** 空间的楼层 id 统计 */
    @Select("SELECT floor_id AS id FROM public.zone_space_base WHERE project_id = #{projectId} AND  NOT floor_id IS NULL GROUP BY floor_id")
    fun spFloorId(@Param("projectId") projectId: String):ArrayList<Floor>

    /**
     * 统计楼层空间类型不为空的
     */
    @Select("SELECT building_id FROM public.zone_space_base WHERE project_id = #{projectId}  and NOT building_id IS NULL  AND object_type = #{zoneType} GROUP BY building_id")
    fun spbuildingIdZoneType(@Param("projectId") projectId: String,@Param("zoneType") zoneType: String):ArrayList<Building>

    /**
     * 空间类型和关键字都不为空
     */
    @Select("SELECT building_id FROM zone_space_base WHERE project_id = #{projectId} AND NOT building_id IS NULL  AND  object_type = #{zoneType} AND   (local_name like'%#{vague}%' or  local_id  like '%#{vague}%') GROUP BY building_id ")
    fun spbuildingIdVagueToZoneType(@Param("projectId") projectId: String,@Param("zoneType") zoneType: String,@Param("vague") vague: String):ArrayList<Building>

    /**
     * 关键字不为空的
     */
    @Select("SELECT building_id FROM zone_space_base WHERE project_id = #{projectId}  AND NOT building_id IS NULL  AND   (local_name like'%#{vague}%' or  local_id  like '%#{vague}%') GROUP BY building_id ")
    fun spbuildingIdVague(@Param("projectId") projectId: String,@Param("vague") vague: String):ArrayList<Building>

    /**
     * 统计楼层空间类型不为空的
     */
    @Select("SELECT floor_id FROM public.zone_space_base WHERE project_id = #{projectId}  and NOT floor_id IS NULL  AND object_type = #{zoneType} GROUP BY floor_id")
    fun spfloorIdZoneType(@Param("projectId") projectId: String,@Param("zoneType") zoneType: String):ArrayList<Floor>

    /**
     * 空间类型和关键字都不为空
     */
    @Select("SELECT floor_id FROM zone_space_base WHERE project_id = #{projectId} AND NOT floor_id IS NULL  AND  object_type = #{zoneType} AND   (local_name like'%#{vague}%' or  local_id  like '%#{vague}%') GROUP BY floor_id ")
    fun spfloorIdVagueToZoneType(@Param("projectId") projectId: String,@Param("zoneType") zoneType: String,@Param("vague") vague: String):ArrayList<Floor>

    /**
     * 关键字不为空的
     */
    @Select("SELECT floor_id FROM zone_space_base WHERE project_id = #{projectId}   AND NOT floor_id IS NULL  AND   (local_name like'%#{vague}%' or  local_id  like '%#{vague}%') GROUP BY floor_id ")
    fun spfloorIdVague(@Param("projectId") projectId: String,@Param("vague") vague: String):ArrayList<Floor>


}