/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.models.entities.graphtype.rel

import RelBuilding
import com.persagy.server.datacenter.models.entities.base.BaseInfo
import com.persagy.service.models.annotations.SCascade
import io.swagger.v3.oas.annotations.media.Schema
import javax.persistence.Column
import javax.persistence.Id
import javax.persistence.Table

/**
 * 系统信息实体类
 *
 * @author 张维新
 */
@Schema(description = "系统信息实体类")
@Table(name = "general_system")
open class RelGeneralSystemBuilding : BaseInfo() {

    /** 系统ID */
    @Schema(description = "系统ID")
    @Id
    @Column(name = "id")
    var id: String? = null

    /** 系统名称 */
    @Schema(description = "系统名称")
    @Column(name = "name")
    var name: String? = null

    /** 系统本地编码 */
    @Schema(description = "系统本地编码")
    @Column(name = "local_id")
    var localId: String? = null

    /** 系统本地名称 */
    @Schema(description = "系统本地名称")
    @Column(name = "local_name")
    var localName: String? = null

    /** 设计图纸中编码 */
    @Schema(description = "设计图纸中编码")
    @Column(name = "cad_id")
    var cadId: String? = null

    /** BIM模型中编码 */
    @Schema(description = "BIM模型中编码")
    @Column(name = "bim_id")
    var bimId: String? = null

    /** 系统分类 */
    @Schema(description = "系统分类",required = true)
    @Column(name = "category")
    var category: String? = null

    /** 项目id */
    @Schema(description = "项目id",required = true)
    @Column(name = "project_id")
    var projectId: String? = null

    /** 建筑id */
    @Schema(description = "建筑id")
    @Column(name = "building_id")
    var buildingId: String? = null

    /** 楼层id */
    @Schema(description = "楼层id")
    @Column(name = "floor_id")
    var floorId: String? = null

    /** 设定参数 */
    @Schema(description = "设定参数")
    @Column(name = "setting_param")
    var settingParam: HashMap<String,Any?>? = null

    /** 技术参数 */
    @Schema(description = "技术参数")
    @Column(name = "tech_param")
    var techParam: HashMap<String,Any?>? = null

    /** 运行参数 */
    @Schema(description = "运行参数")
    @Column(name = "run_param")
    var runParam: HashMap<String,Any?>? = null

    /** 自定义 */
    @Schema(description = "自定义")
    @Column(name = "custom_param")
    var customParam: HashMap<String, Any?>? = null

    /** 运行状态 */
    @Schema(description = "运行状态")
    @Column(name = "running_state")
    var runningState: HashMap<String, Any?>? = null

    /** 对象类型 */
    @Schema(description = "对象类型")
    @Column(name = "object_type")
    override var objectType: String? = "GeneralSystem"

    /** 专业名称 */
    @Schema(description = "专业名称")
    @Column(name = "major_name",insertable = false,updatable = false)
    var majorName: String? = null

    /** 设备数量 */
    @Schema(description = "设备数量")
    @Column(name = "count",insertable = false,updatable = false)
    var count: Long? = 0

    /** 设备数量 */
    @Schema(description = "设备数量")
    @Column(name = "category_name",insertable = false,updatable = false)
    var categoryName: String? = null

    /** 反馈参数 */
    @Schema(description = "反馈参数")
    @Column(name = "feedback_param")
    var feedbackParam: HashMap<String,Any?>? = null

    /** 级联-系统服务于建筑 */
    @Schema(description = "级联-系统服务于建筑")
    @SCascade(table="r_sy_for_bd", idColumn="sys_id", childIdColumn="buildingId")
    var objectInfo: ArrayList<RelBuilding>? = null

}








