/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.datacenter.models.entities.task

import com.persagy.server.datacenter.datacenter.models.entities.objects.Property
import com.persagy.server.datacenter.models.entities.base.BaseInfo
import com.persagy.server.datacenter.models.entities.dict.EquipCategory
import com.persagy.server.datacenter.models.entities.task.SchemeUtils
import com.persagy.server.datacenter.wanda.obj.*
import com.persagy.service.models.annotations.SCascade
import io.swagger.v3.oas.annotations.media.Schema
import javax.persistence.Column
import javax.persistence.Id
import javax.persistence.Table

/**
 * 设备信息实体类、部件实体类
 *
 * @author  张维新
 */
@Schema(description = "设备信息实体类")
@Table(name = "equipment")
open class EquipmentTaskTwo : BaseInfo() {

    /** 设备id */
    @Schema(description = "设备id",required = true)
    @Id
    @Column(name = "id")
    var id: String? = null

    /** 设备名称 */
    @Schema(description = "设备名称",required = true)
    @Column(name = "name")
    var name: String? = null

    /** 设备本地编码 */
    @Schema(description = "设备本地编码")
    @Column(name = "local_id")
    var localId: String? = null

    /** 设备本地名称 */
    @Schema(description = "设备本地名称")
    @Column(name = "local_name")
    var localName: String? = null

    /** 设计图纸中编码 */
    @Schema(description = "设计图纸中编码")
    @Column(name = "cad_id")
    var cadId: String? = null

    /** BIM模型中编码 */
    @Schema(description = "BIM模型中编码")
    @Column(name = "bim_id")
    var bimId: String? = null

    /** 设备类型code */
    @Schema(description = "设备类型code")
    @Column(name = "class_code")
    var classCode: String? = null

    /** 方案id */
    @Schema(description = "方案id")
    var schemeId: String? = null
        get() {
            if (!classCode.isNullOrEmpty()){
                return SchemeUtils.setScheme(classCode!!)
            }
            return null
        }

    /** 部件关联设备id */
    @Schema(description = "部件关联设备id")
    @Column(name = "parent_id")
    var parentId: String? = null


    /** 资产 */
    @Schema(description = "级联-资产")
    @SCascade(table="property", idColumn="equip_id", childIdColumn="id")
    var property: Property? = null

    /** 建筑   table  对应关系表   idColumn 对应 本类的id    childIdColumn 级联对象的关系id */
    @Schema(description = "级联-建筑")
    @SCascade(table="equipment", idColumn="id", childIdColumn="building_id")
    var building: Building? = null

    /** 级联-楼层   table  对应关系表   idColumn 对应 本类的id    childIdColumn 级联对象的关系id */
    @Schema(description = "级联-楼层")
    @SCascade(table="equipment", idColumn="id", childIdColumn="floor_id")
    var floor: Floor? = null

    /** 级联-项目 */
    @Schema(description = "级联-项目")
    @SCascade(table="equipment", idColumn="id", childIdColumn="project_id")
    var project: Project? = null

    /** 级联-设备分类 */
    @Schema(description = "级联-设备分类")
    @SCascade(table="equipment", idColumn="id", childIdColumn="category")
    var equipCategory: EquipCategory? = null

    /** 级联-设备服务的空间的 */
    @Schema(description = "级联-设备服务的空间的")
    @SCascade(table="relationship.r_eq2sp", idColumn="eq_id", childIdColumn="sp_id",filter = "type = 'eq2sp_for'")
    var zoneSpaceForBase: ArrayList<ZoneSpace>? = null

    /** 级联-设备所在的空间的 */
    @Schema(description = "级联-设备所在的空间的")
    @SCascade(table="relationship.r_eq2sp", idColumn="eq_id", childIdColumn="sp_id",filter = "type = 'eq2sp_in'")
    var zoneSpaceInBase: ArrayList<ZoneSpace>? = null

    /** 级联-设备所在的竖井 */
    @Schema(description = "级联-设备所在的竖井")
    @SCascade(table="r_eq_in_sh", idColumn="equip_id", childIdColumn="shaft_id")
    var shaftList: ArrayList<Shaft>? = null

    /** 资产列表 */
    @Schema(description = "资产列表")
    var propertyList: ArrayList<Property>? = null

    /** 级联-部件   table  对应关系表   idColumn 对应 本类的id    childIdColumn 级联对象的关系id */
    @Schema(description = "级联-部件")
    @SCascade(table="equipment", idColumn="parent_id", childIdColumn="id")
    var component: ArrayList<EquipmentTaskTwo>? = null

}