/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.models.entities.tool

import com.alibaba.fastjson.annotation.JSONField
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.persagy.server.datacenter.wanda.AttPic
import com.persagy.server.datacenter.models.entities.assistant.TaskPic
import org.json.JSONObject
import org.slf4j.LoggerFactory
import java.io.Serializable

/**
 * Created by Xiaoyu on 2019/4/28
 */
open class ObjectBaseRequest : Serializable {
    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(ObjectBaseRequest::class.java)
    } // Companion object
    /** id */
    var id: String? = null

    /** 信息点 */
    var infos: HashMap<String, Any?>? = null

    //需要忽略的属性，数据平台中没有的信息点
    @JSONField(serialize=false)
    @Transient
    var ignoreKeySet = hashSetOf("LastUpdate", "CreateTime")

    //聚合在一起的字段名称，然后展开成数据平台需要的格式
    @JSONField(serialize=false)
    @Transient
    var collectKeySet = HashSet<String>()

    open fun addInfo(key: String, value: Any?) {
        addInfo(key, value, false)
    }
    /**
     * 添加信息点到infos
     * @param key   信息点
     * @param value 信息点值
     * @param ignoreNull    是否忽略value为null的情况，true value为null时不添加到infos
     */
    open fun addInfo(key: String, value: Any?, ignoreNull: Boolean) {
        if (collectKeySet.contains(key)) {
            addJSONObjectInfo(value , ignoreNull)
        } else {
            innerAddInfo(key, value, ignoreNull)
        }
    }

    /**
     * 添加信息点到infos
     * @param key   信息点
     * @param value 信息点值
     * @param ignoreNull    是否忽略value为null的情况，true value为null时不添加到infos
     */
     fun innerAddInfo(key: String, value: Any?, ignoreNull: Boolean) {
        if (ignoreKeySet.contains(key)) {
            return
        }
        if (value == null && ignoreNull) {
            return
        }
        infos = infos ?: LinkedHashMap()
        var valueArr = ArrayList<HashMap<String, Any?>>()

        val hashMap = HashMap<String, Any?>()
        if (value.toString()=="null" ){
//            valueArr[0]["value"] = null
            hashMap.put("value",null)
            valueArr.add(hashMap)
            infos!![key] = valueArr
//            logger.debug("value值0 = ${valueArr.toJson()}")
        }else{
            if (key == "Pic"||key == "InstallDrawing"||key == "InstallPic"||key == "CheckReport"||
                    key == "InsuranceFile"||key == "Nameplate" || key == "Drawing" || key == "Archive"||
                     key == "OperationManual" || key == "ProductCertification"||key == "TestReport"
                    ||key == "OriginalCertificate"||key == "AcceptanceReport"||key == "ApproachingAcceptance"
                    ||key == "SupplierContract"||key == "MaintainManual"||key == "InsuranceFile"){

//                logger.debug("Pic值001 = ${value}")
//                var list = ArrayList<AttPic>()
                if (value.toString() != "{}"){

                    try {
                        val gson = Gson()
                        val type = object : TypeToken<ArrayList<TaskPic>>() {
                        }.type
                        val attPicList = gson.fromJson<ArrayList<TaskPic>>(value.toString(),type)
//                        val attPicList = SJsonUtil.fromJson<ArrayList<TaskPic>>(value.toString())
                        if (attPicList.isNotEmpty()){
                            val listPic =ArrayList<AttPic>()
                            for (attPic in attPicList){
                                val taskPic = AttPic()
                                taskPic.key = attPic.key
                                taskPic.name = attPic.name
                                taskPic.systemId = attPic.systemId
                                taskPic.type = attPic.type
                                listPic.add(taskPic)
                            }
                            hashMap.put("value",listPic)
                            valueArr.add(hashMap)
                        }
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
//                    val attPicList = SJsonUtil.fromJson<ArrayList<AttPic>>(value.toString())


                }else{
                    hashMap.put("value",value)
                    valueArr.add(hashMap)
                }
                infos!![key] = valueArr
            }else{
                hashMap.put("value",value)
                valueArr.add(hashMap)
                infos!![key] = valueArr
            }
//            logger.debug("value值1 = ${valueArr.toJson()}")
        }
    }

    /**
     * 按信息点的一级标签，将一级标签里的信息点平铺添加
     */
    open fun addMapInfo(map: Map<String, Any?>, ignoreNull: Boolean) {
        if (map != null) {
            for (key in map.keys) {
                innerAddInfo(key, map[key], ignoreNull)
            }
        }
    }

    /**
     * 按信息点的一级标签，将一级标签里的信息点平铺添加
     */
    open fun addJSONObjectInfo(json: Any?, ignoreNull: Boolean) {
        if (json != null&&json.toString()!="null") {
            //json: JSONObject
            json as JSONObject
            val iterator = json.keys()
            while (iterator.hasNext()) {
                val key = iterator.next()
                val value = json.get(key as String)
//                innerAddInfo(key, value, ignoreNull)
                addInfo(key,value,true)
            }
        }
    }
    /**
     *
     */
    open fun removeInfo(key: String) {
        infos?.remove(key)
    }

}