/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.wanda.obj

import com.persagy.server.datacenter.models.entities.assistant.PointPosition
import com.persagy.server.datacenter.models.entities.base.BaseInfo
import com.persagy.service.models.annotations.SCascade
import io.swagger.v3.oas.annotations.media.Schema
import javax.persistence.Column
import javax.persistence.Id
import javax.persistence.Table

/**
 * 楼层信息实体类
 *
 * @author  张维新
 */
@Schema(name = "楼层信息实体类")
@Table(name = "floor")
open class Floor : BaseInfo() {

    /** 楼层ID */
    @Schema(description = "楼层ID")
    @Id
    @Column(name = "id")
    var id: String? = null

    /** 楼层名称 */
    @Schema(description = "楼层名称")
    @Column(name = "name")
    var name: String? = null

    /** 楼层本地编码 */
    @Schema(description = "楼层本地编码")
    @Column(name = "local_id")
    var localId: String? = null

    /** 楼层本地名称 */
    @Schema(description = "楼层本地名称")
    @Column(name = "local_name")
    var localName: String? = null

    /** 二维码内容 */
    @Schema(description = "二维码内容")
    @Column(name = "default_or_code")
    var defaultQRCode: String? = null

    /** 二维码图片 */
    @Schema(description = "二维码图片")
    @Column(name = "qr_code_pic")
    var qRCodePic: String? = null

    /** 项目id */
    @Schema(description = "项目id")
    @Column(name = "project_id")
    var projectId: String? = null

    /** 建筑id */
    @Schema(description = "建筑id")
    @Column(name = "building_id")
    var buildingId: String? = null

    /** BIM模型中坐标 */
    @Schema(description = "BIM模型中坐标")
    @Column(name = "bim_location")
    var bimLocation: String? = null

    /** BIM模型中编码 */
    @Schema(description = "BIM模型中编码")
    @Column(name = "bim_id")
    var bimId: String? = null

    /** BIM模型中编码 */
    @Schema(description = "BIM模型中编码")
    @Column(name = "bim_id_pre")
    var bimIdPre: String? = null

    /** 楼层顺序码 */
    @Schema(description = "楼层顺序码")
    @Column(name = "sequence_id")
    var floorSequenceId: Int? = null

    /** 楼层常驻人数 */
    @Schema(description = "楼层常驻人数")
    @Column(name = "permanent_people_num")
    var permanentPeopleNum: Int? = null

    /** 楼层编码 */
    @Schema(description = "楼层编码")
    @Column(name = "floor_identity")
    var floorIdentity: String? = null

    /** 信息点 */
    @Schema(description = "信息点")
    @Column(name = "infos")
    var infos: HashMap<String,Any?>? = null

    /** 对象类型 */
    @Schema(description = "对象类型")
    @Column(name = "object_type")
    override var objectType: String? = "floor"

    /** 建筑   table  对应关系表   idColumn 对应 本类的关系id    childIdColumn 级联对象的关系id */
    @Schema(description = "级联-楼层")
    @SCascade(table="floor", idColumn="id", childIdColumn="building_id")
    var building: Building? = null

    /** 项目 */
    @Schema(description = "级联-项目")
    @SCascade(table="floor", idColumn="id", childIdColumn="project_id")
    var project: Project? = null

    /** 贯通关系   self = true  同样的对象查询时 id可以互相调换查询   */
    @Schema(description = "级联-贯通关系")
    @SCascade(table="r_fl_through_fl", idColumn="floor_id", childIdColumn="floor_other_id", self = true)
    var floorThroughList: ArrayList<Floor>? = null

    /** 模型文件id */
    @Schema(description = "模型文件id")
    @Column(name = "model_id")
    var modelId: String? = null

    /** 轮廓线 */
    @Schema(description = "轮廓线")
    @Column(name = "outline")
    var outline: ArrayList<PointPosition>? = null

//    /** 级联-部件   table  对应关系表   idColumn 对应 本类的id    childIdColumn 级联对象的关系id ，注意 idColumn childIdColumn 非数据表中字段名，是实体类属性*/
//    @Schema(description = "级联-轮廓线")
//    @SCascade(table="floor_outline", idColumn="id", childIdColumn="id",self = false)
//    var floorOutline: FloorOutline? = null

    /** 数量 */
    @Schema(description = "数量")
    var count: Int = 0

    /** 自定义信息点 */
    @Schema(description = "自定义信息点")
    @Column(name = "properties")
    var properties: HashMap<String,Any?>? = null

}