/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.jms

import org.springframework.amqp.core.Binding
import org.springframework.amqp.core.BindingBuilder
import org.springframework.amqp.core.Queue
import org.springframework.amqp.core.TopicExchange
import org.springframework.beans.factory.annotation.Value
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.stereotype.Component

/**
 * Created by Administrator on 2018/4/10.
 * 1. yaml
 * 2. RabbitMqConfig
 * a  创建queue
 * b. 创建交换机TopicExchange
 * c. 根据绑定规则将队列绑定到相应的交换机上
 * 3. sender
 * 4. test
 * 5. receiver
 */
@Component
@Configuration
open class RabbitMqConfig {

    @Value("\${mqtopid.exchange}")
    lateinit var exchange: String
    @Value("\${mqtopid.routingKey}")
    lateinit var orgRoutingKey: String
    @Value("\${mqtopid.routingKey01}")
    lateinit var orgRoutingKey01: String
    @Value("\${mqtopid.routingKey02}")
    lateinit var orgRoutingKey02: String
    @Value("\${mqtopid.admQueue}")
    lateinit var admQueue: String
    @Value("\${mqtopid.synQueue}")
    lateinit var synQueue: String

    /**
     * 消息队列
     *
     * @return 创建队列
     */
    @Bean
    open fun admQueueMessage(): Queue {
        return Queue(admQueue, true)
    }

    /**
     * 消息队列
     *
     * @return 创建队列
     */
    @Bean
    open fun synQueueMessages(): Queue {
        return Queue(synQueue, true)
    }

    /**
     * 交换机
     *
     * @return 创建交换机
     */
    @Bean
    open fun exchange(): TopicExchange {
        return TopicExchange(exchange)
    }

    /*
      c. 根据绑定规则将队列绑定到相应的交换机上（bindingKey）--Exchange Queues

     */
//    @Bean
//    public Binding bindingExchangeMessage(@Qualifier("message") Queue queueMessage, TopicExchange exchange){
//        return BindingBuilder.bind(queueMessage).to(exchange).with("topic.message");
//    }
    @Bean
    open fun amdBinding(): Binding {
        return BindingBuilder.bind(admQueueMessage()).to(exchange()).with(orgRoutingKey)
    }

    @Bean
    open fun synBinding01(): Binding {
        return BindingBuilder.bind(synQueueMessages()).to(exchange()).with(orgRoutingKey01)
    }

    @Bean
    open fun synBinding02(): Binding {
        return BindingBuilder.bind(synQueueMessages()).to(exchange()).with(orgRoutingKey02)
    }
}