/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.jms

import com.persagy.server.Opt
import com.persagy.server.datacenter.models.jms.MessageNotice
import com.persagy.server.datacenter.models.jms.SBaseNotice
import org.slf4j.LoggerFactory
import org.springframework.stereotype.Component
import org.springframework.stereotype.Service
import java.util.*

/**
 * 消息
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/10/28 8:43
 */
@Component
@Service
class RabbitMqService: RabbitMqBaseService() {
    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(com.persagy.server.services.base.Service::class.java)
    }

//    /** 缓存对象数据服务 */
//    var synchronousMessageObjService = SObjectService(SMybatisDao(SynchronousMessage::class.java))
//
//    /**
//     * 接收消息
//     *
//     * @param
//     */
//    @RabbitListener(queues = ["queue-adm"])
//    @RabbitHandler
//    fun receiveMessage(message: String) {
//        println("我是监听queue-adm的,仅满足queue-adm的过来 ， $message")
//        val messageNotice = SJsonUtil.fromJson(message, MessageNotice::class.java)
//        if (messageNotice != null) {
//            when(messageNotice.objOrRel) {
//                "objects" -> {
//                    try {
//                        /** id 列表不可以为空 */
//                        if (!messageNotice.idList.isNullOrEmpty()) {
//                            /** 遍历 id 列表 */
//                            for (id in messageNotice.idList!!) {
//                                /** 同步消息对象 */
//                                val messageObj = SynchronousMessage()
//                                messageObj.projectId = messageNotice.projectId
//                                messageObj.objId = id
//                                messageObj.id =  UUID.randomUUID().toString().replace("-", "")
//                                messageObj.sign = 1
//                                messageObj.objectType = objType(id)
//                                messageObj.type = messageNotice.type
//                                /** 插入数据 */
//                                synchronousMessageObjService.insert(messageObj)
//                            }
//                        }
//                    } catch (e: Exception) {
//                        e.printStackTrace()
//                    }
//                }
//                "rel" -> {
//                    /** 关系消息对象 */
//                    val messageRel = SynchronousMessage()
//                    messageRel.projectId = messageNotice.projectId
//                    messageRel.id = UUID.randomUUID().toString().replace("-", "")
//                    messageRel.sign = 1
//                    messageRel.ops = messageNotice.ops
//                    messageRel.type = messageNotice.type
//                    messageRel.fromId = messageNotice.fromId
//                    messageRel.toId = messageNotice.toId
//                    synchronousMessageObjService.insert(messageRel)
//                }
//            }
//        }
//    }


    /**
     * 合并重新
     *
     * @param lastNotice 最后一条消息对象
     * @param notice     当前消息对象
     * @return 是否合并
     */
    override fun mergeNotice(lastNotice: SBaseNotice, notice: SBaseNotice): Boolean {
        // 对象消息合并
        if (notice is MessageNotice) {
            // 如果最后发送对象不是对象消息，则返回false
            val last = lastNotice as? MessageNotice ?: return false
            // 操作不同，对象不同，项目不同的通知不合并
            if (notice.ops != notice.ops || notice.type != last.type || notice.projectId != last.projectId) {
                return false
            }
            // 合并消息
            notice.idList?.let { last.idList?.addAll(it) }
            return true
        }
//        else if (notice is RelationNotice) {  // 关系消息合并
//            // TODO: 补 关系类型通知合并代码
//            return true
//        }
        return false
    }

    /**
     * 发送消息(对象消息)
     *
     * @param projectId     项目 id
     * @param type          操作符类型
     * @param objId         对象 id
     */
    fun sendObjects(projectId: String, type: String, objId: String){
       logger.debug("发送消息= ***********************************************************************************")
        /** 生成 id */
        val id = UUID.randomUUID().toString().replace("-", "")
        /** 消息对象 */
        val messageNotice = MessageNotice(projectId, "objects", type, null, id,
            hashSetOf(objId), null, null)
        /** 发送消息 */
        sendMessage(Opt.exchange!!,Opt.routingKey!!,messageNotice)
    }

    /**
     * 发送消息(关系)
     *
     * @param projectId     项目 id
     * @param type          关系类型
     * @param fromId        主 id
     * @param toId          从 id
     */
    fun sendRel(projectId: String,type: String,ops: String,fromId: String,toId: String){
        try {
            /** 生成 id */
            val id = UUID.randomUUID().toString().replace("-", "")
            /** 消息对象 */
            val messageNotice = MessageNotice(projectId, "rel", type, ops, id,
                null, fromId, toId)
            /** 发送消息 */
            sendMessage(Opt.exchange!!,Opt.routingKey!!,messageNotice)
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }


}