/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.jms

import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.jms.MessageNotice
import com.persagy.server.datacenter.models.jms.SynchronousMessage
import com.persagy.server.datacenter.models.jms.SynchronousMessageRel
import com.persagy.service.SObjectService
import com.persagy.service.utils.SJsonUtil
import org.springframework.amqp.rabbit.annotation.RabbitHandler
import org.springframework.amqp.rabbit.annotation.RabbitListener
import org.springframework.stereotype.Component
import org.springframework.stereotype.Service
import java.util.*

/**
 * 接收消息
 *
 * @author wx
 */
@Component
@Service
class TopicReceiver {

    /** 数据中心到数据中台关系 */
    var synchronousMessageRelService = SObjectService(SMybatisDao(SynchronousMessageRel::class.java))
    /** 缓存对象数据服务 - 数据中心到数据中台  */
    var synchronousMessageObjService = SObjectService(SMybatisDao(SynchronousMessage::class.java))

//    @RabbitListener(queues = ["queue-center-adm"])
//    @RabbitHandler
    fun receiveMessage1(str: String) {
        println("我是监听queue-adm的,仅满足queue-adm的过来 ， $str")
    }

    /**
     * 接收数据中心消息
     *
     * @param
     */
    @RabbitListener(queues = ["queue-center-adm"])
    @RabbitHandler
    fun receiveMessage(message: String) {
        try {
            println("接收数据中心消息 ， $message")
            val messageNotice = SJsonUtil.fromJson(message, MessageNotice::class.java)
            if (messageNotice != null) {
                when(messageNotice.objOrRel) {
                    "objects" -> {
                        try {
                            /** id 列表不可以为空 */
                            if (!messageNotice.idList.isNullOrEmpty()) {
                                /** 遍历 id 列表 */
                                for (id in messageNotice.idList!!) {
                                    /** 同步消息对象 */
                                    val messageObj = SynchronousMessage()
                                    messageObj.projectId = messageNotice.projectId
                                    messageObj.objId = id
                                    messageObj.id =  UUID.randomUUID().toString().replace("-", "")
                                    messageObj.sign = 1
                                    if (messageNotice.type != "equip") {
                                        messageObj.objectType = objType(id)
                                    }else{
                                        messageObj.objectType = "equip"
                                    }
                                    messageObj.type = messageNotice.type
                                    /** 插入数据 */
                                    synchronousMessageObjService.insert(messageObj)
                                }
                            }
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                    }
                    "rel" -> {
                        /** 关系消息对象 */
                        val messageRel = SynchronousMessageRel()
                        messageRel.projectId = messageNotice.projectId
                        messageRel.id = UUID.randomUUID().toString().replace("-", "")
                        messageRel.sign = 1
                        messageRel.ops = messageNotice.ops
                        messageRel.type = messageNotice.type
                        messageRel.fromId = messageNotice.fromId
                        messageRel.toId = messageNotice.toId
                        synchronousMessageRelService.insert(messageRel)
                    }
                }
                /** 处理消息 */
//                messageProcessing.syncDataCenter()
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    /**
     * 根据对象 id 转换对象类型
     *
     * @param id    对象 id
     * @return 对象类型
     */
    private fun objType(id: String): String {
        when {
            id.startsWith("Eq") -> {
                return "equipment"
            }
            id.startsWith("Fl") -> {
                return "floor"
            }
            id.startsWith("Bd") -> {
                return "building"
            }
            id.startsWith("Sp") -> {
                return "space"
            }
            id.startsWith("Sy") -> {
                return "system"
            }
            id.startsWith("Sh") -> {
                return "shaft"
            }
            id.startsWith("Ec") -> {
                return "component"
            }
            id.startsWith("20") -> {
                return "equip"
            }

            else -> return "object"
        }
    }

}