/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.mappers

import com.persagy.server.datacenter.utils.JsonTypeHandler
import com.persagy.server.datacenter.models.entities.Connector
import org.apache.ibatis.annotations.Mapper
import org.apache.ibatis.annotations.Result
import org.apache.ibatis.annotations.Results
import org.apache.ibatis.annotations.Select
import org.apache.ibatis.type.JdbcType
import org.springframework.stereotype.Repository

/**
 * 关系计算-管网
 *
 * @author 邓婷婷
 */
@Mapper
@Repository
interface ICalcSpecialMapper {

    /**
     * 管网系统设备分块
     *@param projectId 项目id
     *@param systemName 需要计算的系统名称
     *@param domain 需要计算的管道系统类型，枚举（水管或风管）：DomainPiping，DomainHvac
     */
    @Select(" SELECT * from public.sys_block(#{projectId},#{buildingId},#{systemName},#{domain}) ")
    fun sysBolck(projectId: String, buildingId: String?, systemName: String,domain: String): Boolean

    /**
     * 管网系统确定流向
     *@param projectId 项目id
     *@param blockId 需要计算的块id
     *@param systemName 需要计算的系统名称
     *@param domain 需要计算的管道系统类型，枚举（水管或风管）：DomainPiping，DomainHvac
     */
    @Select(" SELECT * from public.sys_direction(#{projectId},#{buildingId},#{blockId},#{systemName},#{domain},#{isSource}) ")
    fun sysDirection(projectId: String, buildingId: String?, blockId: String, systemName: String,domain: String,isSource: Boolean): Boolean


    /**
     * @Description: 查询连接件并按指定顺序排序
     * @Param: modelId 模型id
     * @Return: List<Connector>
     */
    @Select(" SELECT * from revit.connector where model_id = #{modelId} order by case belong_type  " +
            "when 'Equipment' then 1 " +
            "when 'EquipPart' then 2 " +
            "when 'JoinObject' then 3 " +
            "when 'Other' then 4 " +
            "when 'Pipe' then 5 " +
            "when 'FlexPipe' then 6 " +
            "when 'Duct' then 7 " +
            "when 'FlexDuct' then 8 " +
            "end asc")
    @Results(*[Result(property = "id", column = "id"),
        Result(property = "modelId", column = "model_id"),
        Result(property = "revitId", column = "revit_id"),
        Result(property = "sourceId", column = "source_id"),
        Result(property = "elementType", column = "type"),
        Result(property = "belongType", column = "belong_type"),
        Result(property = "isConnected", column = "connected"),
        Result(property = "connectedIds", column = "connected_ids",typeHandler = JsonTypeHandler:: class),
        Result(property = "mepSystemType", column = "mep_system_type"),
        Result(property = "connectIdJson", column = "connect_id_json",typeHandler = JsonTypeHandler:: class),
        Result(property = "belongBimId", column = "belong_bimid")])
    fun connectorOrder(modelId:String): ArrayList<Connector>







} // Interface ICalcSpecialMapper