/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.mappers

import com.persagy.server.datacenter.models.entities.graphtype.block.ConnectedBlockSource
import com.persagy.server.datacenter.models.entities.graphtype.block.EquipIdUUIDMap
import com.persagy.server.datacenter.models.entities.task.SpaceType
import org.apache.ibatis.annotations.Mapper
import org.apache.ibatis.annotations.Param
import org.apache.ibatis.annotations.Select
import org.apache.ibatis.annotations.SelectProvider
import org.springframework.stereotype.Repository

/**
 *
 */
@Mapper
@Repository
interface MepSystemMapper {

    /**
     * 根据BIMID获取设备
     *@param projectId 项目id
     *@param tables 参与计算的表的全名称, 带schema名, 以英文逗号隔开
     */
    @SelectProvider(type = RelationProvider::class , method = "getEquipByBimId")
    fun getEquipByBimId(@Param("projectId") projectId: String, @Param("buildingId") buildingId : String?, @Param("mepSystemType") mepSystemType: ArrayList<String>,
                      @Param("bimId") bimId: ArrayList<String>): ArrayList<ConnectedBlockSource>

    /**
     * 获取模型里的设备uuid到数据中心设备的映射
     *@param projectId 项目id
     *@param equipIds 设备uuid集合
     */
    @SelectProvider(type = RelationProvider::class , method = "getEquipIdUUIDMap")
    fun getDCEquipModelEquipMap(@Param("projectId") projectId: String, @Param("equipIds") equipIds : Set<String>): ArrayList<EquipIdUUIDMap>

    /**
     * 获取模型里的部件uuid到数据中心部件的映射
     *@param projectId 项目id
     *@param equipPartIds 部件uuid集合
     */
    @SelectProvider(type = RelationProvider::class , method = "getEquipPartIdUUIDMap")
    fun getDCEquipPartModelEquipPartMap(@Param("projectId") projectId: String, @Param("equipPartIds") equipPartIds : Set<String>): ArrayList<EquipIdUUIDMap>


    @Select("SELECT s.object_type as type,d.name  FROM zone_space_base s LEFT JOIN dict.dict_space_project d ON s.object_type = d.id WHERE s.project_id = #{projectId} AND d.project_id = #{projectId}  GROUP BY object_type, d.name")
    fun getObjectType(projectId: String):ArrayList<SpaceType>


} // Interface MepSystemMapper