/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.assistant

import com.persagy.database.SFilter
import com.persagy.server.Opt
import com.persagy.server.datacenter.datacenter.dao.mappers.SchemeMapper
import com.persagy.server.jms.RabbitMqService
import com.persagy.server.services.objects.FloorService
import com.persagy.service.utils.SSpringContextUtil
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service

/**
 * 数据中心 方案导入服务
 *
 * @author 张维新
 */
@Service
open class SchemeService {

    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(SchemeService::class.java)

    } // Companion object

    /** RabbitMQ消息对象 */
    val rabbitMqService by lazy {
        SSpringContextUtil.getBean(RabbitMqService::class.java) as RabbitMqService
    }

    /** 实例化 mapper 对象 */
    @Autowired
    lateinit var mapper: SchemeMapper

    /**
     * 导入方案处理
     *
     * @param      projectId        项目id
     * @param      schemeId         方案id
     */
    fun setScheme(projectId: String, schemeId: String):Boolean{
        try {
            mapper.setSchemeDict(projectId,schemeId)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return true
    } // Function setScheme()

    /**
     * 导入自定义信息点
     *
     * @param      projectId        项目id
     * @param      schemeId         方案id
     */
    fun setSchemeCustomDict(projectId: String, schemeId: String):Boolean{
        try {
            mapper.setSchemecustomDict(projectId,schemeId)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return true
    } // Function setSchemeCustomDict()

    /**
     * 导入自定义信息点
     *
     * @param      projectId        项目id
     * @param      schemeId         方案id
     */
    fun setSchemeCustomDictFalse(projectId: String, schemeId: String):Boolean{
        try {
            mapper.setSchemecustomDictFalse(projectId,schemeId)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return true
    } // Function setSchemeCustomDict()

    /**
     * 导入原始字典信息
     *
     * @param      projectId        项目id
     */
    fun setControlDict(projectId: String):Boolean{
        try {
            mapper.setControlDict(projectId)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return true
    } // Function setControlDict()


    /**
     * 处理自定义自定数据
     */
    fun setDictCustom():Boolean{
        try {
            val dict = mapper.setDict()
            return dict
        } catch (e: Exception) {
            e.printStackTrace()
            return false
        }
    }

    /**
     * 修改设备和元空间的建筑楼层
     */
    fun rupdateEquipiSpace(projectId: String,oldBimId: String?,bimId: String,floorId: String,floorMap: String,modelId: String,attributeMap: HashMap<String,Any>?): Boolean{
       return  try {
           var oldBimIdNew = ""
           if (!oldBimId.isNullOrEmpty()){
               oldBimIdNew = oldBimId
           }
            val rUpDateEquipAndSapce = mapper.rUpDateEquipAndSapce(projectId, oldBimIdNew, bimId, floorId)

           val floorEntity = FloorService.select(SFilter.eq("id", floorId)).entity()
           if (floorEntity!=null){

               if (!floorEntity.infos.isNullOrEmpty()){
                   val containsKey = floorEntity.infos!!.containsKey(Opt.FLOORMAP)

                   if (containsKey){
                       floorEntity.infos!![Opt.FLOORMAP] = floorMap
                   }else{
                       floorEntity.infos!![Opt.FLOORMAP] = floorMap
                   }
               }else{
                   val map = HashMap<String,Any?>()
                   map[Opt.FLOORMAP] = floorMap
                   floorEntity.infos = map
               }
               floorEntity.modelId = modelId
               if (!floorEntity.bimIdPre.isNullOrEmpty()) {
                   floorEntity.bimIdPre = bimId
               } else {
                   floorEntity.bimIdPre = bimId
               }
               /** 处理楼层标高 */
               if (!attributeMap.isNullOrEmpty()&&attributeMap.size>0){
                   val containsKey = attributeMap.containsKey(Opt.FLOORELEVATION)
                   if (containsKey){
                       try {
                           val toDouble = attributeMap[Opt.FLOORELEVATION]!!.toString().toDoubleOrNull()
                           if (toDouble!=null){
                               floorEntity.infos!![Opt.FLOORELEVATION] = toDouble/1000
                           }
                       } catch (e: Exception) {
                           e.printStackTrace()
                       }
                   }
               }
               var list = ArrayList<SFilter>()
               list.add(SFilter.eq("id",floorId))

               val update = FloorService.update(floorEntity, null, list)
               if (update){
                   rabbitMqService.sendObjects(projectId!!, Opt.UPDATE,floorEntity.id!!)
                   rabbitMqService.sendObjects(projectId!!, "equip",floorEntity.id!!)
               }
           }
           return rUpDateEquipAndSapce
        } catch (e: Exception) {
            e.printStackTrace()
            false
        }
    } // Fun

    fun dictInfo(projectId: String, groupId: String,  code:String){
        mapper.dictInfo(projectId,groupId,code)
    } // Fun

    fun dictInfoProperty(projectId: String, groupId: String,  code:String,propertyCode: String){
        mapper.dictInfoProperty(projectId,groupId,code,propertyCode)
    }

} // Class SchemeService