/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.services.assistant

import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.wanda.obj.Equipment
import com.persagy.server.datacenter.models.entities.rel.RSpinBd
import com.persagy.server.datacenter.models.entities.rel.RSpinFl
import com.persagy.server.datacenter.models.requests.UpDataFloorAndBuildingRequests
import com.persagy.server.datacenter.wanda.obj.ZoneSpace
import com.persagy.server.services.base.Service
import com.persagy.server.services.objects.FloorService
import com.persagy.server.datacenter.services.objects.ZoneSpaceBaseService
import com.persagy.server.services.rel.RSpInBdService
import com.persagy.server.services.rel.RSpInFlService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import org.json.JSONObject

/**
 * 修改关系 楼层或建下的对象
 *
 * @author 张维新
 */
object UpDataFloorAndBuildingServuce : Service<Equipment>(SMybatisDao(Equipment::class.java)) {


    /**
     * 修改空间对象的楼层建筑信息
     *
     * @param   spaceId     业务空间id
     * @param   id          楼层或建筑id
     * @param   type        业务空间类型
     */
    fun upDataFloorORBuilding(upDataFloorAndBuildingRequestsList: ArrayList<UpDataFloorAndBuildingRequests>): SBaseResponse {
        try {
            val projectId = SPageContext.getHeader("projectId")

            if (upDataFloorAndBuildingRequestsList.isNullOrEmpty()) {
                val sBaseResponse = SBaseResponse(SResponseType.failure)
                sBaseResponse.message = "不可以传入空数据"
                return sBaseResponse
            }
            for (upDataFloorAndBuildingRequests in upDataFloorAndBuildingRequestsList) {

                val spaceId = upDataFloorAndBuildingRequests.spaceId
                val id = upDataFloorAndBuildingRequests.id
                val type = upDataFloorAndBuildingRequests.type

                try {
                    if (!id.isNullOrEmpty()) {
                        if (id.startsWith("Fl")) {
                            val floor =
                                FloorService.select(SFilter.eq("projectId", projectId!!), SFilter.eq("FloorID", id))
                                    .entity()

                            if (floor != null) {
                                /** 清除空间和建筑楼层的的数据 */
                                RSpInBdService.delete(
                                    SFilter.eq("projectId", projectId),
                                    SFilter.eq("spaceId", spaceId!!)
                                )
                                RSpInFlService.delete(
                                    SFilter.eq("projectId", projectId),
                                    SFilter.eq("spaceId", spaceId)
                                )
                                val rSpinFl = RSpinFl()
                                rSpinFl.floorId = floor.id
                                rSpinFl.projectId = projectId
                                rSpinFl.spaceId = spaceId
                                rSpinFl.objectType = type
                                rSpinFl.sign = 1
                                RSpInFlService.replace(rSpinFl)

                                val rSpinBd = RSpinBd()
                                rSpinBd.objectType = type
                                rSpinBd.buildingId = floor.buildingId
                                rSpinBd.spaceId = spaceId
                                rSpinBd.sign = 1
                                rSpinBd.projectId = projectId
                                RSpInBdService.replace(rSpinBd)
                            }

                        } else if (id.startsWith("Bd")) {
                            /** 清除空间和建筑楼层的的数据 */
                            RSpInBdService.delete(
                                SFilter.eq("projectId", projectId!!),
                                SFilter.eq("spaceId", spaceId!!)
                            )
                            RSpInFlService.delete(
                                SFilter.eq("projectId", projectId),
                                SFilter.eq("spaceId", spaceId)
                            )
                            val rSpinBd = RSpinBd()
                            rSpinBd.objectType = type
                            rSpinBd.buildingId = id
                            rSpinBd.spaceId = spaceId
                            rSpinBd.sign = 1
                            rSpinBd.projectId = projectId
                            RSpInBdService.replace(rSpinBd)
                        }
                    } else {
                        /** 清除空间和建筑楼层的的数据 */
                        RSpInBdService.delete(SFilter.eq("projectId", projectId!!), SFilter.eq("spaceId", spaceId!!))
                        RSpInFlService.delete(SFilter.eq("projectId", projectId), SFilter.eq("spaceId", spaceId))
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }


                val zonePowerSupply = ZoneSpace()
                zonePowerSupply.id = spaceId
                if (!id.isNullOrEmpty()) {
                    if (id.startsWith("Fl")) {
                        zonePowerSupply.floorId = id
                        val floor = FloorService.select(SFilter.eq("projectId", projectId!!), SFilter.eq("FloorID", id))
                            .entity()
                        zonePowerSupply.buildingId = floor?.buildingId
                        val list = ArrayList<String>()
                        list.add("FloorId")
                        list.add("BuildingId")
                        ZoneSpaceBaseService.update(zonePowerSupply, list)
                    } else if (id.startsWith("Bd")) {
                        val list = ArrayList<String>()
                        list.add("FloorId")
                        list.add("BuildingId")
                        zonePowerSupply.buildingId = id
                        zonePowerSupply.floorId = null
                        ZoneSpaceBaseService.update(zonePowerSupply, list)
                    }
                } else {
                    /** 清空建筑楼层 */
                    val list = ArrayList<String>()
                    list.add("FloorId")
                    list.add("BuildingId")
                    zonePowerSupply.buildingId = null
                    zonePowerSupply.floorId = null
                    ZoneSpaceBaseService.update(zonePowerSupply, list)
                }
                /** 发送消息 */
                sendMessages(projectId!!, spaceId!!, id)
            }
            return SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure)
        }

    } // Function upDataFloorORBuilding

    /**
     * 发送消息
     *
     * @param   projectId   项目id
     * @param   spaceId     业务空间id
     * @param   id          楼层或建筑id
     */
    fun sendMessages(projectId: String, spaceId: String, id: String?) {
        try {
            val jsonObject = JSONObject()
            jsonObject.put("type", "20")
            jsonObject.put("projectId", projectId)
            jsonObject.put("id", spaceId)
            jsonObject.put("linkId", id)
            /** 发送消息 */
            messageHandler.sendMessage(jsonObject.toString())
        } catch (e: Exception) {
            e.printStackTrace()
        }
    } // Function sendMessages

} // Object UpDataFloorAndBuildingServuce