/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.base

import com.persagy.database.SAbstractDao
import com.persagy.database.SFilter
import com.persagy.database.extensions.keyValue
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.models.entities.base.BaseInfo
import com.persagy.server.datacenter.datacenter.models.entities.dictnew.DefClass
import com.persagy.server.datacenter.models.entities.logs.DataCenterLogs
import com.persagy.server.jms.RabbitMqService
import com.persagy.server.services.assistant.SchemeService
import com.persagy.server.services.log.DataCenterLogsService
import com.persagy.server.services.rel.RSpInBdService
import com.persagy.server.services.rel.RSpInFlService
import com.persagy.server.synchronize.MessageHandler
import com.persagy.service.SObjectService
import com.persagy.service.utils.SSpringContextUtil
import org.slf4j.LoggerFactory


/**
 * 服务基类
 *
 * @author 张维新
 */
open class Service<ENTITY : BaseInfo>(daoClass: SAbstractDao<ENTITY>) : SObjectService<ENTITY>(daoClass) {
    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(Service::class.java)

        /** MQ消息对象 */
        val messageHandler by lazy {
            SSpringContextUtil.getBean(MessageHandler::class.java) as MessageHandler
        }

        /** 自定义sql */
        val schemeService by lazy {
            SSpringContextUtil.getBean(SchemeService::class.java) as SchemeService
        }

        /** RabbitMQ消息对象 */
        val rabbitMqService by lazy {
            SSpringContextUtil.getBean(RabbitMqService::class.java) as RabbitMqService
        }
    } // Companion object
    /** 日志 */
    var logService = SObjectService(SMybatisDao(DataCenterLogs::class.java))

    /** 平台集团对象类型 */
    val defClassService = SObjectService(SMybatisDao(DefClass::class.java))


    /**
     * 创建成功回调函数
     *
     * @param   entityList          创建成功对象
     */
    override fun onCreateSuccess(entityList: ArrayList<ENTITY>) {
        entityList.forEach {
            try {
                if (!it.keyValue("id").toString().startsWith("Si")&&!it.keyValue("id").toString().startsWith("Pe")){
                    rabbitMqService.sendObjects(Opt.projectId!!, Opt.CREATE,it.keyValue("id").toString())
                }

            } catch (e: Exception) {
                e.printStackTrace()
            }
            try {
                /** 日志 */
                DataCenterLogsService.addLogs(Opt.projectId!!,"create",it.objectType!!,
                    it.keyValue("id").toString(),it,Opt.account,Opt.coming)
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }

        return
    }

    /**
     * 删除成功回调函数
     *
     * @param   entityList      删除成功对象列表
     */
    override fun onDeleteSuccess(entityList: ArrayList<ENTITY>) {
        entityList.forEach {
            try {
                if (it.keyValue("id").toString().startsWith("Sp")){
                    /** 清除空间和建筑楼层的的数据 */
                    RSpInBdService.delete(SFilter.eq("projectId",Opt.projectId!!), SFilter.eq("spaceId",it.keyValue("id").toString()))
                    RSpInFlService.delete(SFilter.eq("projectId",Opt.projectId!!), SFilter.eq("spaceId",it.keyValue("id").toString()))
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }

            try {
                if (!it.keyValue("id").toString().startsWith("Si")&&!it.keyValue("id").toString().startsWith("Pe")) {
                    rabbitMqService.sendObjects(Opt.projectId!!, Opt.DELETE, it.keyValue("id").toString())
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            try {
                DataCenterLogsService.addLogs(Opt.projectId!!,"delete",it.objectType!!,
                    it.keyValue("id").toString(),it,Opt.account,Opt.coming)
            } catch (e: Exception) {
            }
        }
        return
    }


    override fun onUpdateBefore(entity: ENTITY): Boolean {

        try {
            /** 日志 */
            DataCenterLogsService.addLogs(Opt.projectId!!,"update-before",entity.objectType!!,
                entity.keyValue("id").toString(),entity,Opt.account,Opt.coming)
        } catch (e: Exception) {
            e.printStackTrace()
    }

        return super.onUpdateBefore(entity)
    }

    /**
     * 更新成功能回调函数
     *
     * @param   entityList      更新成功对象列表
     */
    override fun onUpdateSuccess(entityList: ArrayList<ENTITY>) {
        try {
            for (entity in entityList) {
                if (!entity.keyValue("id").toString().startsWith("Si")&&!entity.keyValue("id").toString().startsWith("Pe")) {
                    rabbitMqService.sendObjects(Opt.projectId!!, Opt.UPDATE, entity.keyValue("id").toString())
                }
                try {
                    /** 日志 */
                    DataCenterLogsService.addLogs(Opt.projectId!!,"update-after",entity.objectType!!,
                        entity.keyValue("id").toString(),entity,Opt.account,Opt.coming)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return
    }

}
