/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.services.objects

import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.wanda.obj.Building
import com.persagy.server.services.assistant.ObjInfoService
import com.persagy.server.services.base.Service
import com.persagy.server.services.log.DataCenterLogsService
import com.persagy.server.services.objects.FloorService
import com.persagy.server.utils.IdUtils
import com.persagy.server.utils.ShaftNameUtil
import com.persagy.service.SPageContext
import java.util.*

/**
 * 建筑信息服务
 *
 * @author 张维新
 */
object BuildingService : Service<Building>(SMybatisDao(Building::class.java)) {

    /**
     * 创建对象前回调函数
     *
     * @param   building      创建前的回调函数
     * @return  返回true,可以创建对象
     */
    override fun onCreateBefore(building: Building): Boolean  {
        val projectId = SPageContext.getHeader("projectId")
        building.projectId = projectId
        if (!building.id.isNullOrEmpty()) {
            val isWith = building.id!!.startsWith("Bd")
            if (!isWith) {
                building.id = IdUtils.buildIdCreate(projectId!!)
            }
        } else {
            building.id = IdUtils.buildIdCreate(projectId!!)
        }

        if (building.name.isNullOrEmpty()){
            val buildingName = ShaftNameUtil.shaftName("建筑-")
            building.name = buildingName
        }
        return true
    } // Function onCreateBefore()


    /**
     * 删除之前
     */
    override fun onDeleteBefore(entity: Building): Boolean {
        try {
            val projectId = SPageContext.getHeader("projectId")
            /** 来源 */
            val coming = SPageContext.getHeader("coming")
            /** 账号 */
            val account = SPageContext.getHeader("account")
            entity.projectId = projectId

            val exec = FloorService.select(SFilter.eq("buildingId", entity.id!!)).exec()

            /** 日志 */
            DataCenterLogsService.addLogs(projectId!!,"delete","building",
                entity.id!!,entity,account,coming)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return super.onDeleteBefore(entity)
    } // Function onDeleteBefore()

    /**
     * @Description:创建之后
     * @Param:
     * @Return:
     */
    override fun onCreateSuccess(entityList: ArrayList<Building>) {
        super.onCreateSuccess(entityList)
        try {
            /** 来源 */
            val coming = SPageContext.getHeader("coming")
            /** 账号 */
            val account = SPageContext.getHeader("account")
            for (building in entityList) {
                try {
                    ObjInfoService.createInfo(building,true,"Bd","Building")
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                try {
                    rabbitMqService.sendObjects(building.projectId!!, Opt.CREATE,building.id!!)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                try {
                    /** 日志 */
                    DataCenterLogsService.addLogs(building.projectId!!,"create","building",
                        building.id!!,building,account,coming)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    /**
     * @Description: 更新之后
     * @Param: entityList
     */
    override fun onUpdateSuccess(entityList: ArrayList<Building>) {
        super.onUpdateSuccess(entityList)
        try {
            /** 来源 */
            val coming = SPageContext.getHeader("coming")
            /** 账号 */
            val account = SPageContext.getHeader("account")
            for (building in entityList) {
                try {
                    building.projectId = Opt.projectId
                    ObjInfoService.createInfo(building,false,"Bd","Building")
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                try {
                    rabbitMqService.sendObjects(building.projectId!!, Opt.UPDATE,building.id!!)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                try {
                    /** 日志 */
                    DataCenterLogsService.addLogs(building.projectId!!,"update","building",
                        building.id!!,building,account,coming)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }


    /**
     * @Description: 删除之后
     * @Param: entityList
     */
    override fun onDeleteSuccess(entityList: ArrayList<Building>) {
        super.onDeleteSuccess(entityList)
        try {
            val projectId = SPageContext.getHeader("projectId")
            for (building in entityList) {
                ObjInfoService.delete(SFilter.eq("projectId",projectId!!), SFilter.eq("objId",building.id!!), SFilter.eq("objType","Bd"))
                rabbitMqService.sendObjects(building.projectId!!, Opt.DELETE,building.id!!)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }


} // Object BuildingService
