/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.services.objects

import com.persagy.server.utils.IdUtils
import com.persagy.server.utils.ShaftNameUtil
import com.persagy.database.SFilter
import com.persagy.database.SQueryBuilder
import com.persagy.database.extensions.keyValue
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.wanda.obj.Equipment
import com.persagy.server.services.assistant.ObjInfoService
import com.persagy.server.services.base.Service
import com.persagy.server.services.log.DataCenterLogsService

/**
 * 部件信息服务
 *
 * @author 张维新
 */
object ComponentService : Service<Equipment>(SMybatisDao(Equipment::class.java)) {

    /**
     * 创建对象前回调函数
     *
     * @param   component      创建前的回调函数
     */
    override fun onCreateBefore(component: Equipment): Boolean  {

        component.projectId = Opt.projectId
        if (!component.id.isNullOrEmpty()) {
            val isStartsWith = component.id.toString().startsWith("Ec")
            if (!isStartsWith) {
                /** 部件id */
                component.id = IdUtils.equipmentComponentIdCreate(Opt.projectId!!)
            }
        } else {
            /** 部件id */
            component.id = IdUtils.equipmentComponentIdCreate(Opt.projectId!!)
        }
        /** 项目id */
        component.projectId = Opt.projectId
        /** 部件名字 */
        if (component.name.isNullOrEmpty()) {
            component.name = ShaftNameUtil.shaftName("部件-")
        }
        component.objectType = "component"
        return true
    }

    override fun onDeleteBefore(cquipment: Equipment): Boolean {
        try {
            cquipment.objectType = "component"
            val  entity = this.select(SFilter.eq("id", cquipment.id!!)).entity()
            /** 日志 */
            DataCenterLogsService.addLogs(Opt.projectId!!,"delete",cquipment.objectType!!,
                entity!!.id!!,entity,Opt.account,Opt.coming)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return super.onDeleteBefore(cquipment)
    }

    override fun onUpdateBefore(entity: Equipment): Boolean {
        entity.objectType = "component"
        return super.onUpdateBefore(entity)
    }

    /**
     * @Description:添加成功之后
     */
    override fun onCreateSuccess(entityList: ArrayList<Equipment>) {
        super.onCreateSuccess(entityList)
        try {

            for (equipment in entityList) {
                try {
                    rabbitMqService.sendObjects(Opt.projectId!!, Opt.CREATE,equipment.id!!)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                try {
                    /** 日志 */
                    DataCenterLogsService.addLogs(Opt.projectId!!,"create",equipment.objectType!!,
                        equipment.keyValue("id").toString(),equipment,Opt.account,Opt.coming)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                ObjInfoService.createInfo(equipment,true,"Ec",equipment.classCode!!)

            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    /**
     * @Description:更新成功后
     */
    override fun onUpdateSuccess(entityList: ArrayList<Equipment>) {
        super.onUpdateSuccess(entityList)
        for (equipment in entityList) {
            try {
                rabbitMqService.sendObjects(Opt.projectId!!, Opt.UPDATE,equipment.id!!)
            } catch (e: Exception) {
                e.printStackTrace()
            }
            try {
                /** 日志 */
                DataCenterLogsService.addLogs(Opt.projectId!!,"update",equipment.objectType!!,
                    equipment.keyValue("id").toString(),equipment,Opt.account,Opt.coming)
            } catch (e: Exception) {
                e.printStackTrace()
            }
            ObjInfoService.createInfo(equipment,false,"Ec",equipment.classCode!!)
        }
    }

    /**
     * @Description:删除成功之后
     */
    override fun onDeleteSuccess(entityList: ArrayList<Equipment>) {
        super.onDeleteSuccess(entityList)
        try {
            for (equipment in entityList) {
                try {
                    rabbitMqService.sendObjects(Opt.projectId!!, Opt.DELETE,equipment.id!!)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                try {
                    /** 日志 */
                    DataCenterLogsService.addLogs(Opt.projectId!!,"delete",equipment.objectType!!,
                        equipment.keyValue("id").toString(),equipment,Opt.account,Opt.coming)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                ObjInfoService.delete(SFilter.eq("projectId",Opt.projectId!!), SFilter.eq("objId",equipment.id!!), SFilter.eq("objType","Ec"))
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

} // Object ComponentService