/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.objects

import cn.sagacloud.server.scanbuilding.User
import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.assistant.PointImage
import com.persagy.server.datacenter.models.entities.objects.LocationPoint
import com.persagy.server.scanbuilding.UserLog
import com.persagy.server.datacenter.services.base.RService
import com.persagy.server.utils.IdUtils
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext
import org.slf4j.LoggerFactory

/**
 * 点位标签服务
 *
 * @author 张维新
 */
object LocationPointServices : RService<LocationPoint>(SMybatisDao(LocationPoint::class.java)) {

    /** 日志 */
    private val logger = LoggerFactory.getLogger(LocationPointServices::class.java)
    /** 点位图片库 */
    var pointImageService = SObjectService(SMybatisDao(PointImage::class.java))
    /** 扫楼日志 */
    var userLogService = SObjectService(SMybatisDao(UserLog::class.java))
    /** 用户 */
    var userService = SObjectService(SMybatisDao(User::class.java))

    /**
     * 创建之前的操作
     */
    override fun onCreateBefore(entity: LocationPoint): Boolean {
        logger.debug("创建的点位=${entity.toJson()}")
        val projectId = SPageContext.getHeader("projectId")
        if (entity.id.isNullOrEmpty()) {
            entity.id = IdUtils.ptIdCreate()
        }
        if (entity.imageList != null && entity.imageList!!.isNotEmpty()) {
            for (image in entity.imageList!!) {
                try {
                    image.objectId = entity.id
                    val replace = pointImageService.replace(image)
                    if (replace){
                        logger.debug("点位图片存储成功*****************************")
                    }else{
                        logger.debug("点位图片存储失败*****************************")
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        }
        entity.projectId = projectId
        return super.onCreateBefore(entity)
    } // Function onCreateBefore()

    override fun onCreateSuccess(entityList: ArrayList<LocationPoint>) {
        super.onCreateSuccess(entityList)

        try {
            val projectId = SPageContext.getHeader("projectId")
            val comming = SPageContext.getHeader("coming")
            val account = SPageContext.getHeader("account")
            for (entity in entityList){
                try {
                    if (!comming.isNullOrEmpty()){
                        if (comming == "app"){
                            val user = userService.select(SFilter.eq("projectId", projectId!!), SFilter.eq("phone", account!!)).entity()
                            val userLog = UserLog()
                            userLog.comming = "app"
                            userLog.action = "创建点位标签"
                            userLog.id = IdUtils.uuidCreate()
                            userLog.note = "项目ID: "+projectId+" 用户ID："+user!!.id+" 手机号："+ account+" 请求来源："+comming+ " "+entity.toJson()
                            userLog.phone = account
                            userLog.projectId = projectId
                            userLog.userId = user.id
                            userLog.name = user.name
                            userLog.objectId = entity.id
                            userLogService.insert(userLog)
                        }
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }

    } // Fun

    /**
     * 更新之前的操作
     */
    override fun onUpdateBefore(entity: LocationPoint): Boolean {
        if (entity.id.isNullOrEmpty()) {
            return false
        }
        if (entity.imageList != null && entity.imageList!!.isNotEmpty()) {
            for (image in entity.imageList!!) {
                try {
                    image.objectId = entity.id
                    if (image.id.isNullOrEmpty()) {
                        image.id = IdUtils.uuidCreate()
                    }
                    pointImageService.replace(image)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        }
        return super.onUpdateBefore(entity)
    } // Function onCreateBefore()

    override fun onUpdateSuccess(entityList: ArrayList<LocationPoint>) {
        super.onUpdateSuccess(entityList)
        try {
            val projectId = SPageContext.getHeader("projectId")
            val comming = SPageContext.getHeader("coming")
            val account = SPageContext.getHeader("account")
            for (entity in entityList){
                try {
                    if (!comming.isNullOrEmpty()){
                        if (comming == "app"){
                            val user = userService.select(SFilter.eq("projectId", projectId!!), SFilter.eq("phone", account!!)).entity()
                            val userLog = UserLog()
                            userLog.comming = "app"
                            userLog.action = "更新点位标签"
                            userLog.id = IdUtils.uuidCreate()
                            userLog.note = "项目ID: "+projectId+" 用户ID："+user!!.id+" 手机号："+ account+" 请求来源："+comming+ " "+entity.toJson()
                            userLog.phone = account
                            userLog.projectId = projectId
                            userLog.userId = user.id
                            userLog.name = user.name
                            userLog.objectId = entity.id
                            userLogService.insert(userLog)
                        }
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

} // Object LocationPointServices