/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.services.objects

import cn.sagacloud.server.scanbuilding.User
import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.database.SQueryBuilder
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.datacenter.models.entities.objects.Property
import com.persagy.server.datacenter.models.entities.logs.DataCenterLogs
import com.persagy.server.datacenter.wanda.obj.Equipment
import com.persagy.server.scanbuilding.UserLog
import com.persagy.server.services.base.Service
import com.persagy.server.services.dict.EquipFamilyService
import com.persagy.server.utils.IdUtils
import com.persagy.server.utils.ShaftNameUtil
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext

/**
 * 资产信息服务
 *
 * @author 张维新
 */
object PropertyService : Service<Property>(SMybatisDao(Property::class.java)) {

    /** 扫楼日志 */
    var userLogService = SObjectService(SMybatisDao(UserLog::class.java))
    /** 用户 */
    var userService = SObjectService(SMybatisDao(User::class.java))

    /**
     * 创建之前操作
     */
    override fun onCreateBefore(property: Property): Boolean  {
        val projectId = SPageContext.getHeader("projectId")
        val coming = SPageContext.getHeader("coming")
        if (!property.id.isNullOrEmpty()) {
            val isStartsWith = property.id.toString().startsWith("Pe")
            if (!isStartsWith) {
                /** 资产id */
                property.id = IdUtils.peIdCreate(projectId!!)
            }
        } else {
            /** 资产id */
            property.id = IdUtils.peIdCreate(projectId!!)
        }
        /** 项目id */
        property.projectId = projectId
        // 创建标记
        if(coming.isNullOrEmpty()&&coming == "app"){
            property.source = "app"
        }
        if (!property.family.isNullOrEmpty()){
            val family = EquipFamilyService.select(SFilter.eq("family", property.family!!)).entity()
            if (family!=null){
                /** 资产名字 */
                if (property.name.isNullOrEmpty()) {
                    property.name = ShaftNameUtil.shaftName(family.familyName+"-")
                }
            }else{
                /** 资产名字 */
                if (property.name.isNullOrEmpty()) {
                    property.name = ShaftNameUtil.shaftName("资产-")
                }
            }
            return true
        }else{
            return false
        }
    } // Function onCreateBefore()

    override fun onCreateSuccess(entityList: ArrayList<Property>) {
        super.onCreateSuccess(entityList)
        try {
            val projectId = SPageContext.getHeader("projectId")
            val comming = SPageContext.getHeader("coming")
            val account = SPageContext.getHeader("account")
            for (entity in entityList){
                try {
                    if (!comming.isNullOrEmpty()){
                        if (comming == "app"){
                            val user = userService.select(SFilter.eq("projectId", projectId!!), SFilter.eq("phone", account!!)).entity()
                            val userLog = UserLog()
                            userLog.comming = "app"
                            userLog.action = "创建资产"
                            userLog.id = IdUtils.uuidCreate()
                            userLog.note = "项目ID: "+projectId+" 用户ID："+user!!.id+" 手机号："+ account+" 请求来源："+comming+ " "+entity.toJson()
                            userLog.phone = account
                            userLog.projectId = projectId
                            userLog.userId = user.id
                            userLog.name = user.name
                            userLog.objectId = entity.id
                            userLogService.insert(userLog)
                        }
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }



    /**
     * 查询资产数量前的操作
     *
     * @param   queryBuilder  条件
     */
    override fun onCountBefore(queryBuilder: SQueryBuilder<Property>) {
        queryBuilder.tableName = "v_property"
        return
    } // Function onCountBefore()

    /**
     * 查询资产数量前的操作
     *
     * @param   queryBuilder  条件
     */
    override fun onQueryBefore(queryBuilder: SQueryBuilder<Property>) {
        queryBuilder.tableName = "v_property"
        return
    } // Function onQueryBefore()


    override fun onUpdateBefore(entity: Property): Boolean {

        try {
            val projectId = SPageContext.getHeader("projectId")
            val comming = SPageContext.getHeader("coming")
            val account = SPageContext.getHeader("account")
                try {
                    if (!comming.isNullOrEmpty()){
                        if (comming == "app"){
                            val user = userService.select(SFilter.eq("projectId", projectId!!), SFilter.eq("phone", account!!)).entity()
                            val userLog = UserLog()
                            userLog.comming = "app"
                            userLog.action = "创建资产"
                            userLog.id = IdUtils.uuidCreate()
                            userLog.note = "项目ID: "+projectId+" 用户ID："+user!!.id+" 手机号："+ account+" 请求来源："+comming+ " "+entity.toJson()
                            userLog.phone = account
                            userLog.projectId = projectId
                            userLog.userId = user.id
                            userLog.name = user.name
                            userLog.objectId = entity.id
                            userLogService.insert(userLog)
                        }
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }

        } catch (e: Exception) {
            e.printStackTrace()
        }
        return super.onUpdateBefore(entity)
    }



    /**
     * 删除之前操作
     */
    override fun onDeleteBefore(entity: Property): Boolean {
        try {
            val  property = this.select(SFilter.eq("id", entity.id!!)).entity()

            val projectId = SPageContext.getHeader("projectId")
            val coming = SPageContext.getHeader("coming")
            val account = SPageContext.getHeader("account")
            try {
                /** 日志 */
                val dataCenterLogs = DataCenterLogs()
                dataCenterLogs.id = IdUtils.uuidCreate()
                dataCenterLogs.author = "dataCenter"
                dataCenterLogs.projectId = projectId
                dataCenterLogs.objectType = entity.objectType
                dataCenterLogs.type = "delete"
                dataCenterLogs.account = account
                dataCenterLogs.coming = coming
                dataCenterLogs.objectId = property!!.id
                val hashMap = HashMap<String, Any?>()
                hashMap[entity.objectType!!] = property
                dataCenterLogs.dataSource = hashMap
                logService.insert(dataCenterLogs)
            } catch (e: Exception) {
                e.printStackTrace()
            }

            try {
                /** 清除 */
                if (property!=null&&!property.equipId.isNullOrEmpty()){

                    if (property.equipId!!.startsWith("Eq")){
                        val equipment = EquipmentService.select(SFilter.eq("equipId", property.equipId!!)).entity()
                        if (equipment!=null&&equipment.taskState!=null&&equipment.taskState == -1&&equipment.bimId.isNullOrEmpty() ){
                            val listDel = ArrayList<Equipment>()
                            listDel.add(equipment)
                            EquipmentService.deleteByKeysList(listDel)
                        }
                    }else if (property.equipId!!.startsWith("Ec")){
                        val component = ComponentService.select(SFilter.eq("equipId", property.equipId!!)).entity()
                        if (component?.taskState != null &&component.taskState == -1&&component.bimId.isNullOrEmpty() ){
                            val listDel = ArrayList<Equipment>()
                            listDel.add(component)
                            EquipmentService.deleteByKeysList(listDel)
                        }
                    }
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }

        return super.onDeleteBefore(entity)
    } // Function onDeleteBefore()

} // Object PropertyService
