/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.objects

import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.logs.DataCenterLogs
import com.persagy.server.datacenter.wanda.obj.ZoneIspace
import com.persagy.server.datacenter.models.requests.DCSQueryRequest
import com.persagy.server.datacenter.services.objects.ZoneSpaceBaseService
import com.persagy.server.datacenter.wanda.obj.ZoneSpace
import com.persagy.server.services.base.Service
import com.persagy.server.utils.IdUtils
import com.persagy.server.utils.ShaftNameUtil
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.responses.SQueryResponse

/**
 * 模型空间信息服务
 *
 * @author 张维新
 */
object SpaceService : Service<ZoneIspace>(SMybatisDao(ZoneIspace::class.java)) {

    /**
     * 创建前的操作
     *
     * @param   space   创建的对象
     */
    override fun onCreateBefore(space: ZoneIspace): Boolean {
        val projectId = SPageContext.getHeader("projectId")
        /** 元空间id */
        if (!space.id.isNullOrEmpty()) {
            val isWith = space.id!!.startsWith("Si")
            if (!isWith) {
                space.id = IdUtils.iSpaceIdCreate(projectId!!)
            }
        } else {
            space.id = IdUtils.iSpaceIdCreate(projectId!!)
        }
        space.projectId = projectId

        /** 设备名字 */
        if (space.name.isNullOrEmpty()) {
            space.name = ShaftNameUtil.shaftName("空间-")
        }
        return true
    } // Function onCreateBefore()

    /**
     * @Description: 创建元空间后批量创建业务空间
     */
    override fun onCreateSuccess(entityList: ArrayList<ZoneIspace>) {
        super.onCreateSuccess(entityList)
        val zoneSpaces = ArrayList<ZoneSpace>()
        for (zoneIspace in entityList) {
            if(select(SFilter.eq("id",zoneIspace.id!!)).entity() == null){
                continue
            }
            val zoneSpace = ZoneSpace()
            zoneSpace.name = zoneIspace.name
            zoneSpace.bimLocation = zoneIspace.bimLocation
            zoneSpace.height = zoneIspace.height
            zoneSpace.localId = zoneIspace.localId
            zoneSpace.localName = zoneIspace.localName
            zoneSpace.projectId = zoneIspace.projectId
            zoneSpace.buildingId = zoneIspace.buildingId
            zoneSpace.floorId = zoneIspace.floorId
            zoneSpace.bimId = zoneIspace.bimId
            zoneSpace.classCode = "FunctionZone"
            zoneSpace.infos = zoneIspace.infos
            if(zoneIspace.outline != null){
                zoneSpace.outline = arrayListOf(zoneIspace.outline!!)
            }
            zoneSpace.state = 0
            zoneSpaces.add(zoneSpace)
        }
        val request = SCreateRequest<ZoneSpace>()
        request.content = zoneSpaces
        ZoneSpaceBaseService.createList(request)

    }

    override fun onDeleteBefore(entity: ZoneIspace): Boolean {
        try {
            val zoneIspace = this.select(SFilter.eq("id", entity.id!!)).entity()

            val projectId = SPageContext.getHeader("projectId")
            val coming = SPageContext.getHeader("coming")
            val account = SPageContext.getHeader("account")
            /** 日志 */
            val dataCenterLogs = DataCenterLogs()
            dataCenterLogs.id = IdUtils.uuidCreate()
            dataCenterLogs.author = "dataCenter"
            dataCenterLogs.projectId = projectId
            dataCenterLogs.objectType = entity.objectType
            dataCenterLogs.type = "delete"
            dataCenterLogs.coming = coming
            dataCenterLogs.account = account
            dataCenterLogs.objectId = zoneIspace!!.id
            val hashMap = HashMap<String, Any?>()
            hashMap[entity.objectType!!] = zoneIspace
            dataCenterLogs.dataSource = hashMap
            logService.insert(dataCenterLogs)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return super.onDeleteBefore(entity)
    }

    /**
     * 查询元空间
     */
    fun queryIspace(request: DCSQueryRequest): SQueryResponse<ZoneIspace> {
        return try {
            if (!request.floorId.isNullOrEmpty()) {

                val projectId = SPageContext.getHeader("projectId")

                val queryBuilder = select(request.filters)
                val entity = FloorService.select(SFilter.eq("id", request.floorId!!)).entity()
                if (entity!=null&&entity.modelId.isNullOrEmpty()){
                    queryBuilder.filter(SFilter.eq("floorId", request.floorId!!))
                }else{
                    queryBuilder.tableName = "f_ispace('$projectId','${request.floorId}')"
                }
                if (null != request.projection) {
                    queryBuilder.projection(request.projection!!)
                }
                if (null != request.orders) {
                    queryBuilder.order(request.orders!!)
                }

                val sPageQueryInfo = queryBuilder.pageQuery(request.pageNumber, request.pageSize)

                val sQueryResponse = SQueryResponse(sPageQueryInfo)
                sQueryResponse.message = request.floorId!!
                sQueryResponse

            } else {
                pageQuery(request, arrayListOf(SFilter.eq("projectId", SPageContext.getHeader("projectId").toString())))
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SQueryResponse(SResponseType.failure, e.message!!)
        }
    } //


} // Object SpaceService