/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.services.objects

import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.datacenter.models.entities.objects.Tenant
import com.persagy.server.datacenter.models.entities.objects.TenantWithCount
import com.persagy.server.datacenter.wanda.obj.ZoneSpace
import com.persagy.server.services.assistant.ObjInfoService
import com.persagy.server.services.base.Service
import com.persagy.server.services.log.DataCenterLogsService
import com.persagy.server.utils.IdUtils.tenantIdCreate
import com.persagy.server.utils.ShaftNameUtil
import com.persagy.service.SBaseService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.responses.SQueryResponse
import org.springframework.web.bind.annotation.RequestBody
import java.util.*

/**
 * 租户信息服务
 *
 * @author  李井强
 */
object TenantService : Service<Tenant>(SMybatisDao(Tenant::class.java)) {

    /**
     * 创建之前操作
     *
     * @param   tenant  租户对象
     */
    override fun onCreateBefore(tenant: Tenant): Boolean  {
        tenant.projectId = Opt.projectId
        if (tenant.id.isNullOrEmpty()){
            tenant.id = tenantIdCreate(Opt.projectId!!)
        }

        if (tenant.name.isNullOrEmpty()) {
            tenant.name = ShaftNameUtil.shaftName("租户-")
        }
        return true
    } // Function onCreateBefore()

    /**
     * 查询租户信息-包含统计信息
     *
     * @param       request
     * @return      租户统计信息
     */
    fun queryWithCount(@RequestBody request: SQueryRequest): SQueryResponse<TenantWithCount> {
        val service = SBaseService(SMybatisDao(TenantWithCount::class.java))
        return try {
            val queryBuilder = service.select(request.filters)
                    .filter(SFilter.eq("ProjectId", SPageContext.getHeader("projectId")!!))
            if (null != request.projection) {
                queryBuilder.projection(request.projection!!)
            }
            if (null != request.orders) {
                queryBuilder.order(request.orders!!)
            }
            val pageInfo = queryBuilder.pageQuery(request.pageNumber, request.pageSize)
            SQueryResponse(pageInfo)
        } catch (e: Exception) {
            e.printStackTrace()
            SQueryResponse(SResponseType.failure, e.message!!)
        }
    } // Function queryWithCount()


    /**
     * 空间是否存在
     *
     * @param       zone       failureList
     * @return      是否存在
     */
    private fun zoneExists(zone: String, failureList: ArrayList<Any>): Boolean {
        val service = SBaseService(SMybatisDao(ZoneSpace::class.java))
        val list = service.select(SFilter.eq("RoomID", zone)).exec()
        if (list.size > 0) {
            return true
        }
        failureList.add(zone)
        return false
    }// Function zoneExists()

    /** 删除之前操作 */
    override fun onDeleteBefore(entity: Tenant): Boolean {
        try {
            val  tenant = this.select(SFilter.eq("id", entity.id!!)).entity()
            /** 日志 */
            DataCenterLogsService.addLogs(Opt.projectId!!,"delete",entity.objectType!!,
                tenant!!.id!!,tenant,Opt.account,Opt.coming)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return super.onDeleteBefore(entity)
    } // Function onDeleteBefore()


    /**
     * @Description: 创建成功
     */
    override fun onCreateSuccess(entityList: ArrayList<Tenant>) {
        super.onCreateSuccess(entityList)
        for (tenant in entityList) {

            try {
                rabbitMqService.sendObjects(Opt.projectId!!, Opt.CREATE,tenant.id!!)
            } catch (e: Exception) {
                e.printStackTrace()
            }
            try {
                /** 日志 */
                DataCenterLogsService.addLogs(Opt.projectId!!,"create",tenant.objectType!!,
                    tenant.id!!,tenant,Opt.account,Opt.coming)
            } catch (e: Exception) {
                e.printStackTrace()
            }
            ObjInfoService.createInfo(tenant,true,"VOTn","Tenant")
        }
    }


    /**
     * @Description: 更新信息点
     */
    override fun onUpdateSuccess(entityList: ArrayList<Tenant>) {
        super.onUpdateSuccess(entityList)
        for (tenant in entityList) {
            try {
                rabbitMqService.sendObjects(Opt.projectId!!, Opt.UPDATE,tenant.id!!)
            } catch (e: Exception) {
                e.printStackTrace()
            }
            try {
                /** 日志 */
                DataCenterLogsService.addLogs(Opt.projectId!!,"update",tenant.objectType!!,
                    tenant.id!!,tenant,Opt.account,Opt.coming)
            } catch (e: Exception) {
                e.printStackTrace()
            }
            ObjInfoService.createInfo(tenant,false,"VOTn","Tenant")
        }
    }


    /**
     * @Description:删除成功之后
     */
    override fun onDeleteSuccess(entityList: ArrayList<Tenant>) {
        super.onDeleteSuccess(entityList)

        try {
            for (tenant in entityList) {
                try {
                    rabbitMqService.sendObjects(Opt.projectId!!, Opt.DELETE,tenant.id!!)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                try {
                    /** 日志 */
                    DataCenterLogsService.addLogs(Opt.projectId!!,"delete",tenant.objectType!!,
                        tenant.id!!,tenant,Opt.account,Opt.coming)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                ObjInfoService.delete(SFilter.eq("projectId",Opt.projectId!!), SFilter.eq("objId",tenant.id!!), SFilter.eq("objType","VOTn"))
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

} // Object TenantService