/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.rel

import com.persagy.base.extensions.toJson
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.models.entities.rel.REqForFl
import com.persagy.server.datacenter.models.entities.rel.REqForFlList
import com.persagy.server.datacenter.models.entities.rel.RFlForEqList
import com.persagy.server.datacenter.services.base.RService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse

/**
 * 设备服务于楼层关系服务
 *
 * @author 付国宝
 */
object REqForFlService: RService<REqForFl>(SMybatisDao(REqForFl::class.java)) {

    /**
     * 创建设备服务于楼层关系
     *
     * @param    rEqForFl      设备服务于楼层关系对象
     */
    fun create(rEqForFl: REqForFl): SCreateResponse<REqForFl> {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            rEqForFl.projectId = projectId
            val isInsert = insert(rEqForFl)
            if (isInsert){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Eq2Fl","11",rEqForFl.equipId!!,rEqForFl.floorId!!)
                SCreateResponse(SResponseType.success)
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    }

    /**
     * 根据对象删除
     *
     * @param    rEqForFl     关系对象实例
     */
    fun deletes(rEqForFl: REqForFl): SBaseResponse {
        return try {
            val isDelete = delete(rEqForFl)
            /** 删除成功 */
            if (isDelete) {
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Eq2Fl","12",rEqForFl.equipId!!,rEqForFl.floorId!!)
                val sBaseResponse = SBaseResponse(SResponseType.success)
                sBaseResponse.message = rEqForFl.toJson()
                sBaseResponse
            } else {
                SBaseResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Function deletes()

    /**
     * 创建设备服务于楼层关系
     *
     * @param   rEqForFlList    设备和楼层关系的对象列表
     */
    fun createEq(rEqForFlList: REqForFlList): SCreateResponse<REqForFl> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("equipId = '${rEqForFlList.equipId}'")
            for (floorId in rEqForFlList.floorIdList!!){
                val rEqForFl = REqForFl()
                rEqForFl.equipId = rEqForFlList.equipId
                rEqForFl.floorId = floorId
                rEqForFl.projectId = Opt.projectId
                val replace = replace(rEqForFl)
                if (replace){
                    successList.add(floorId)
                }else{
                    failureList.add(rEqForFl)
                }
            }
            if (successList.size>0){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Eq2Fl","13",rEqForFlList.equipId!!,successList.toJson()!!)
                val sCreateResponse = SCreateResponse<REqForFl>(SResponseType.success)
                if (failureList.size>0){
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                }
                sCreateResponse
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    }

    /**
     * 创建服务于楼层的设备关系
     *
     * @param    rEqForFlList     设备和楼层关系的对象列表
     */
    fun createsFl(rEqForFlList: RFlForEqList): SCreateResponse<REqForFl> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("floorId = '${rEqForFlList.floorId}'")
            for (equipId in rEqForFlList.equipIdList!!){
                val rEqForFl = REqForFl()
                rEqForFl.floorId = rEqForFlList.floorId
                rEqForFl.equipId = equipId
                rEqForFl.projectId = Opt.projectId
                val replace = replace(rEqForFl)
                if (replace){
                    successList.add(equipId)
                }else{
                    failureList.add(rEqForFl)
                }
            }
            if (successList.size>0){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Eq2Fl","13",rEqForFlList.floorId!!,successList.toJson()!!)
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<REqForFl>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<REqForFl>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    }

}