/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.rel

import com.persagy.base.extensions.toJson
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.wanda.rel.REq2Sp
import com.persagy.server.datacenter.models.entities.rel.REqForSpBase
import com.persagy.server.datacenter.models.entities.rel.RSpForEqBaseList
import com.persagy.server.datacenter.services.base.RService
import com.persagy.service.SObjectService
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse

/**
 * 设备服务于业务空间关系服务
 *
 * @author 付国宝
 */
object REqForSpBaseService: RService<REqForSpBase>(SMybatisDao(REqForSpBase::class.java)){


    var rEq2SpService = SObjectService(SMybatisDao( REq2Sp::class.java))

    /**
     * 创建设备服务于空间关系
     *
     * @param    rEqForSpBase    设备服务于空间关系对象
     */
    fun create(type: String, rEqForSpBase: SCreateRequest<REqForSpBase>): SCreateResponse<REqForSpBase> {
        return try {
            for (rEqForSp in rEqForSpBase.content){

                try {
                    val rEq2Sp = REq2Sp()
                    rEq2Sp.id1 = rEqForSp.equipId
                    rEq2Sp.id2 = rEqForSp.spaceId
                    rEq2Sp.projectId = Opt.projectId
                    rEq2Sp.sign = 1
                    rEq2Sp.zoneType = type
                    rEq2Sp.type = "eq2sp_for"
                    val isInsert = rEq2SpService.insert(rEq2Sp)
                    if (isInsert){
                        /** 发送消息 */
                        rabbitMqService.sendRel(Opt.projectId!!, "EquipforSpace","11",rEqForSp.equipId!!,rEqForSp.spaceId!!)
                    }else{

                    }
                }catch (e: Exception){
                    e.printStackTrace()
                }
            }
            SCreateResponse(SResponseType.success)

        } catch (e: Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    }

    /**
     * 根据对象删除
     *
     * @param    rEqForSpBaseList    关系对象实例
     */
    fun deletes(type: String, rEqForSpBaseList: ArrayList<REqForSpBase>): SBaseResponse {
        return try {
            for (rEqForSpBase in rEqForSpBaseList){
                try {
                    /** 删除成功 */
                    val rEq2Sp = REq2Sp()
                    rEq2Sp.projectId = Opt.projectId
                    rEq2Sp.id1 = rEqForSpBase.equipId
                    rEq2Sp.id2 = rEqForSpBase.spaceId
                    rEq2Sp.zoneType = type
                    rEq2Sp.type = "eq2sp_for"
                    val isDelete = rEq2SpService.delete(rEq2Sp)
                    if (isDelete) {
                        /** 发送消息 */
                        rabbitMqService.sendRel(Opt.projectId!!, "EquipforSpace","12",rEqForSpBase.equipId!!,rEqForSpBase.spaceId!!)
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
            SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    }

    /**
     * 创建服务于业务空间的设备关系
     *
     * @param    rEqForSpBaseList    设备和业务空间关系的对象列表
     */
    fun createsSpEq(type: String,rEqForSpBaseList: RSpForEqBaseList): SCreateResponse<RSpForEqBaseList> {
        return try {

            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            for (equipId in rEqForSpBaseList.equipIdList!!){
                val rEq2Sp = REq2Sp()
                rEq2Sp.id2 = rEqForSpBaseList.spaceId
                rEq2Sp.id1 = equipId
                rEq2Sp.projectId = Opt.projectId
                rEq2Sp.sign = 1
                rEq2Sp.zoneType = type
                rEq2Sp.type = "eq2sp_for"
                val replace = rEq2SpService.replace(rEq2Sp)
                if (replace){
                    successList.add(equipId)
                }else{
                    failureList.add(rEq2Sp)
                }
            }
            if (successList.size>0){
                rabbitMqService.sendRel(Opt.projectId!!, "EquipforSpace","14",rEqForSpBaseList.spaceId!!,successList.toJson()!!)
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<RSpForEqBaseList>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<RSpForEqBaseList>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    }

}