/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.rel

import com.persagy.base.extensions.toJson
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.wanda.rel.REq2Sp
import com.persagy.server.datacenter.models.entities.rel.REqInSpBase
import com.persagy.server.datacenter.models.entities.rel.REqInSpBaseList
import com.persagy.server.datacenter.models.entities.rel.RSpInEqBaseList
import com.persagy.server.datacenter.services.base.RService
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import org.json.JSONObject

/**
 * 设备所在业务空间关系服务
 *
 * @author 付国宝
 */
object REqInSpBaseService: RService<REqInSpBase>(SMybatisDao(REqInSpBase::class.java)) {

    /**
     * 设备所在业务空间
     */
    var rEq2SpService = SObjectService(SMybatisDao( REq2Sp::class.java))

    /**
     * 创建设备所在业务空间关系
     *
     * @param    rEqInSpBase    设备所在业务空间关系对象
     */
    fun create(type: String, rEqInSpBaseList: SCreateRequest<REqInSpBase>): SCreateResponse<REqInSpBase> {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            for (rEqInSpBase in rEqInSpBaseList.content){
                try {
                    val rEq2Sp = REq2Sp()
                    rEq2Sp.zoneType = type
                    rEq2Sp.type = "eq2sp_in"
                    rEq2Sp.sign = 1
                    rEq2Sp.id1 = rEqInSpBase.equipId
                    rEq2Sp.id2 = rEqInSpBase.spaceId
                    rEq2Sp.projectId = projectId
                    val replace = rEq2SpService.replace(rEq2Sp)
                    if (replace){
                        val jsonObject = JSONObject()
                        jsonObject.put("type", "11")
                        jsonObject.put("projectId", projectId)
                        jsonObject.put("id", rEqInSpBase.spaceId)
                        jsonObject.put("obj_type", "EquipinSpace")
                        jsonObject.put("linkId",rEqInSpBase.equipId)

                        /** 发送消息 */
//                        messageHandler.sendMessage(jsonObject.toString())
                        /** 发送消息 */
                        rabbitMqService.sendRel(Opt.projectId!!, "EquipinSpace","11",rEqInSpBase.equipId!!,rEqInSpBase.spaceId!!)
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
            SCreateResponse(SResponseType.success)

        } catch (e: Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function create()

    /**
     * 根据对象删除
     *
     * @param    rEqInSpBaseList    关系对象实例
     */
    fun deletes(type: String, rEqInSpBaseList: ArrayList<REqInSpBase>): SBaseResponse {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            val list= ArrayList<REq2Sp>()
            for (rEqInSpBase in rEqInSpBaseList){
                val rEq2Sp = REq2Sp()
                rEq2Sp.projectId = projectId
                rEq2Sp.id1 = rEqInSpBase.equipId
                rEq2Sp.id2 = rEqInSpBase.spaceId
                rEq2Sp.zoneType = type
                rEq2Sp.type = "eq2sp_in"
                list.add(rEq2Sp)
            }
            REq2SpService.deleteByKeysList(list)

        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Function deletes()

    /**
     * 创建设备所在业务空间关系
     *
     * @param    rEqInSpBaseList    设备和业务空间关系的对象列表
     */
    fun createEq(rEqInSpBaseList: REqInSpBaseList): SCreateResponse<REqInSpBase> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            REq2SpService.delete("id1 = '${rEqInSpBaseList.equipId}'")
            for (spaceId in rEqInSpBaseList.spaceIdList!!){
                val rEqInSpBase = REqInSpBase()
                rEqInSpBase.equipId = rEqInSpBaseList.equipId
                rEqInSpBase.spaceId = spaceId
                rEqInSpBase.projectId = Opt.projectId
                val replace = replace(rEqInSpBase)
                if (replace){
                    successList.add(spaceId)
                }else{
                    failureList.add(rEqInSpBase)
                }
            }
            if (successList.size>0){
                val jsonObject = JSONObject()
                jsonObject.put("type", "13")
                jsonObject.put("projectId", Opt.projectId)
                jsonObject.put("id", rEqInSpBaseList.equipId)
                jsonObject.put("obj_type", "EquipinSpace")
                jsonObject.put("linkId",successList.toJson())

                /** 发送消息 */
//                RService.messageHandler.sendMessage(jsonObject.toString())
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "EquipinSpace","13",rEqInSpBaseList.equipId!!,successList.toJson()!!)
                val sCreateResponse = SCreateResponse<REqInSpBase>(SResponseType.success)
                if (failureList.size>0){
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                }
                sCreateResponse
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createEq()

    /**
     * 创建所在业务空间的设备关系
     *
     * @param    rEqInSpBaseList    设备和业务空间关系的对象列表
     */
    fun createsSp(rEqInSpBaseList: RSpInEqBaseList): SCreateResponse<REqInSpBase> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            REq2SpService.delete("id2 = '${rEqInSpBaseList.spaceId}'")
            for (equipId in rEqInSpBaseList.equipIdList!!){
                val rEqInSpBase = REqInSpBase()
                rEqInSpBase.spaceId = rEqInSpBaseList.spaceId
                rEqInSpBase.equipId = equipId
                rEqInSpBase.projectId = Opt.projectId
                val replace = replace(rEqInSpBase)
                if (replace){
                    successList.add(equipId)
                }else{
                    failureList.add(rEqInSpBase)
                }
            }
            if (successList.size>0){
                val jsonObject = JSONObject()
                jsonObject.put("type", "13")
                jsonObject.put("projectId", Opt.projectId)
                jsonObject.put("id", rEqInSpBaseList.spaceId)
                jsonObject.put("obj_type", "EquipinSpace")
                jsonObject.put("linkId",successList.toJson())

                /** 发送消息 */
//                RService.messageHandler.sendMessage(jsonObject.toString())
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "EquipinSpace","13",rEqInSpBaseList.spaceId!!,successList.toJson()!!)
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<REqInSpBase>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<REqInSpBase>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsSp()

    /**
     * 创建所在业务空间的设备关系
     *
     * @param    rEqInSpBaseList    设备和业务空间关系的对象列表
     */
    fun createsSpEq(type: String,rEqInSpBaseList: RSpInEqBaseList): SCreateResponse<RSpInEqBaseList> {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            /** 业务空间类型服务 */

            if (!type.isNullOrEmpty()){
                try {
                    /** 成功集合 */
                    val successList = ArrayList<Any>()
                    /** 失败集合 */
                    val failureList = ArrayList<Any>()
                    for (rEqInSpBase in rEqInSpBaseList.equipIdList!!){
                        val rEq2Sp = REq2Sp()
                        rEq2Sp.id1 = rEqInSpBase
                        rEq2Sp.id2 = rEqInSpBaseList.spaceId
                        rEq2Sp.projectId = projectId
                        rEq2Sp.sign = 1
                        rEq2Sp.type ="eq2sp_in"
                        rEq2Sp.zoneType =  type
                        val replace = rEq2SpService.replace(rEq2Sp)
                        if (replace){
                            successList.add(rEq2Sp.id1!!)
                        }else{
                            failureList.add(rEq2Sp)
                        }
                    }

                    if (successList.size>0){
                        val jsonObject = JSONObject()
                        jsonObject.put("type", "14")
                        jsonObject.put("projectId", projectId)
                        jsonObject.put("id", rEqInSpBaseList.spaceId)
                        jsonObject.put("obj_type", "EquipinSpace")
                        jsonObject.put("linkId",successList.toJson())
                        /** 发送消息 */
//                        messageHandler.sendMessage(jsonObject.toString())
                        rabbitMqService.sendRel(Opt.projectId!!, "EquipinSpace","14",rEqInSpBaseList.spaceId!!,successList.toJson()!!)
                        if (failureList.size>0){
                            val sCreateResponse = SCreateResponse<RSpInEqBaseList>(SResponseType.success)
                            sCreateResponse.message ="失败对象："+failureList.toJson()
                            sCreateResponse
                        }else{
                            val sCreateResponse = SCreateResponse<RSpInEqBaseList>(SResponseType.success)
                            sCreateResponse
                        }
                    }else{
                        SCreateResponse(SResponseType.failure)
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                val sCreateResponse = SCreateResponse<RSpInEqBaseList>(SResponseType.success)
                sCreateResponse
            }else{
                val sCreateResponse = SCreateResponse<RSpInEqBaseList>(SResponseType.failure)
                sCreateResponse.message = "空间分区类型不存在：$type"
                sCreateResponse
            }

        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsSp()

} // Object REqInSpBaseService