/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.rel

import com.persagy.base.extensions.toJson
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.models.entities.rel.RFlThroughFl
import com.persagy.server.datacenter.models.entities.rel.RFloorOtherThroughFloorList
import com.persagy.server.datacenter.models.entities.rel.RFloorThroughFloorOtherList
import com.persagy.server.datacenter.services.base.RService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse

/**
 * 楼层的贯通关系服务
 *
 * @author 付国宝
 */
object RFlThroughFlService : RService<RFlThroughFl>(SMybatisDao(RFlThroughFl::class.java)) {

    /**
     * 创建楼层的贯通关系
     *
     * @param   rFlThroughFl    楼层的贯通关系对象
     */
    fun create(rFlThroughFl: RFlThroughFl): SCreateResponse<RFlThroughFl> {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            rFlThroughFl.projectId = projectId
            val isInsert = insert(rFlThroughFl)
            if (isInsert){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Fl2Fl","11",rFlThroughFl.floorId!!,rFlThroughFl.floorOtherId!!)
                SCreateResponse(SResponseType.success)
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Tenant create()

    /**
     * 根据对象删除
     *
     * @param   rFlThroughFl    关系对象实例
     */
    fun deletes(rFlThroughFl: RFlThroughFl): SBaseResponse {
        return try {
            val isDelete = delete(rFlThroughFl)
            /** 删除成功 */
            if (isDelete) {
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Fl2Fl","12",rFlThroughFl.floorId!!,rFlThroughFl.floorOtherId!!)
                val sBaseResponse = SBaseResponse(SResponseType.success)
                sBaseResponse.message = rFlThroughFl.toJson()
                sBaseResponse
            } else {
                SBaseResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Tenant deletes()

    /**
     * 创建楼层的贯通关系
     *
     * @param   rFloorThroughFloorOtherList    楼层和楼层列表关系对象
     */
    fun createFl(rFloorThroughFloorOtherList: RFloorThroughFloorOtherList): SCreateResponse<RFlThroughFl> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()

            try {
                delete("floorId = '${rFloorThroughFloorOtherList.floorId}'")
                delete("floorOtherId = '${rFloorThroughFloorOtherList.floorId}'")
            } catch (e: Exception) {
                e.printStackTrace()
            }
            if (rFloorThroughFloorOtherList.floorOtherIdList!=null&&rFloorThroughFloorOtherList.floorOtherIdList!!.isNotEmpty()){
                for (floorOtherId in rFloorThroughFloorOtherList.floorOtherIdList!!){
                    val rFlThroughFl = RFlThroughFl()
                    rFlThroughFl.floorId = rFloorThroughFloorOtherList.floorId
                    rFlThroughFl.floorOtherId = floorOtherId
                    rFlThroughFl.projectId = Opt.projectId
                    val replace = replace(rFlThroughFl)
                    if (replace){
                        successList.add(floorOtherId)
                    }else{
                        failureList.add(rFlThroughFl)
                    }
                }
                if (successList.size>0){
                    /** 发送消息 */
                    rabbitMqService.sendRel(Opt.projectId!!, "Fl2Fl","13",rFloorThroughFloorOtherList.floorId!!,successList.toJson()!!)

                    val sCreateResponse = SCreateResponse<RFlThroughFl>(SResponseType.success)
                    if (failureList.size>0){
                        sCreateResponse.message ="失败对象："+failureList.toJson()
                    }
                    sCreateResponse
                }else{
                    SCreateResponse(SResponseType.failure)
                }
            }else{
                SCreateResponse(SResponseType.success)
            }

        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Tenant createFl()

    /**
     * 创建楼层的贯通关系
     *
     * @param   rFollrOtherThroughFloorList    楼层列表和楼层关系对象
     */
    fun createsFl(rFollrOtherThroughFloorList: RFloorOtherThroughFloorList): SCreateResponse<RFlThroughFl> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()

            try {
                delete("floorId = '${rFollrOtherThroughFloorList.floorOtherId}'")
                delete("floorOtherId = '${rFollrOtherThroughFloorList.floorOtherId}'")
            } catch (e: Exception) {
                e.printStackTrace()
            }
            val projectId = SPageContext.getHeader("projectId")
            if (rFollrOtherThroughFloorList.floorIdList!=null&&rFollrOtherThroughFloorList.floorIdList!!.isNotEmpty()){
                for (floorId in rFollrOtherThroughFloorList.floorIdList!!){
                    val rFlThroughFl = RFlThroughFl()
                    rFlThroughFl.floorOtherId = rFollrOtherThroughFloorList.floorOtherId
                    rFlThroughFl.floorId = floorId
                    rFlThroughFl.projectId = projectId
                    val replace = replace(rFlThroughFl)
                    if (replace){
                        successList.add(floorId)
                    }else{
                        failureList.add(rFlThroughFl)
                    }
                }
                if (successList.size>0){
                    /** 发送消息 */
                    rabbitMqService.sendRel(Opt.projectId!!, "Fl2Fl","13",rFollrOtherThroughFloorList.floorOtherId!!,successList.toJson()!!)
                    if (failureList.size>0){
                        val sCreateResponse = SCreateResponse<RFlThroughFl>(SResponseType.success)
                        sCreateResponse.message ="失败对象："+failureList.toJson()
                        sCreateResponse
                    }else{
                        val sCreateResponse = SCreateResponse<RFlThroughFl>(SResponseType.success)
                        sCreateResponse
                    }
                }else{
                    SCreateResponse(SResponseType.failure)
                }
            }else{
                SCreateResponse(SResponseType.success)
            }

        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    }

}