/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.rel

import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.models.entities.rel.RSpinBd
import com.persagy.server.datacenter.services.base.RService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse

/**
 * 业务空间所在的建筑
 *
 * @author 张维新
 */
object RSpInBdService: RService<RSpinBd>(SMybatisDao(RSpinBd::class.java)) {

    /**
     * 创建业务空间和建筑的关系
     */
    fun  spInBdCreates(rSiInSpBase: SCreateRequest<RSpinBd>): SCreateResponse<RSpinBd> {
        try {
            val projectId = SPageContext.getHeader("projectId")
            val contentList = rSiInSpBase.content
            if (contentList.isNotEmpty()){
                for (rSpinBd in contentList){

                    if (!rSpinBd.objectType.isNullOrEmpty()){
                        rSpinBd.projectId = projectId
                        val insert = insert(rSpinBd)
                        if (insert){
                            /** 关系创建成功 */
                            /** 发送消息 */
                            rabbitMqService.sendRel(Opt.projectId!!, "Bd2Sp","11",rSpinBd.buildingId!!,rSpinBd.spaceId!!)
                        }
                    }else{
                        val sCreateResponse = SCreateResponse<RSpinBd>(SResponseType.failure)
                        sCreateResponse.message = "空间类型不能为空"
                        return  sCreateResponse
                    }
                }
                val sCreateResponse = SCreateResponse<RSpinBd>(SResponseType.success)
                return sCreateResponse
            }else{
                val sCreateResponse = SCreateResponse<RSpinBd>(SResponseType.failure)
                sCreateResponse.message = "列表不可以为空"
                return sCreateResponse
            }

        } catch (e: Exception) {
            e.printStackTrace()
            return SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function spInBdCreates()


    /**
     * 删除业务空间也建筑的关系
     */
    fun spInBdDeLs(rSpinBdList: ArrayList<RSpinBd>): SBaseResponse {
        try {
            val projectId = SPageContext.getHeader("projectId")
            if (!rSpinBdList.isNullOrEmpty()){
                for (rSpinBd in rSpinBdList){
                    if (!rSpinBd.spaceId.isNullOrEmpty()&&!rSpinBd.buildingId.isNullOrEmpty()){
                        rSpinBd.projectId = projectId
                        val delete = delete(rSpinBd)
                        if (delete){
                            /** 删除成功 */
                            /** 发送消息 */
                           rabbitMqService.sendRel(Opt.projectId!!, "Bd2Sp","12",rSpinBd.buildingId!!,rSpinBd.spaceId!!)
                        }
                    }else{
                        val sBaseResponse = SBaseResponse(SResponseType.failure)
                        sBaseResponse.message = "spaceId和buildingId 不可以为空"
                        return sBaseResponse
                    }
                }
                return SBaseResponse(SResponseType.success)
            }else{
                val sBaseResponse = SBaseResponse(SResponseType.failure)
                sBaseResponse.message = "列表不能为空"
                return sBaseResponse
            }
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Function spInBdDeLs()

    /**
     * 修改业务空间所在的建筑
     */
    fun spInBdUpdates(request: SUpdateRequest<RSpinBd>): SBaseResponse{
        try {
            val projectId = SPageContext.getHeader("projectId")
            val contentList = request.content
            if (!contentList.isNullOrEmpty()){

                for (content in contentList){
                    if (!content.buildingId.isNullOrEmpty()&&!content.spaceId.isNullOrEmpty()&&!content.objectType.isNullOrEmpty()){

                        try {
                            val delete = delete(SFilter.eq("projectId", projectId!!), SFilter.eq("SpaceId", content.spaceId!!))
                            if (delete){
                                /** 删除原始的空间和建筑的数据 */

                            }
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }

                        val insert = insert(content)
                        if (insert){
                            /** 添加关系 */
                            /** 发送消息 */
                            rabbitMqService.sendRel(Opt.projectId!!, "Bd2Sp","13",content.buildingId!!,content.spaceId!!)
                        }

                    }else{
                        val sBaseResponse = SBaseResponse(SResponseType.failure)
                        sBaseResponse.message = "buildingId、spaceId、objectType 不能为空"
                        return sBaseResponse
                    }
                }
                return SBaseResponse(SResponseType.success)
            }else{
                val sBaseResponse = SBaseResponse(SResponseType.failure)
                sBaseResponse.message = "列表不能为空"
                return sBaseResponse
            }

        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message.toString())
        }

    } // Function spInBdUpdates()

} // Object RSpInBdService