/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.rel

import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.rel.RSpatialConnection
import com.persagy.server.datacenter.services.objects.ZoneSpaceBaseService
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import org.slf4j.LoggerFactory

/**
 * 空间关系
 *
 * @author 张维新
 */
object RSpatialConnectionService : SObjectService<RSpatialConnection>(SMybatisDao(RSpatialConnection::class.java)) {

    private val logger = LoggerFactory.getLogger(RSpatialConnectionService::class.java)

    /**
     * 创建
     */
    fun create(rSiInSpBase: SCreateRequest<RSpatialConnection>): SCreateResponse<RSpatialConnection> {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            val contentList = rSiInSpBase.content
            if (contentList.size > 0) {
                for (content in contentList) {
                    content.projectId = projectId!!
                        logger.debug("项目id =${content.projectId}")
                    if (content.graphType.isNullOrEmpty() || content.floorId.isNullOrEmpty()) {
                        val sCreateResponse = SCreateResponse<RSpatialConnection>(SResponseType.failure)
                        sCreateResponse.message = "请检查数据的完整性"
                        return sCreateResponse
                    }

                    if (content.zoneType.isNullOrEmpty()) {
                        val zoneSpace = ZoneSpaceBaseService.select(SFilter.eq("id", content.spaceIdOne!!)).entity()
                        if (zoneSpace != null) {
                            content.zoneType = zoneSpace.classCode
                        }
                    }
                    val insert = replace(content)
                    if (insert) {
                    }
                }
                val sCreateResponse = SCreateResponse<RSpatialConnection>(SResponseType.success)
                sCreateResponse.entityList = contentList
                return sCreateResponse
            } else {
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure, e.message.toString())
        }


    } // Fun onCreateBefore()

    /**
     * 查询
     */
    fun pageQuerys(request: SQueryRequest): SQueryResponse<RSpatialConnection> {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            val filter = select(request.filters).filter(SFilter.eq("projectId", projectId!!)).exec()
            val sQueryResponse = SQueryResponse<RSpatialConnection>(SResponseType.success)
            sQueryResponse.total = filter.size.toLong()
            sQueryResponse.content = filter
            sQueryResponse
        } catch (e: Exception) {
            e.printStackTrace()
            SQueryResponse(SResponseType.failure)
        }
    } //

    /**
     * 删除
     */
    fun  delete(idList: ArrayList<RSpatialConnection>): SBaseResponse {
        return try {
            if (idList.size>0){
                val projectId = SPageContext.getHeader("projectId")
                for (rSpatialConnection in idList){
                    try {
                        delete(SFilter.eq("zoneType",rSpatialConnection.zoneType!!),SFilter.eq("graphType",rSpatialConnection.graphType!!),SFilter.eq("projectId",projectId!!),SFilter.eq("spaceIdOne",rSpatialConnection.spaceIdOne!!),SFilter.eq("spaceIdTwo",rSpatialConnection.spaceIdTwo!!))
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }

                    try {
                        delete(SFilter.eq("zoneType",rSpatialConnection.zoneType!!),SFilter.eq("graphType",rSpatialConnection.graphType!!),SFilter.eq("projectId",projectId!!),SFilter.eq("spaceIdOne",rSpatialConnection.spaceIdTwo!!),SFilter.eq("spaceIdTwo",rSpatialConnection.spaceIdOne!!))
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                }
                SBaseResponse(SResponseType.success)
            }else{
                SBaseResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SBaseResponse(SResponseType.failure)
        }

    } //


    /**
     * 删除某个楼层下的数据
     */
    fun  deleteFloor(rSpatialConnection: RSpatialConnection): SBaseResponse {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            delete(SFilter.eq("graphType",rSpatialConnection.graphType!!),SFilter.eq("projectId",projectId!!),SFilter.eq("floorId",rSpatialConnection.floorId!!),SFilter.eq("zoneType",rSpatialConnection.zoneType!!))

            SBaseResponse(SResponseType.success)

        } catch (e: Exception) {
            e.printStackTrace()
            SBaseResponse(SResponseType.failure)
        }

    } //



} // Object RSpatialConnectionService