/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.rel

import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.models.entities.rel.REqSyList
import com.persagy.server.datacenter.models.entities.rel.RSyEq
import com.persagy.server.datacenter.models.entities.rel.RSyEqList
import com.persagy.server.datacenter.services.base.RService
import com.persagy.server.datacenter.services.objects.EquipmentService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import org.slf4j.LoggerFactory

/**
 * 系统下的设备设施关系服务
 *
 * @author 张维新
 */
object RSyEqService : RService<RSyEq>(SMybatisDao(RSyEq::class.java)) {

    private val logger = LoggerFactory.getLogger(RSyEqService::class.java)

    /**
     * 创建系统下设备关系
     *
     * @param   rSyEq   系统和设备的关系对象
     */
    fun create(rSyEq: RSyEq): SCreateResponse<RSyEq> {
        return try {
            rSyEq.projectId = Opt.projectId
            val isInsert = replace(rSyEq)
            if (isInsert){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "SystemEquip","11",rSyEq.sysId!!,rSyEq.equipId!!)
                SCreateResponse(SResponseType.success)
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    }

    /**
     * 根据对象删除
     *
     * @param   rSyEq      关系对象实例
     * @return
     */
     fun deletes(rSyEq: RSyEq): SBaseResponse {
        return try {
                val isDelete = delete(rSyEq)
                /** 删除成功 */
                if (isDelete) {
                    try {
                        /** 处理关系标记 */
                        val listtype = ArrayList<String>()
                        listtype.add("syeq2bd")
                        listtype.add("sy2fl")
                        listtype.add("sy2sh")
                        listtype.add("sy2sp")
                        for (type in listtype){
                            val entity = EquipmentService.relationTypeProjectservice.select(SFilter.eq("projectId", Opt.projectId!!), SFilter.eq("relationType", type)).entity()
                            if (entity!=null){
                                entity.computationalState = 2
                                EquipmentService.relationTypeProjectservice.update(entity)
                            }
                        }
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                    val sBaseResponse = SBaseResponse(SResponseType.success)
                    sBaseResponse.message = rSyEq.toJson()
                    try {
                        /** 发送消息 */
                        rabbitMqService.sendRel(Opt.projectId!!, "SystemEquip","12",rSyEq.sysId!!,rSyEq.equipId!!)
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                    sBaseResponse
                } else {
                    SBaseResponse(SResponseType.failure)
                }
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    }

    /**
     * 创建系统下设备关系
     *
     * @param   rSyEqList   系统和设备的关系对象列表
     */
    fun createSy(rSyEqList: RSyEqList): SCreateResponse<RSyEq> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            try {
                delete("sysId = '${rSyEqList.sysId}'")
            } catch (e: Exception) {
                e.printStackTrace()
            }
            if (rSyEqList.equipIdList!=null&&rSyEqList.equipIdList!!.isNotEmpty()){
                for (eqId in rSyEqList.equipIdList!!){
                    val rSyEq = RSyEq()
                    rSyEq.equipId = eqId
                    rSyEq.sysId =  rSyEqList.sysId
                    rSyEq.projectId = Opt.projectId
                    val replace = replace(rSyEq)
                    if (replace){
                        successList.add(eqId)
                    }else{
                        failureList.add(rSyEq)
                    }
                }
                if (successList.size>0){

                    try {
                        /** 发送消息 */
                        rabbitMqService.sendRel(Opt.projectId!!, "SystemEquip","13",rSyEqList.sysId!!,successList.toJson()!!)
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                    val sCreateResponse = SCreateResponse<RSyEq>(SResponseType.success)
                    if (failureList.size>0){
                        sCreateResponse.message ="失败对象："+failureList.toJson()
                    }
                    sCreateResponse
                }else{
                    SCreateResponse(SResponseType.failure)
                }
            }else{
                SCreateResponse(SResponseType.success)
            }

        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    }

    /**
    * 创建系统下设备关系
    *
    * @param   rSyEqList   系统和设备的关系对象列表
    */
    fun createsEq(rSyEqList: REqSyList): SCreateResponse<RSyEq> {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()

            try {
                delete("equipId = '${rSyEqList.equipId}'")
            } catch (e: Exception) {
                e.printStackTrace()
            }

            if (rSyEqList.sysIdList!=null&&rSyEqList.sysIdList!!.isNotEmpty()){
                for (eqId in rSyEqList.sysIdList!!){
                    val rSyEq = RSyEq()
                    rSyEq.equipId = rSyEqList.equipId
                    rSyEq.sysId = eqId
                    rSyEq.projectId = projectId
                    val replace = replace(rSyEq)
                    if (replace){
                        successList.add(eqId)
                    }else{
                        failureList.add(rSyEq)
                    }
                }
                if (successList.size>0){
                    try {
                        /** 发送消息 */
                        rabbitMqService.sendRel(Opt.projectId!!, "SystemEquip","13",rSyEqList.equipId!!,successList.toJson()!!)
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                    if (failureList.size>0){
                        val sCreateResponse = SCreateResponse<RSyEq>(SResponseType.success)
                        sCreateResponse.message ="失败对象："+failureList.toJson()
                        sCreateResponse
                    }else{
                        val sCreateResponse = SCreateResponse<RSyEq>(SResponseType.success)
                        sCreateResponse
                    }
                }else{
                    SCreateResponse(SResponseType.failure)
                }
            }else{
                SCreateResponse(SResponseType.success)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    }

    /**
     * 创建系统下设备关系
     *
     * @param   rSyEqList   系统和设备的关系对象列表
     */
    fun createSyEq(rSyEqList: RSyEqList): SCreateResponse<RSyEq> {
        return try {

            /** 失败集合 */
            val failureList = ArrayList<RSyEq>()
            if (rSyEqList.equipIdList!=null&&rSyEqList.equipIdList!!.isNotEmpty()){
                /** 成功集合 */
                val successList = ArrayList<Any>()
                for (eqId in rSyEqList.equipIdList!!){
                    val rSyEq = RSyEq()
                    rSyEq.equipId = eqId
                    rSyEq.sysId =  rSyEqList.sysId
                    rSyEq.projectId = Opt.projectId
                    val replace = replace(rSyEq)
                    if (replace){
                        successList.add(eqId)
                    }else{
                        failureList.add(rSyEq)
                    }
                }
                if (successList.size>0){
                    try {
                        /** 发送消息 */
                        rabbitMqService.sendRel(Opt.projectId!!, "SystemEquip","14",rSyEqList.sysId!!,successList.toJson()!!)
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                    val sCreateResponse = SCreateResponse<RSyEq>(SResponseType.success)
                    if (failureList.size>0){
                        sCreateResponse.message ="失败对象："+failureList.toJson()
                    }

                    try {
                        /** 处理关系标记 */
                        val listtype = ArrayList<String>()
                        listtype.add("syeq2bd")
                        listtype.add("sy2fl")
                        listtype.add("sy2sh")
                        listtype.add("sy2sp")
                        for (type in listtype){
                            val entity = EquipmentService.relationTypeProjectservice.select(SFilter.eq("projectId", Opt.projectId!!), SFilter.eq("relationType", type)).entity()
                            if (entity!=null){
                                entity.computationalState = 2
                                EquipmentService.relationTypeProjectservice.update(entity)
                            }
                        }
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }

                    sCreateResponse
                }else{
                    SCreateResponse(SResponseType.failure)
                }
            }else{
                SCreateResponse(SResponseType.success)
            }

        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    }

    /**
     * 删除前的操作
     */
    override fun onDeleteBefore(entity: RSyEq): Boolean {

        return if (entity.sysId.isNullOrEmpty()||entity.equipId.isNullOrEmpty()){
            false
        }else{
            super.onDeleteBefore(entity)
        }
    }

    /**
     * 创建关系
     */
    override fun onCreateSuccess(entityList: ArrayList<RSyEq>) {
        super.onCreateSuccess(entityList)
        try {
            for (rSyEq in entityList){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "SystemEquip","11",rSyEq.sysId!!,rSyEq.equipId!!)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    /**
     * 删除关系
     */
    override fun onDeleteSuccess(entityList: ArrayList<RSyEq>) {
        super.onDeleteSuccess(entityList)
        try {
            for (rSyEq in entityList){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "SystemEquip","12",rSyEq.sysId!!,rSyEq.equipId!!)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

}