/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.rel

import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.wanda.obj.Floor
import com.persagy.server.datacenter.models.entities.rel.RFlInSyList
import com.persagy.server.datacenter.models.entities.rel.RSyInFl
import com.persagy.server.datacenter.models.entities.rel.RSyInFlList
import com.persagy.server.datacenter.services.base.RService
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse

/**
 * 系统所在楼层关系服务
 *
 * @author 付国宝
 */
object RSyInFlService: RService<RSyInFl>(SMybatisDao(RSyInFl::class.java)) {

    /**
     * 创建系统所在楼层关系
     *
     * @param    rSyInFl     系统所在楼层关系对象
     */
    fun create(rSyInFl: RSyInFl): SCreateResponse<RSyInFl> {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            rSyInFl.projectId = projectId
            val floorService = SObjectService(SMybatisDao(Floor::class.java))
            val floorObj = floorService.select(SFilter.eq("id", rSyInFl.floorId!!)).entity()
            if(floorObj != null){
                rSyInFl.buildingId = floorObj.buildingId
            }
            val isInsert = insert(rSyInFl)
            if (isInsert){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "sy2fl","11",rSyInFl.sysId!!,rSyInFl.floorId!!)
                SCreateResponse(SResponseType.success)
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function create()

    /**
     * 根据对象删除
     *
     * @param    rSyInFl    关系对象实例
     * @return
     */
    fun deletes(rSyInFl: RSyInFl): SBaseResponse {
        return try {
            val isDelete = delete(rSyInFl)
            /** 删除成功 */
            if (isDelete) {
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "sy2fl","12",rSyInFl.sysId!!,rSyInFl.floorId!!)
                val sBaseResponse = SBaseResponse(SResponseType.success)
                sBaseResponse.message = rSyInFl.toJson()
                sBaseResponse
            } else {
                SBaseResponse(SResponseType.failure)
            }
        }catch (e: Exception){
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Function deletes()

    /**
     * 创建系统所在楼层关系
     *
     * @param    rSyInFlList    系统和楼层关系的对象列表
     */
    fun createSy(rSyInFlList: RSyInFlList): SCreateResponse<RSyInFl>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("sysId = '${rSyInFlList.sysId}'")
            val projectId = SPageContext.getHeader("projectId")
            val floorService = SObjectService(SMybatisDao(Floor::class.java))
            if (!rSyInFlList.floorIdList.isNullOrEmpty()){
                for (floorId in rSyInFlList.floorIdList!!){
                    val rSyInFl = RSyInFl()
                    rSyInFl.sysId = rSyInFlList.sysId
                    rSyInFl.floorId = floorId
                    rSyInFl.projectId = projectId

                    val floorObj = floorService.select(SFilter.eq("id", rSyInFl.floorId!!)).entity()
                    if(floorObj != null){
                        rSyInFl.buildingId = floorObj.buildingId
                    }

                    val replace = replace(rSyInFl)
                    if (replace){
                        successList.add(floorId)
                    }else{
                        failureList.add(rSyInFl)
                    }
                }
                if (successList.size>0){
                    /** 发送消息 */
                    rabbitMqService.sendRel(Opt.projectId!!, "sy2fl","13",rSyInFlList.sysId!!,successList.toJson())
                    val sCreateResponse = SCreateResponse<RSyInFl>(SResponseType.success)
                    if (failureList.size>0){
                        sCreateResponse.message ="失败对象："+failureList.toJson()
                    }
                    sCreateResponse
                }else{
                    SCreateResponse(SResponseType.failure)
                }
            }else{
                val sCreateResponse = SCreateResponse<RSyInFl>(SResponseType.success)
                sCreateResponse
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createSy()


    /**
     * 创建系统所在楼层关系
     *
     * @param    rSyInFlList    系统和楼层关系的对象列表
     */
    fun createSyFl(rSyInFlList: RSyInFlList): SCreateResponse<RSyInFl>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
//            delete("sysID = '${rSyInFlList.sysId}'")
            val projectId = SPageContext.getHeader("projectId")
            val floorService = SObjectService(SMybatisDao(Floor::class.java))
            for (floorId in rSyInFlList.floorIdList!!){
                val rSyInFl = RSyInFl()
                rSyInFl.sysId = rSyInFlList.sysId
                rSyInFl.floorId = floorId
                rSyInFl.projectId = projectId

                val floorObj = floorService.select(SFilter.eq("id", rSyInFl.floorId!!)).entity()
                if(floorObj != null){
                    rSyInFl.buildingId = floorObj.buildingId
                }

                val replace = replace(rSyInFl)
                if (replace){
                    successList.add(floorId)
                }else{
                    failureList.add(rSyInFl)
                }
            }
            if (successList.size>0){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "sy2fl","14",rSyInFlList.sysId!!,successList.toJson())
                val sCreateResponse = SCreateResponse<RSyInFl>(SResponseType.success)
                if (failureList.size>0){
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                }
                sCreateResponse
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createSy()


    /**
     * 创建所在楼层的系统关系
     *
     * @param    rFlInSyList     系统和楼层关系的对象列表
     */
    fun createsFl(rFlInSyList: RFlInSyList): SCreateResponse<RSyInFl>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("floorId = '${rFlInSyList.floorId}'")
            val projectId = SPageContext.getHeader("projectId")
            val floorService = SObjectService(SMybatisDao(Floor::class.java))
            for (sysId in rFlInSyList.sysIdList!!){
                val rSyInBd = RSyInFl()
                rSyInBd.floorId = rFlInSyList.floorId
                rSyInBd.sysId = sysId
                rSyInBd.projectId = projectId

                val floorObj = floorService.select(SFilter.eq("id", rFlInSyList.floorId!!)).entity()
                if(floorObj != null){
                    rSyInBd.buildingId = floorObj.buildingId
                }

                val replace = replace(rSyInBd)
                if (replace){
                    successList.add(sysId)
                }else{
                    failureList.add(rSyInBd)
                }
            }
            if (successList.size>0){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "sy2fl","13",rFlInSyList.floorId!!,successList.toJson())
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<RSyInFl>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<RSyInFl>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsFl()

} // Object RSyInFlService