/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.rel

import com.persagy.base.extensions.toJson
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.models.entities.rel.RShInSyList
import com.persagy.server.datacenter.models.entities.rel.RSyInSh
import com.persagy.server.datacenter.models.entities.rel.RSyInShList
import com.persagy.server.datacenter.services.base.RService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse

/**
 * 系统所在竖井关系服务
 *
 * @author 付国宝
 */
object RSyInShService: RService<RSyInSh>(SMybatisDao(RSyInSh::class.java)) {

    /**
     * 创建系统所在竖井关系
     *
     * @param    rSyInSh    系统所在竖井关系对象
     */
    fun create(rSyInSh: RSyInSh): SCreateResponse<RSyInSh> {
        return try {

            rSyInSh.projectId = Opt.projectId
            val isInsert = insert(rSyInSh)
            if (isInsert){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sh","11",rSyInSh.sysId!!,rSyInSh.shaftId!!)
                SCreateResponse(SResponseType.success)
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function create()

    /**
     * 根据对象删除
     *
     * @param    rSyInSh    关系对象实例
     * @return
     */
    fun deletes(rSyInSh: RSyInSh): SBaseResponse {
        return try {
            val isDelete = delete(rSyInSh)
            /** 删除成功 */
            if (isDelete) {
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sh","12",rSyInSh.sysId!!,rSyInSh.shaftId!!)
                val sBaseResponse = SBaseResponse(SResponseType.success)
                sBaseResponse.message = rSyInSh.toJson()
                sBaseResponse
            } else {
                SBaseResponse(SResponseType.failure)
            }
        }catch (e: Exception){
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Function deletes()

    /**
     * 创建系统所在竖井关系
     *
     * @param    rSyInShList    系统和竖井关系的对象列表
     */
    fun createSy(rSyInShList: RSyInShList): SCreateResponse<RSyInSh>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("sysId = '${rSyInShList.sysId}'")
            val projectId = SPageContext.getHeader("projectId")
            for (shaftId in rSyInShList.shaftIdList!!){
                val rSyInSh = RSyInSh()
                rSyInSh.sysId = rSyInShList.sysId
                rSyInSh.shaftId = shaftId
                rSyInSh.projectId = projectId
                val replace = replace(rSyInSh)
                if (replace){
                    successList.add(shaftId)
                }else{
                    failureList.add(rSyInSh)
                }
            }
            if (successList.size>0){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sh","13",rSyInShList.sysId!!,successList.toJson())
                val sCreateResponse = SCreateResponse<RSyInSh>(SResponseType.success)
                if (failureList.size>0){
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                }
                sCreateResponse
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createSy()

    /**
    * 创建所在竖井的系统关系
    *
    * @param    rShInSyList    系统和竖井关系的对象列表
    */
    fun createsSh(rShInSyList: RShInSyList): SCreateResponse<RSyInSh>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("shaftId = '${rShInSyList.shaftId}'")
            val projectId = SPageContext.getHeader("projectId")
            for (sysId in rShInSyList.sysIdList!!){
                val rSyInSh = RSyInSh()
                rSyInSh.shaftId = rShInSyList.shaftId
                rSyInSh.sysId = sysId
                rSyInSh.projectId = projectId
                val replace = replace(rSyInSh)
                if (replace){
                    successList.add(sysId)
                }else{
                    failureList.add(rSyInSh)
                }
            }
            if (successList.size>0){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sh","13",rShInSyList.shaftId!!,successList.toJson())
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<RSyInSh>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<RSyInSh>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsSh()

    /**
     * 创建所在竖井的系统关系
     *
     * @param    rShInSyList    系统和竖井关系的对象列表
     */
    fun createsShSys(rShInSyList: RShInSyList): SCreateResponse<RSyInSh>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            val projectId = SPageContext.getHeader("projectId")
            for (sysId in rShInSyList.sysIdList!!){
                val rSyInSh = RSyInSh()
                rSyInSh.shaftId = rShInSyList.shaftId
                rSyInSh.sysId = sysId
                rSyInSh.projectId = projectId
                val replace = replace(rSyInSh)
                if (replace){
                    successList.add(sysId)
                }else{
                    failureList.add(rSyInSh)
                }
            }
            if (successList.size>0){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sh","14",rShInSyList.shaftId!!,successList.toJson())
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<RSyInSh>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<RSyInSh>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsSh()

    /**
     * 创建系统所在竖井关系
     *
     * @param    rSyInShList    系统和竖井关系的对象列表
     */
    fun createSyShs(rSyInShList: RSyInShList): SCreateResponse<RSyInSh>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()

            for (shaftId in rSyInShList.shaftIdList!!){
                val rSyInSh = RSyInSh()
                rSyInSh.sysId = rSyInShList.sysId
                rSyInSh.shaftId = shaftId
                rSyInSh.projectId = Opt.projectId
                val replace = replace(rSyInSh)
                if (replace){
                    successList.add(shaftId)
                }else{
                    failureList.add(rSyInSh)
                }
            }
            if (successList.size>0){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sh","14",rSyInShList.sysId!!,successList.toJson())
                val sCreateResponse = SCreateResponse<RSyInSh>(SResponseType.success)
                if (failureList.size>0){
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                }
                sCreateResponse
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createSyShs()

    /**
     * 创建关系
     */
    override fun onCreateSuccess(entityList: ArrayList<RSyInSh>) {
        super.onCreateSuccess(entityList)
        for (rSyInSh in entityList){
            /** 发送消息 */
            rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sh","11",rSyInSh.sysId!!,rSyInSh.shaftId!!)
        }

    } // Function onCreateSuccess()

    /**
     * 删除关系
     */
    override fun onDeleteSuccess(entityList: ArrayList<RSyInSh>) {
        super.onDeleteSuccess(entityList)
        val projectId = SPageContext.getHeader("projectId")
        for (rSyInSh in entityList){
            /** 发送消息 */
            rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sh","12",rSyInSh.sysId!!,rSyInSh.shaftId!!)
        }
    } // Function onDeleteSuccess()

}