/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.services.rel

import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.models.entities.rel.RSpInSyBaseList
import com.persagy.server.datacenter.models.entities.rel.RSyInSpBase
import com.persagy.server.datacenter.models.entities.rel.RSyInSpBaseList
import com.persagy.server.datacenter.services.base.RService
import com.persagy.server.datacenter.services.objects.EquipmentService
import com.persagy.server.datacenter.services.objects.ZoneSpaceBaseService
import com.persagy.server.datacenter.wanda.rel.RSy2Sp
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse

/**
 * 系统所在业务空间关系服务
 *
 * @author 付国宝
 */
object RSyInSpBaseService: RService<RSyInSpBase>(SMybatisDao(RSyInSpBase::class.java)) {

    private var rSy2SpService = SObjectService(SMybatisDao(RSy2Sp::class.java))
    /**
     * 创建系统所在业务空间关系
     *
     * @param   rSyInSpBaseList    系统所在业务空间关系对象
     */
    fun create(rSyInSpBaseList: SCreateRequest<RSyInSpBase>): SCreateResponse<RSyInSpBase> {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            for (rSyInSpBase in rSyInSpBaseList.content){
                val rSy2Sp = RSy2Sp()
                rSy2Sp.id1 = rSyInSpBase.sysId
                rSy2Sp.id2 = rSyInSpBase.spaceId
                rSy2Sp.sign = 1
                rSy2Sp.zoneType = rSyInSpBase.objectType
                rSy2Sp.type = "sy2sp"
                rSy2Sp.projectId = projectId
                val isInsert = rSy2SpService.insert(rSy2Sp)
                if (isInsert){
                    /** 发送消息 */
                    rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sp","11",rSyInSpBase.sysId!!,rSyInSpBase.spaceId!!)
                }
            }
            try {
                /** 处理关系标记 */
                val listType = ArrayList<String>()
                listType.add("sh2bd")
                listType.add("eq2sh")
                for (type in listType){
                    val entity = EquipmentService.relationTypeProjectservice.select(SFilter.eq("projectId", projectId!!), SFilter.eq("relationType", type)).entity()
                    if (entity!=null){
                        entity.computationalState = 2
                        EquipmentService.relationTypeProjectservice.update(entity)
                    }
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            val sCreateResponse = SCreateResponse<RSyInSpBase>(SResponseType.success)
            sCreateResponse
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function create()

    /**
     * 根据对象删除
     *
     * @param   rSyInSpBaseList    关系对象实例
     * @return
     */
    fun deletes(type: String, rSyInSpBaseList: ArrayList<RSyInSpBase>): SBaseResponse {
        return try {

            val projectId = SPageContext.getHeader("projectId")
            for (rSyInSpBase in rSyInSpBaseList){
                val rSy2Sp = RSy2Sp()
                rSy2Sp.id2 = rSyInSpBase.spaceId
                rSy2Sp.id1 = rSyInSpBase.sysId
                rSy2Sp.projectId = projectId
                rSy2Sp.type = "sy2sp"
                rSy2Sp.zoneType = type
                val isDelete = rSy2SpService.delete(rSy2Sp)
                if (isDelete) {
                    /** 发送消息 */
                    rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sp","12",rSyInSpBase.sysId!!,rSyInSpBase.spaceId!!)
                }
            }
            try {
                /** 处理关系标记 */
                val listType = ArrayList<String>()
                listType.add("sh2bd")
                listType.add("eq2sh")
                for (type in listType){
                    val entity = EquipmentService.relationTypeProjectservice.select(SFilter.eq("projectId", projectId!!), SFilter.eq("relationType", type)).entity()
                    if (entity!=null){
                        entity.computationalState = 2
                        EquipmentService.relationTypeProjectservice.update(entity)
                    }
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }

            val sBaseResponse = SBaseResponse(SResponseType.success)
            sBaseResponse
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Function deletes()

    /**
     * 创建系统所在业务空间关系
     *
     * @param    rSyInSpBaseList    系统所在业务空间关系对象列表
     */
    fun createSy(rSyInSpBaseList: RSyInSpBaseList): SCreateResponse<RSyInSpBase> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            rSy2SpService.delete("id1 = '${rSyInSpBaseList.sysId}'")
            val projectId = SPageContext.getHeader("projectId")
            for (spaceId in rSyInSpBaseList.spaceIdList!!){
                val entity = ZoneSpaceBaseService.select(SFilter.eq("projectId", projectId!!), SFilter.eq("id", spaceId)).entity()
                val rSy2Sp = RSy2Sp()
                rSy2Sp.id1 = rSyInSpBaseList.sysId
                rSy2Sp.id2 = spaceId
                rSy2Sp.sign = 1
                rSy2Sp.zoneType = entity!!.classCode
                rSy2Sp.type = "sy2sp"
                rSy2Sp.projectId = projectId
                val replace = rSy2SpService.replace(rSy2Sp)
                if (replace){
                    successList.add(spaceId)
                }else{
                    failureList.add(rSy2Sp)
                }
            }
            if (successList.size>0){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sp","13",rSyInSpBaseList.sysId!!,successList.toJson())
                val sCreateResponse = SCreateResponse<RSyInSpBase>(SResponseType.success)
                if (failureList.size>0){
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                }
                sCreateResponse
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createSy()

    /**
     * 创建所在业务空间的系统关系
     *
     * @param    rSyInSpBaseList    所在业务空间的系统关系对象列表
     */
    fun createsSp(rSyInSpBaseList: RSpInSyBaseList): SCreateResponse<RSyInSpBase> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            val projectId = SPageContext.getHeader("projectId")
            delete("id2 = '${rSyInSpBaseList.spaceId}'")
            val entity = ZoneSpaceBaseService.select(SFilter.eq("projectId", projectId!!), SFilter.eq("id", rSyInSpBaseList.spaceId!!)).entity()

            for (sysId in rSyInSpBaseList.sysIdList!!){
                val rSy2Sp = RSy2Sp()
                rSy2Sp.id2 = rSyInSpBaseList.spaceId
                rSy2Sp.id1 = sysId
                rSy2Sp.projectId = projectId
                rSy2Sp.sign = 1
                rSy2Sp.zoneType = entity!!.classCode
                rSy2Sp.type = "sy2sp"
                val replace = rSy2SpService.replace(rSy2Sp)
                if (replace){
                    successList.add(sysId)
                }else{
                    failureList.add(rSy2Sp)
                }
            }
            if (successList.size>0){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sp","13",rSyInSpBaseList.spaceId!!,successList.toJson())
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<RSyInSpBase>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<RSyInSpBase>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsSp()

    /**
     * 创建所在业务空间的系统关系
     *
     * @param    rSyInSpBaseList    所在业务空间的系统关系对象列表
     */
    fun createsSpSys(type: String, rSyInSpBase: RSpInSyBaseList): SCreateResponse<RSpInSyBaseList> {
        return try {


            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            val projectId = SPageContext.getHeader("projectId")
            for (sysId in rSyInSpBase.sysIdList!!){
                val rSy2Sp = RSy2Sp()
                rSy2Sp.id2 = rSyInSpBase.spaceId
                rSy2Sp.id1 = sysId
                rSy2Sp.projectId = projectId
                rSy2Sp.sign = 1
                rSy2Sp.zoneType = type
                rSy2Sp.type = "sy2sp"
                val replace = rSy2SpService.replace(rSy2Sp)
                if (replace){
                    successList.add(sysId)
                }else{
                    failureList.add(rSy2Sp)
                }
            }
            if (successList.size>0){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sp","14",rSyInSpBase.spaceId!!,successList.toJson())
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<RSpInSyBaseList>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<RSpInSyBaseList>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }

        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsSp()


    /**
     * 创建系统所在业务空间关系
     *
     * @param    rSyInSpBaseList    系统所在业务空间关系对象列表
     */
    fun createSySps(type: String, rSyInSpBase: RSyInSpBaseList): SCreateResponse<RSyInSpBaseList> {
        return try {


            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            val projectId = SPageContext.getHeader("projectId")
            for (spaceId in rSyInSpBase.spaceIdList!!){
                val rSy2Sp = RSy2Sp()
                rSy2Sp.id1 = rSyInSpBase.sysId
                rSy2Sp.id2 = spaceId
                rSy2Sp.projectId = projectId
                rSy2Sp.sign = 1
                rSy2Sp.zoneType = type
                rSy2Sp.type = "sy2sp"
                val replace = rSy2SpService.replace(rSy2Sp)
                if (replace){
                    successList.add(spaceId)
                }else{
                    failureList.add(rSy2Sp)
                }
            }
            if (successList.size>0){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "Sy2Sp","14",rSyInSpBase.sysId!!,successList.toJson())
                val sCreateResponse = SCreateResponse<RSyInSpBaseList>(SResponseType.success)
                if (failureList.size>0){
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                }
                sCreateResponse
            }else{
                SCreateResponse(SResponseType.failure)
            }

        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createSySps()




}