/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.rel

import com.persagy.base.extensions.toJson
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.models.entities.rel.RSpInTnBaseList
import com.persagy.server.datacenter.models.entities.rel.RTn2Sp
import com.persagy.server.datacenter.models.entities.rel.RTnInSpBase
import com.persagy.server.datacenter.models.entities.rel.RTnInSpBaseList
import com.persagy.server.datacenter.services.base.RService
import com.persagy.server.synchronize.MessageHandler
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.utils.SSpringContextUtil

/**
 * 租户所租用的业务空间关系服务
 *
 * @author 付国宝
 */
object RTnInSpBaseService: RService<RTnInSpBase>(SMybatisDao(RTnInSpBase::class.java)) {


    /** 租户和业务空间的关系 */
    var rTn2SpService = SObjectService(SMybatisDao(RTn2Sp::class.java))

    /** MQ消息对象 */
    private val messageHandler by lazy {
        SSpringContextUtil.getBean(MessageHandler::class.java) as MessageHandler
    }

    /**
     * 创建租户所租用的业务空间关系
     *
     * @param    rTnInSpBase    租户所租用的业务空间关系对象
     */
    fun create(type: String,rTnInSpBaseList: SCreateRequest<RTnInSpBase>): SCreateResponse<RTnInSpBase> {
        return try {

            if (!rTnInSpBaseList.content.isNullOrEmpty()){
                for (rTnInSpBase in rTnInSpBaseList.content){
                    val rTn2Sp = RTn2Sp()
                    rTn2Sp.id2 = rTnInSpBase.spaceId
                    rTn2Sp.id1 = rTnInSpBase.tenantId
                    rTn2Sp.projectId = Opt.projectId
                    rTn2Sp.sign = 1
                    rTn2Sp.zoneType = type
                    rTn2Sp.type = "tn2sp"
                    val insert = rTn2SpService.insert(rTn2Sp)
                    if (insert){
                        try {
                            /** 发送消息 */
                            rabbitMqService.sendRel(Opt.projectId!!, "tn2sp","11", rTnInSpBase.tenantId!!,rTnInSpBase.spaceId!!)
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                    }
                }
            }
            SCreateResponse(SResponseType.success)

        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function create()

    /**
     * 根据对象删除
     *
     * @param   rTnInSpBase    关系对象实例
     * @return
     */
    fun deletes(type: String,rTnInSpBaseList: ArrayList<RTnInSpBase>): SBaseResponse {
        return try {
            for (rTnInSpBase in rTnInSpBaseList) {
                val rTn2Sp = RTn2Sp()
                rTn2Sp.id1 = rTnInSpBase.tenantId
                rTn2Sp.id2 = rTnInSpBase.spaceId
                rTn2Sp.zoneType = type
                rTn2Sp.projectId = Opt.projectId
                rTn2Sp.type = "tn2sp"
                val delete = rTn2SpService.delete(rTn2Sp)
                if (delete){
                    /** 发送消息 */
                    rabbitMqService.sendRel(Opt.projectId!!, "tn2sp","12", rTnInSpBase.tenantId!!,rTnInSpBase.spaceId!!)
                }
            }
            val sBaseResponse = SBaseResponse(SResponseType.success)
            sBaseResponse
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    }

    /**
     * 创建租户所租用的业务空间关系
     *
     * @param   rTnInSpBaseList    租户所租用的业务空间关系对象列表
     */
    fun createTn(type: String, rTnInSpBaseList: RTnInSpBaseList): SCreateResponse<RTnInSpBase> {
        return try {

            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            rTn2SpService.delete("id1 = '${rTnInSpBaseList.tenantId}'")
            val projectId = SPageContext.getHeader("projectId")
            if (!rTnInSpBaseList.spaceIdList.isNullOrEmpty()) {
                for (spaceId in rTnInSpBaseList.spaceIdList!!){
                    val rTn2Sp = RTn2Sp()
                    rTn2Sp.id1 = rTnInSpBaseList.tenantId
                    rTn2Sp.id2 = spaceId
                    rTn2Sp.projectId = projectId
                    rTn2Sp.sign = 1
                    rTn2Sp.zoneType = type
                    rTn2Sp.type = "tn2sp"
                    val replace = rTn2SpService.replace(rTn2Sp)
                    if (replace){
                        successList.add(spaceId)
                    }else{
                        failureList.add(rTn2Sp)
                    }
                }
                if (successList.size>0){
                    /** 发送消息 */
                    rabbitMqService.sendRel(Opt.projectId!!, "tn2sp","13", rTnInSpBaseList.tenantId!!,successList.toJson())
                    val sCreateResponse = SCreateResponse<RTnInSpBase>(SResponseType.success)
                    if (failureList.size>0){
                        sCreateResponse.message ="失败对象："+failureList.toJson()
                    }
                    sCreateResponse
                }else{
                    SCreateResponse(SResponseType.failure)
                }
            }else {
                SCreateResponse(SResponseType.success)
            }


        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createTn()

    /**
     * 创建所租用的业务空间的租户关系
     *
     * @param     rTnInSpBaseList       所租用的业务空间的租户关系对象列表
     */
    fun createsSp(rTnInSpBaseList: RSpInTnBaseList): SCreateResponse<RTnInSpBase> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            rTn2SpService.delete("id2 = '${rTnInSpBaseList.spaceId}'")
            val projectId = SPageContext.getHeader("projectId")
            for (tenantId in rTnInSpBaseList.tenantIdList!!){
                val rTn2Sp = RTn2Sp()
                rTn2Sp.id2 = rTnInSpBaseList.spaceId
                rTn2Sp.id1 = tenantId
                rTn2Sp.projectId = projectId
                rTn2Sp.sign = 1
                rTn2Sp.zoneType = rTnInSpBaseList.zoneType
                rTn2Sp.type = "tn2sp"
                val replace = rTn2SpService.replace(rTn2Sp)
                if (replace){
                    successList.add(tenantId)
                }else{
                    failureList.add(rTn2Sp)
                }
            }
            if (successList.size>0){
                /** 发送消息 */
                rabbitMqService.sendRel(Opt.projectId!!, "tn2sp","13", rTnInSpBaseList.spaceId!!,successList.toJson())
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<RTnInSpBase>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<RTnInSpBase>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    }

}