/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.services.rel

import com.persagy.database.SFilter
import com.persagy.server.datacenter.services.graphtype.RelationTypeProjectService
import com.persagy.server.datacenter.services.objects.EquipmentService
import com.persagy.server.datacenter.wanda.obj.Equipment
import com.persagy.server.mappers.ICalcSpaceMapper
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SQueryResponse
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import org.springframework.web.bind.annotation.RequestParam
import java.util.stream.Collectors

/**
 * 关系统计
 *
 */
@Service
open class RelStatisticService {
    /** 实例化 mapper 对象 */
//    @Autowired
//    lateinit var mapper: ICalcSpaceMapper
    /**
     * @Description: 查询关系相关的数据类型
     * @Param:
     * @Return:
     */
    fun relToType(zoneType: String?,graphCode: String,type: String,relType: String,status: Int): SQueryResponse<HashMap<String,String?>> {
        val projectId = SPageContext.getHeader("ProjectId")
        if(projectId == null){
            return SQueryResponse(SResponseType.failure,"项目ID不能为空")
        }
        val results = ArrayList<HashMap<String,String?>>()
        val defClasses  = CommonObjectService.dictClassService.select(SFilter.eq("type","group"), SFilter.eq("groupCode","WD")).exec()
        val eqMap = HashMap<String,String>(512)
//        val spMap = HashMap<String,String>()
        for (defClass in defClasses) {
//            if(defClass.objType == "equipment"){
//                eqMap[defClass.code!!] = defClass.name!!
//            }else if(defClass.objType == "space"){
//                spMap[defClass.code!!] = defClass.name!!
//            }
            eqMap[defClass.code!!] = defClass.name!!
        }
        when (relType) {
            RelationType.Eq2Bd -> {
                val bdEquipments = CommonObjectService.equipmentService.select(SFilter.eq("projectId",projectId), SFilter.not(SFilter.isNull("buildingId"))).projection(listOf("classCode")).distinct().exec()
                for (bdEquipment in bdEquipments) {
                    val classCode = bdEquipment.classCode!!
                    val item = HashMap<String,String?>()
                    item["classCode"] = classCode
                    item["className"] = eqMap[classCode]
                    results.add(item)
                }
            }
            RelationType.Eq2Fl -> {
                val flEquipments = CommonObjectService.equipmentService.select(SFilter.eq("projectId",projectId), SFilter.not(SFilter.isNull("floorId"))).projection(listOf("classCode")).distinct().exec()
                for (flEquipment in flEquipments) {
                    val classCode = flEquipment.classCode!!
                    val item = HashMap<String,String?>()
                    item["classCode"] = classCode
                    item["className"] = eqMap[classCode]
                    results.add(item)
                }

            }
            RelationType.Eq2Sp -> {
                val rEq2Sps = CommonObjectService.rEq2SpService.select(SFilter.eq("projectId",projectId),SFilter.eq("type",type), SFilter.eq("graphCode",graphCode), SFilter.eq("zoneType",zoneType!!)).exec()
                val equipIds = rEq2Sps.stream().map { r -> r.id1!! }.collect(Collectors.toList())
                if(equipIds.size > 0){
                    val equipments = EquipmentService.select(SFilter.eq("projectId",projectId), SFilter.inList("id",equipIds)).projection(listOf("classCode")).distinct().exec()
                    for (equipment in equipments) {
                        val classCode = equipment.classCode!!
                        val item = HashMap<String,String?>()
                        item["classCode"] = classCode
                        item["className"] = eqMap[classCode]
                        results.add(item)
                    }
                }

            }
            RelationType.Sy2Eq -> {
                val rEq2Sys = CommonObjectService.rEq2SyTwoService.select(SFilter.eq("projectId",projectId)).exec()
                if(status == 1){
                    for (rEq2Sy in rEq2Sys) {
                        val classCode = rEq2Sy.classCode1!!
                        val item = HashMap<String,String?>()
                        item["classCode"] = classCode
                        item["className"] = eqMap[classCode]
                        results.add(item)
                    }
                }else if(status == 2){
                    for (rEq2Sy in rEq2Sys) {
                        val classCode = rEq2Sy.classCode2!!
                        val item = HashMap<String,String?>()
                        item["classCode"] = classCode
                        item["className"] = eqMap[classCode]
                        results.add(item)
                    }


                }


            }
            RelationType.Bd2Sp -> {
                val bdZoneSpaces = CommonObjectService.zoneSpaceService.select(SFilter.eq("projectId",projectId), SFilter.not(SFilter.isNull("buildingId"))).projection(listOf("classCode")).distinct().exec()
                for (bdZoneSpace in bdZoneSpaces) {
                    val classCode = bdZoneSpace.classCode!!
                    val item = HashMap<String,String?>()
                    item["classCode"] = classCode
                    item["className"] = eqMap[classCode]
                    results.add(item)
                }
            }
            RelationType.Fl2Sp -> {
                val flZoneSpaces = CommonObjectService.zoneSpaceService.select(SFilter.eq("projectId",projectId), SFilter.not(SFilter.isNull("floorId"))).projection(listOf("classCode")).distinct().exec()
                for(flZoneSpace in flZoneSpaces){
                    val classCode = flZoneSpace.classCode!!
                    val item = HashMap<String,String?>()
                    item["classCode"] = classCode
                    item["className"] = eqMap[classCode]
                    results.add(item)
                }
            }
            RelationType.Sp2Sp -> {
                val rSp2Sps = CommonObjectService.sp2SpService.select(SFilter.eq("projectId",projectId),SFilter.eq("type",type), SFilter.eq("graphCode",graphCode)).projection(listOf("zoneType")).distinct().exec()
                for(rSp2Sp in rSp2Sps){
                    val classCode = rSp2Sp.zoneType!!
                    val item = HashMap<String,String?>()
                    item["classCode"] = classCode
                    item["className"] = eqMap[classCode]
                    results.add(item)
                }

            }

        }
        val response = SQueryResponse<HashMap<String,String?>>()
        response.content = results
        return response
        
    }

    fun updateCalcTime(graphCode: String,relType: String, computingTime: String): SBaseResponse {
        val projectId = SPageContext.getHeader("ProjectId") ?: return SBaseResponse(SResponseType.failure,"项目ID不能为空")
        val relationTypeProject = RelationTypeProjectService.select(SFilter.eq("projectId",projectId), SFilter.eq("relationType",relType), SFilter.eq("graphicType",graphCode)).entity()
        if(relationTypeProject != null){
            relationTypeProject.computingTime = computingTime
            RelationTypeProjectService.update(relationTypeProject)
            return SBaseResponse(SResponseType.success,"")
        }
        return SBaseResponse(SResponseType.failure,"关系不存在")

    }


} // Class CalcSpaceService