/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.wanda.controllers.obj

import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.server.datacenter.wanda.obj.Equipment
import com.persagy.server.datacenter.services.objects.ComponentService
import com.persagy.service.SPageContext
import com.persagy.service.models.requests.SCountRequest
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCountResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 * 部件接口
 *
 * @author 张维新
 */
@Tag(name = "005、部件")
@RestController
@RequestMapping("/object/component")
open class ComponentController {
    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(ComponentController::class.java)
    } // Companion object

    /**
     * 批量创建部件
     *
     * @param   request     部件对象列表
     * @return                    创建结果信息
     */
   @Operation(summary = "创建部件信息", description = "")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody request: SCreateRequest<Equipment>): SCreateResponse<Equipment> {
        logger.debug("equip =${request.toJson()}")
        return ComponentService.createList(request)
    } // Function create()

    /**
     * 根据id删除部件
     *
     * @param   idList      id数组
     * @return              删除的结果信息
     */
   @Operation(summary = "根据id删除部件信息", description = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<Equipment>): SBaseResponse {
        return ComponentService.deleteByKeysList(idList)
    } // Function delete()

    /**
     * 更新部件信息
     *
     * @param   request        更新的内容对象
     * @return                  更新的结果
     */
   @Operation(summary = "更新部件信息", description = "")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<Equipment>): SBaseResponse {
        logger.debug("equipUpdate =${request.toJson()}")
        return ComponentService.updateList(request)
    } // Function update()

    /**
     * 查询部件信息
     *
     * @param   request     查询信息条件
     * @return              查询结果
     */
   @Operation(summary = "查询部件信息", description = "")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<Equipment> {
        logger.debug("equipQuery =${request.toJson()}")
        val withFilterList = ArrayList<SFilter>()
        withFilterList.add(SFilter.eq("projectId", SPageContext.getHeader("projectId").toString()))
        withFilterList.add(SFilter.eq("objectType", "component"))
        return ComponentService.pageQuery(request,withFilterList)
    } // Function query()

    /**
     * 根据条件查询统计数量
     */
   @Operation(summary = "根据条件查询统计数量", description = "例子： 已经关联岗位的资产数量（\"Filters\": \" not EquipId isNull\"），未关联岗位的资产数量（\"Filters\": \" EquipId isNull\"）")
    @PostMapping(value = ["/count"])
    fun count(@RequestBody request: SCountRequest): SCountResponse {
        val withFilterList = ArrayList<SFilter>()
        withFilterList.add(SFilter.eq("projectId", SPageContext.getHeader("projectId").toString()))
        withFilterList.add(SFilter.eq("objectType", "component"))
        return ComponentService.count(request,withFilterList)
    } // Function count()

} // Class EquipController