/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.wanda.controllers.obj

import com.persagy.database.SFilter
import com.persagy.server.Opt
import com.persagy.server.datacenter.services.graphtype.RelationTypeProjectService
import com.persagy.server.datacenter.services.objects.EquipmentService
import com.persagy.server.datacenter.wanda.obj.Equipment
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SCountRequest
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCountResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.springframework.http.MediaType
import org.springframework.web.bind.annotation.*
import org.springframework.web.multipart.MultipartFile
import javax.servlet.http.HttpServletResponse

/**
 * 设备接口
 *
 * @author 张维新
 */
@Tag(name = "004、设备")
@RestController
@RequestMapping("/object/equip")
open class EquipController {

    /**
     * 创建设备
     *
     * @param   request         设备对象
     * @return                  创建结果信息
     */
    @Operation(summary = "创建设备信息", description = "")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody request: SCreateRequest<Equipment>): SCreateResponse<Equipment> {
        return EquipmentService.createList(request)
    }

    /**
     * 根据id删除设备
     *
     * @param   idList      id数组
     * @return              删除的结果信息
     */
    @Operation(summary = "根据id删除设备信息", description = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<Equipment>): SBaseResponse {
        return EquipmentService.deleteByKeysList(idList)
    }

    /**
     * 更新设备信息
     *
     * @param   request        更新的内容对象
     * @return                  更新的结果
     */
    @Operation(summary = "更新设备信息", description = "")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<Equipment>): SBaseResponse {
        return EquipmentService.updateList(request)
    }

    /**
     * 查询设备信息
     *
     * @param   request     查询信息条件
     * @return              查询结果
     */
    @Operation(summary = "查询设备信息", description = "")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<Equipment> {
        return EquipmentService.pageQuery(request, arrayListOf(SFilter.eq("projectId", Opt.projectId!!)))
    }

    /**
     * 根据条件查询统计数量
     */
    @Operation(summary = "根据条件查询统计数量", description = "")
    @PostMapping(value = ["/count"])
    fun count(@RequestBody request: SCountRequest): SCountResponse {
        return EquipmentService.count(request, arrayListOf(SFilter.eq("projectId", Opt.projectId!!)))
    }


    /**
     * 帮助模板下载
     */
    @Operation(summary="帮助模板下载", description ="")
    @GetMapping(value = ["/downloads"])
    fun  downloads(response: HttpServletResponse){
        RelationTypeProjectService.downloadsService.downloadsEquip(response)
    }

    /**
     * 导出设备
     */
    @Operation(summary="导出设备*", description="")
    @GetMapping(value = ["/export"])
    fun exportExcel( response: HttpServletResponse, @RequestParam projectId: String,@RequestParam classCode: String) {
        EquipmentService.exportExcel(response, projectId,classCode)
    } // Fun exportExcel

    /**
     * 导入excel
     *
     * @param   file            导入的excel文件
     * @return  导入状态
     */
    @Operation(summary = "导入excel", description = "")
    @PostMapping("/import",consumes = [MediaType.MULTIPART_FORM_DATA_VALUE])
    fun importExcel(@RequestPart("file") file: MultipartFile, @RequestParam projectId: String, @RequestParam classCode: String) : SBaseResponse {
       return EquipmentService.importExcel(file,projectId,classCode)
    } // Fun importExcel()

}