/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.wanda.controllers.obj

import com.persagy.database.SFilter
import com.persagy.server.datacenter.wanda.obj.Project
import com.persagy.server.services.objects.ProjectService
import com.persagy.service.SPageContext
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 * 项目接口
 *
 * @author 张维新
 */
@RestController
@RequestMapping("/object/project")
@Tag(name = "001、项目")
//@ApiIgnore
private open class ProjectController {
    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(ProjectController::class.java)
    } // Companion object

    /**
     * 创建项目
     *
     * @param  request      对象列表
     * @return              创建结果信息
     */
    @Operation(summary = "创建项目", description = "")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody request: SCreateRequest<Project>): SCreateResponse<Project> {
        return ProjectService.createList(request)
    }

//    /**
//     * 根据id删除项目
//     *
//     * @param   idList      id数组
//     * @return              删除的结果信息
//     */
//    @Operation(summary = "根据id删除项目", description = "")
//    @PostMapping(value = ["/delete"])
//    fun delete(@RequestBody idList: ArrayList<String>): SDeleteResponse {
//        return ProjectService.delete(idList,messageHandler)
//    } // Function delete()

    /**
     * 更新项目信息
     *
     * @param   request     更新的内容对象
     * @return              更新的结果
     */
    @Operation(summary = "更新项目信息", description = "")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<Project>): SBaseResponse {
        return ProjectService.updateList(request)
    }

    /**
     * 查询项目信息
     *
     * @param   request     查询信息条件
     * @return              查询结果
     */
    @Operation(summary = "查询项目信息", description = "")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<Project> {
        val projectId = SPageContext.getHeader("projectId")
//        if (projectId.isNullOrEmpty()){
            return ProjectService.pageQuery(request)
//        }else{
//            return ProjectService.pageQuery(request, arrayListOf(SFilter.eq("id",projectId)))
//        }
    }

    @Operation(summary = "查询所有项目信息", description = "")
    @PostMapping(value = ["/query/all"])
    fun queryAll(@RequestBody request: SQueryRequest): SQueryResponse<Project> {
        return ProjectService.pageQuery(request)
    }
}