/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.wanda.controllers.relation

import com.persagy.server.datacenter.services.relation_calc.CalcSpecialService
import com.persagy.service.SPageContext
import com.persagy.service.models.responses.SBaseResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

/**
 * 关系计算-管网
 *
 * @author 邓婷婷
 */
@Tag(name = "016、关系计算-管网接口")
@RestController
@RequestMapping("/calc_special")
open class CalcSpecialController {
    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(CalcSpecialController::class.java)
    } // Companion object

    @Autowired
    lateinit var service: CalcSpecialService

    /**
     * 管网系统设备分块  +  计算流向
     *
     * @param   SystemName  需要计算的系统名称
     * @param   Domain      需要计算的管道系统类型，枚举（水管或风管）：DomainPiping，DomainHvac
     */
    @Operation(summary = "管网系统设备分块", description = "Domain 需要计算的管道系统类型，枚举（水管或风管）：DomainPiping，DomainHvac;需要计算的building_id，如果没有提供时直接为空")
    @PostMapping(value = ["/sys-block"])
    fun sysBolck(@RequestParam relationType: String,@RequestParam buildingId: String): SBaseResponse{
        return service.sysBolck(relationType,buildingId)
    } // Function sysBolck()


//    /**
//     * 管网系统确定流向
//     * @param   BlockId     需要计算的块id
//     * @param   SystemName  需要计算的系统名称
//     * @param   Domain      需要计算的管道系统类型，枚举（水管或风管）：DomainPiping，DomainHvac
//     * @param   isSource    跟据源或端计算：bool。true表示源，false表示端
//     */
//    @Operation(summary = "管网系统确定流向", description = "Domain 需要计算的管道系统类型，枚举（水管或风管）：DomainPiping，DomainHvac;需要计算的building_id，如果没有提供时直接为空;isSource 跟据源或端计算 true表示源，false表示端")
//    @PostMapping(value = ["/sys-direction"])
//    fun sysDirection(BuildingId: String?, BlockId: String, SystemName: String,Domain: String, isSource: Boolean): SBaseResponse{
//        return service.sysDirection(BuildingId, BlockId, SystemName, Domain, isSource)
//    } // Function sysDirection()


    /**
     * @Description: 管网计算
     * @Param:
     * @Return:
     */
    @Operation(summary = "管网计算", description = "管网计算")
    @PostMapping(value = ["/calcNetworks"])
    fun calcNetworks(): SBaseResponse{
        val projectId = SPageContext.getHeader("projectId").toString()
        return service.calcNetworks(projectId)
    } // Function sysBolck()

} // Class CalcSpecialController