/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.wanda.controllers.relation

import com.persagy.server.datacenter.datacenter.models.entities.graphtype.block.RelationAddRequest
import com.persagy.server.datacenter.datacenter.models.requests.ManualRelationCalcRequest
import com.persagy.server.datacenter.models.requests.ManualRelationCalcDel
import com.persagy.server.datacenter.models.response.ManualRelationCalcSCreateResponse
import com.persagy.server.datacenter.services.relation_calc.ManualRelationCalcAddService
import com.persagy.server.datacenter.services.relation_calc.ManualRelationCalcService
import com.persagy.server.datacenter.wanda.response.ManualRelationCalcResponse
import com.persagy.service.models.responses.SBaseResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController
import javax.validation.Valid

/**
 * 手动计算
 *
 * @author 张维新
 */
@Tag(name = "013、关系计算-手动计算")
@RestController
@RequestMapping("/rel/manual")
class ManualRelationCalcController {

    /**
     * 服务
     */
    @Autowired
    lateinit var service: ManualRelationCalcService

    @Autowired
    lateinit var addService: ManualRelationCalcAddService


    /**
     * 查询关系数据
     */
    @Operation(summary = "查询关系数据", description = "")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody @Valid manualRelationCalcRequest: ManualRelationCalcRequest): ManualRelationCalcResponse {
        return service.queryRel(manualRelationCalcRequest)
    }

    /**
     * 添加关系
     */
    @Operation(summary = "添加关系", description = "")
    @PostMapping(value = ["/add"])
    fun addRels(@RequestBody relationAddRequest: RelationAddRequest): ManualRelationCalcSCreateResponse {
        return addService.addRel(relationAddRequest)
    }

    /**
     * 删除关系
     */
    @Operation(summary = "删除关系", description = "")
    @PostMapping(value = ["/del"])
    fun delRels(@RequestBody manualRelationCalcDel: ManualRelationCalcDel): SBaseResponse {
        return service.delRel(manualRelationCalcDel)
    }

}














