/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.syn.controllers.obj

import com.alibaba.fastjson.JSONArray
import com.alibaba.fastjson.JSONObject
import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.dictnew.DictProject
import com.persagy.server.datacenter.models.entities.objects.*
import com.persagy.server.datacenter.syn.models.tools.EquipUtil
import com.persagy.server.datacenter.syn.models.tools.SpaceUtil
import com.persagy.server.datacenter.wanda.obj.*
import com.persagy.server.syn.Configure
import com.persagy.server.syn.models.api.obj.BuildingApi
import com.persagy.server.syn.models.api.obj.EquipApi
import com.persagy.server.datacenter.syn.models.api.obj.FloorApi
import com.persagy.server.syn.models.tools.ShaftUtil
import com.persagy.server.syn.models.tools.SystemUtil
import com.persagy.service.SBaseService
import com.persagy.service.SObjectService
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

/**
 * 数据中心 - 数据中台
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/10/22 21:39
 */
@Tag(name = "0003、对象-数据中心到中台")
@RestController
@RequestMapping("/datacenter/object/sync")
class ObjectToFromController {
    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(ObjectToFromController::class.java)
    } // Companion object
    /** 数据中心建筑服务 */
    private val buildingService = SObjectService(SMybatisDao(Building::class.java))
    /** 数据中心楼层服务 */
    private val floorService = SObjectService(SMybatisDao(Floor::class.java))
    /** 数据中心楼层轮廓线服务 */
    private val floorOutlineService = SObjectService(SMybatisDao(FloorOutline::class.java))
    /** 数据中心设备服务 */
    private val equipmentService = SObjectService(SMybatisDao(Equipment::class.java))
    /** 数据中心系统服务 */
    private val generalSystemService = SObjectService(SMybatisDao(GeneralSystem::class.java))
    /** 数据中心竖井服务 */
    private val shaftService = SObjectService(SMybatisDao(Shaft::class.java))
    /** 数据中心空间服务 */
    private val spaceService = SObjectService(SMybatisDao(ZoneSpace::class.java))
    /** 数据中心空间轮廓线服务 */
    private val spaceOutlineService = SObjectService(SMybatisDao(SpaceOutline::class.java))
    /** 字典项目服务 */
    private val dictProjectService = SBaseService(SMybatisDao(DictProject::class.java))


    /**
     * 建筑同步
     *
     * @param projectId     项目 id
     */
    @Operation(summary= "建筑-数据中心同步到数据中台", description = "")
    @PostMapping("/building")
    fun buildingSynCreate(@RequestParam projectId: String): SBaseResponse {
        try {
            val dictProject = dictProjectService.select(SFilter.eq("id", projectId)).entity()
            val buildingList = buildingService.select(SFilter.eq("projectId", projectId)).exec()
            if (buildingList.size>0){
                for (building in buildingList){
                    val createBuilding = BuildingApi.createObj(dictProject!!.groupCode!!,projectId, building)
                    if (!createBuilding){
                        logger.debug("建筑创建失败：${building}")
                        val updateBuilding = BuildingApi.updateObj(dictProject!!.groupCode!!,projectId, building)
                        if (!updateBuilding){
                            logger.debug("建筑更新失败：${building.toJson()}")
                        }
                    }
                }
            }
            return SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    }

    /**
     * 楼层-数据中心同步到数据中台
     */
    @Operation(summary= "楼层-数据中心同步到数据中台", description = "")
    @PostMapping("/floor")
    fun floorSynCreate(@RequestParam projectId: String): SBaseResponse {
        try {
            val dictProject = dictProjectService.select(SFilter.eq("id", projectId)).entity()
            val floorList = floorService.select(SFilter.eq("projectId", projectId)).exec()
            for (floor in floorList){
//                val floorOutline = floorOutlineService.select(SFilter.eq("id", floor.id!!)).entity()
//                if (floorOutline!=null){
//                    floor.outline = floorOutline.outline
//                }
                val createObj = FloorApi.createObj(dictProject!!.groupCode!!, projectId, floor)
                if (!createObj){
                    val updateObj = FloorApi.updateObj(dictProject!!.groupCode!!, projectId, floor)
                    if (!updateObj){
                        logger.debug("楼层更新失败：${floor.toJson()}")
                    }
                }


                val ObjArray = toJsonObjArray("ArchSubset", "Bd2Fl", floor.buildingId!!, floor.id!!)
                val createRel = EquipApi.createRel(dictProject.groupCode!!, projectId, ObjArray)
            }
            return SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    }

    /**
     * 设备-数据中心同步到物理世界
     */
    @Operation(summary= "设备-数据中心同步到数据中台", description = "")
    @PostMapping("/equip")
    fun equipSync(@RequestParam projectId: String): SBaseResponse {
        try {
            val dictProject = dictProjectService.select(SFilter.eq("id", projectId)).entity()
            val equipList = equipmentService.select(SFilter.eq("projectId", projectId)).exec()
            for (equip in equipList) {
                val createObj =
                    EquipApi.createObj(dictProject!!.groupCode!!, projectId, EquipUtil.equipTools(projectId, equip))
                if (!createObj) {

                }
                val updateObj =
                    EquipApi.updateObj(dictProject!!.groupCode!!, projectId, EquipUtil.equipTools(projectId, equip))
                if (!updateObj) {
                    logger.debug("设备更新失败：${equip.toJson()}")
                }
                if (!equip.buildingId.isNullOrEmpty()) {
                    val toJsonObjArray =
                        Configure.toJsonObjArray("MechInArch", "Eq2Bd", equip.id!!, equip.buildingId!!)
                    if (!equip.floorId.isNullOrEmpty()){
                        val toJsonObjArray2 =
                            Configure.toJsonObjArray("MechInArch", "Eq2Fl", equip.id!!, equip.floorId!!)
                        toJsonObjArray.add(toJsonObjArray2[0])
                    }
                    val createRel = EquipApi.createRel(dictProject.groupCode!!, projectId, toJsonObjArray)
                    if (!createRel){
                        logger.debug("设备和楼层的关系失败：${equip.toJson()}")
                    }
                }
            }
            return SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    }

    /**
     * 系统
     */
    @Operation(summary= "系统-数据中心同步到数据中台", description = "")
    @PostMapping("/sys")
    fun sysSync(@RequestParam projectId: String): SBaseResponse {
        try {
            val systemList = generalSystemService.select(SFilter.eq("projectId", projectId)).exec()
            val dictProject = dictProjectService.select(SFilter.eq("id", projectId)).entity()
            if (systemList.size>0){
                for (system in systemList) {
                    val createObj =
                        EquipApi.createObj(dictProject!!.groupCode!!, projectId, SystemUtil.systemTools(projectId, system))
                    if (!createObj) {
                        val updateObj =
                            EquipApi.updateObj(dictProject!!.groupCode!!, projectId, SystemUtil.systemTools(projectId, system))
                        if (!updateObj) {
                            logger.debug("系统更新失败：${system.toJson()}")
                        }
                    }
                }
            }
            return SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    }

    /**
     * 竖井
     */
    @Operation(summary= "竖井-数据中心同步到数据中台", description = "")
    @PostMapping("/shaft")
    fun shaftSync(@RequestParam projectId: String): SBaseResponse {
        try {
            val shaftList = shaftService.select(SFilter.eq("projectId", projectId)).exec()
            val dictProject = dictProjectService.select(SFilter.eq("id", projectId)).entity()
            if (shaftList.size>0){
                for (shaft in shaftList){
                    val createObj =
                        EquipApi.createObj(dictProject!!.groupCode!!, projectId,ShaftUtil.shaftTools(projectId, shaft))
                    if (!createObj) {
                        val updateObj =
                            EquipApi.updateObj(dictProject!!.groupCode!!, projectId, ShaftUtil.shaftTools(projectId, shaft))
                        if (!updateObj) {
                            logger.debug("竖井更新失败：${shaft.toJson()}")
                        }
                    }
                }
            }
            return SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    }


    /**
     * 空间
     */
    @Operation(summary= "空间-数据中心同步到数据中台", description = "")
    @PostMapping("/space")
    fun spaceSync(@RequestParam projectId: String): SBaseResponse {
        try {
            val spaceList = spaceService.select(SFilter.eq("projectId", projectId)).exec()
            val dictProject = dictProjectService.select(SFilter.eq("id", projectId)).entity()
            if (spaceList.size>0){
                for (space in spaceList){
//                    val spaceOutline = spaceOutlineService.select(SFilter.eq("id", space.id!!)).entity()
//                    if (spaceOutline!=null){
//                        space.outline = spaceOutline.outline
//                    }
                    val createObj =
                        EquipApi.createObj(dictProject!!.groupCode!!, projectId, SpaceUtil.spaceTools(projectId, space))
                    if (!createObj) {
                        val updateObj =
                            EquipApi.updateObj(dictProject!!.groupCode!!, projectId,SpaceUtil.spaceTools(projectId, space))
                        if (!updateObj) {
                            logger.debug("空间更新失败：${space.toJson()}")
                        }
                    }
                }
            }
            return SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    }

    /**
     * 创建关系列表
     *
     * @param graphCode     图类型编码
     * @param relCode       边类型编码
     * @param objFrom       主对象 id
     * @param objTo         从对象 id
     */
    fun toJsonObjArray(graphCode: String, relCode: String, objFrom: String, objTo: String): JSONArray {
        /** 列表 */
        val jsonArray = JSONArray()
        /** 对象 */
        val jsonObject = JSONObject()
        /** 图类型编码 */
        jsonObject["graphCode"] = graphCode
        /** 边类型编码 */
        jsonObject["relCode"] = relCode
        /** 主对象 id */
        jsonObject["objFrom"] = objFrom
        /** 从对象 id */
        jsonObject["objTo"] = objTo
        /** 添加到列表 */
        jsonArray.plus(jsonObject)
        return jsonArray
    }


}