/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.syn.controllers.obj

import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.dictnew.DictProject
import com.persagy.server.datacenter.wanda.obj.Equipment
import com.persagy.server.datacenter.wanda.obj.Floor
import com.persagy.server.datacenter.wanda.obj.Project
import com.persagy.server.syn.models.api.obj.BuildingApi
import com.persagy.server.syn.models.api.obj.EquipApi
import com.persagy.server.datacenter.syn.models.tools.EquipUtil
import com.persagy.service.SBaseService
import com.persagy.service.SObjectService
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

/**
 * 测试
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/12/24 11:47
 */
@Tag(name = "0003、对象-中台到数据中心 - 测试")
@RestController
@RequestMapping("/test/object")
class TestControllers {

    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(ObjectFromToControllers::class.java)
    } // Companion object
    /** 字典项目服务 */
    private val dictProjectService = SBaseService(SMybatisDao(DictProject::class.java))
    /** 数据中心楼层服务 */
    private val floorService = SObjectService(SMybatisDao(Floor::class.java))
    /** 数据中心设备服务 */
    private val equipmentService = SObjectService(SMybatisDao(Equipment::class.java))
    /** 数据中心项目服务 */
    private val projectService = SObjectService(SMybatisDao(Project::class.java))

    /**
     * 设备
     *
     * @param projectId         项目 id
     * @param switch            开关
     * @param isBuildingFloor   是否查询楼层
     * @param page              页码
     * @return 返回值
     */
    @Operation(summary = "设备信息-数据中台存储到数据中心 ", description = "switch true  根据输入的项目 id  同步楼层，false 同步所有项目的 楼层")
    @PostMapping(value = ["/equip-syn-dan"])
    fun equipSyn(@RequestParam projectId: String, @RequestParam isBuildingFloor: Boolean = true,@RequestParam id: String): SBaseResponse {
        return try {

                val dictProject = dictProjectService.select(SFilter.eq("id", projectId)).entity()
//                val equipSynApi = EquipApi.equipSynApi(dictProject!!.groupCode!!,projectId, page)
                val queryId = BuildingApi.queryId(dictProject?.groupCode!!, projectId!!, id!!)
                    val equipSyn = EquipUtil.equipSyns(projectId, dictProject!!.groupCode!!, queryId[0])
                    logger.debug("设备=${equipSyn.toJson()}")
                    if (isBuildingFloor){
                        /** 查询建筑 */
                        val equipInBuildingList = EquipApi.equipInBuilding(dictProject!!.groupCode!!,projectId, equipSyn.id!!)
                        if (equipInBuildingList.size>0){
                            val hashMap = equipInBuildingList[0]
                            if (hashMap.containsKey("objTo")){
                                equipSyn.buildingId = hashMap["objTo"].toString()
                            }
                        }
                        /** 查询楼层 */
                        val equipInFloorList = EquipApi.equipInFloor(dictProject!!.groupCode!!,projectId, equipSyn.id!!)
                        if (equipInFloorList.size>0){
                            val hashMap = equipInFloorList[0]
                            if (hashMap.containsKey("objTo")){
                                equipSyn.floorId = hashMap["objTo"].toString()
                            }
                        }
                        if (equipSyn.buildingId.isNullOrEmpty()){
                            if (!equipSyn.floorId.isNullOrEmpty()){
                                val floor = floorService.select(SFilter.eq("id", equipSyn.floorId!!)).entity()
                                if (floor!=null){
                                    equipSyn.buildingId = floor.buildingId
                                }
                            }
                        }
                    }
                    equipmentService.replace(equipSyn)


            SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            SBaseResponse(SResponseType.failure, e.message!!)
        }
    }



}