/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.syn.jms

import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.jms.SynchronousMessage
import com.persagy.server.datacenter.models.jms.SynchronousMessageRel
import com.persagy.server.datacenter.syn.jms.DataCenterSync
import com.persagy.server.syn.jms.bean.SyncRwdDatacenterMessage
import com.persagy.server.syn.task.SchedulerTask
import com.persagy.service.SObjectService
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service

/**
 * 消息处理
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/11/1 21:54
 */
@Service
class MessageProcessing {

    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(SchedulerTask::class.java)
    } // Companion object

    /** 缓存对象数据服务 - 数据中心到数据中台  */
    var synchronousMessageObjService = SObjectService(SMybatisDao(SynchronousMessage::class.java))
    var synchronousMessageRelService = SObjectService(SMybatisDao(SynchronousMessageRel::class.java))
    /** 缓存对象数据 - 数据中台到数据中心 */
    var syncRwdMessageService = SObjectService(SMybatisDao(SyncRwdDatacenterMessage::class.java))

    /** 数据中心处理数据同步到数据中台 */
    @Autowired
    lateinit var dataCenterSync: DataCenterSync
    /** 数据中台到物理世界 */
    @Autowired
    lateinit var rwdSync: RwdSync

    var messageTypeRwd = 1
    var messageTypeDataCerten = 1
    var messageTypeDataCenterRel = 1
    /**
     * 处理数据中台的消息到数据中心
     */
    fun syncRwdData(){
        /** 子线程处理数据 */

        try {
            logger.debug("状态-中台-  ${messageTypeRwd}")
            if (messageTypeRwd!=0){
                try {
                    messageTypeRwd = 0
                    val syncRwdMessageList = syncRwdMessageService.select(SFilter.not(SFilter.eq("state", 0))).order("lastUpdate").limit(1000).exec()
                    if (syncRwdMessageList.size >0){
                        rwdSync.dataSync(syncRwdMessageList)
                        messageTypeRwd = 1
                        syncRwdData()
                    } else {
                        logger.debug("释放状态-中台")
                        messageTypeRwd = 1
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                    messageTypeRwd = 1
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
            messageTypeRwd = 1
        }

    }


    /**
     * 处理数据中心的消息到物理世界
     */
    fun syncDataCenter(){
        /** 子线程处理数据 */
//        object : Thread() {
//            override fun run() {
//                super.run()
//                while (true){
//                }
//            }
//        }
        try {
            logger.debug("标记状态= ${messageTypeDataCerten}  *********************************************************")
            /** 不等于 0 时候执行同步 */
//            if (messageTypeDataCerten!=0){
                logger.debug("执行任务*********************************************************")
                /** 设置为 0  防止重复执行 */
                messageTypeDataCerten =0
                /** 查询待处理的消息 */
                try {
                    val messageList =
                        synchronousMessageObjService.select(SFilter.not(SFilter.eq("sign", 0)),SFilter.lte("count",20)).order("createTime").exec()
                    if (messageList.size>0) {
                        /** 处理消息 */
                        dataCenterSync.dataSync(messageList)
                        /** 开启执行 */
                        messageTypeDataCerten = 1
                        /** 查询是否有待处理的消息 */
                        syncDataCenter()
                    } else {
                        logger.debug("释放状态*********************************************************")
                        /** 开启下次执行 */
                        messageTypeDataCerten = 1
                    }
                }catch (e:Exception){
                    e.printStackTrace()
                    messageTypeDataCerten = 1
                }
//            }
        } catch (e: Exception) {
            e.printStackTrace()
            messageTypeDataCerten = 1
        }
    }

    /**
     * 处理关系数据到数据中台
     */
    fun syncDataCenterRel(){

        try {
            /** 不等于 0 时候执行同步 */
            if (messageTypeDataCenterRel!=0){
                /** 设置为 0  防止重复执行 */
                messageTypeDataCenterRel =0
                /** 查询待处理的消息 */
                val messageList =
                        synchronousMessageRelService.select(SFilter.not(SFilter.eq("sign", 0))).order("createTime").exec()
                if (messageList.size>0) {
                    /** 处理消息 */
                    dataCenterSync.dataSyncRel(messageList)
                    /** 开启执行 */
                    messageTypeDataCenterRel = 1
                    /** 查询是否有待处理的消息 */
                    syncDataCenter()
                } else {
                    /** 开启下次执行 */
                    messageTypeDataCenterRel = 1
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
            /** 开启下次执行 */
            messageTypeDataCenterRel = 1
        }
    }





}