/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.syn.models.api.obj

import com.alibaba.fastjson.JSONObject
import com.persagy.base.utils.SHttpUtil
import com.persagy.server.datacenter.wanda.obj.Building
import com.persagy.server.syn.Configure
import com.persagy.server.syn.models.obj.MiddleGroundDataRespose
import com.persagy.server.syn.models.tools.BuildingUtil
import org.slf4j.LoggerFactory

/**
 * 建筑 api 中台至数据中心
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/10/19 9:10
 */
object BuildingApi {

    /** 日志 */
    private val logger = LoggerFactory.getLogger(BuildingApi::class.java)

    /**
     * 获取建筑
     *
     * @param projectId     项目 id
     * @param type          类型
     * @return              返回结果
     */
    fun buildingSyn(groupCode: String,projectId: String,type: String): ArrayList<HashMap<String, Any?>> {
        try {
            val jsonObject = JSONObject()
            val jsonObject2 = JSONObject()
            jsonObject2["classCode"] = type
            jsonObject2["valid"] = 1
            jsonObject["criteria"] = jsonObject2
            /** 直接 http 请求还是通过注册中心 */
            if (!Configure.customProperties.eurekaorhttp) {
                val url = "${Configure.opts.data_rwd}/rwd/instance/object/query?groupCode=$groupCode&projectId=${projectId}"
                val postObject = SHttpUtil.postObject<MiddleGroundDataRespose>(url, jsonObject)
                if (postObject.result == "success"&&!postObject.data.isNullOrEmpty()) {
                    return postObject.data!!
                }
            } else {
                val url = "http://${Configure.serviceNameConfig.DMPRWD}/rwd/instance/object/query?groupCode=$groupCode&projectId=${projectId}"
                val postObject = Configure.restTemplate.postForObject(url, jsonObject, MiddleGroundDataRespose::class.java)
                if (postObject.result == "success"&&!postObject.data.isNullOrEmpty()) {
                    return postObject.data!!
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return ArrayList<HashMap<String, Any?>>()
    }

    /**
     * 中台创建
     *
     * @param projectId     项目 id
     * @param building      建筑对象
     * @return 是否成功
     */
    fun  createObj(groupCode: String,projectId: String, building: Building): Boolean {
        try {
            val buildingTools = BuildingUtil.buildingTools(projectId, building)
            /** 直接 http 请求还是通过注册中心 */
            if (!Configure.customProperties.eurekaorhttp) {
                val url = "${Configure.opts.data_rwd}/rwd/instance/object/create?groupCode=$groupCode&projectId=${projectId}&appId=datacenter"
                val postObject = SHttpUtil.postObject<MiddleGroundDataRespose>(url, buildingTools)
                if (postObject.result == "success") {
                    return true
                }
            } else {
                val url = "http://${Configure.serviceNameConfig.DMPRWD}/rwd/instance/object/create?groupCode=$groupCode&projectId=${projectId}&appId=datacenter"
                val postObject = Configure.restTemplate.postForObject(url, buildingTools, MiddleGroundDataRespose::class.java)
                if (postObject.result == "success") {
                    return true
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return false
    }

    /**
     * 中台更新建筑
     *
     * @param projectId     项目 id
     * @param building      建筑对象
     * @return 是否成功
     */
    fun  updateObj(groupCode: String,projectId: String, building: Building): Boolean {
        try {
            val buildingTools = BuildingUtil.buildingTools(projectId, building)
            /** 直接 http 请求还是通过注册中心 */
            if (!Configure.customProperties.eurekaorhttp) {
                val url = "${Configure.opts.data_rwd}/rwd/instance/object/update?groupCode=$groupCode&projectId=${projectId}&appId=datacenter"
                val postObject = SHttpUtil.postObject<MiddleGroundDataRespose>(url, buildingTools)
                if (postObject.result == "success") {
                    return true
                }
            } else {
                val url = "http://${Configure.serviceNameConfig.DMPRWD}/rwd/instance/object/update?groupCode=$groupCode&projectId=${projectId}&appId=datacenter"
                val postObject = Configure.restTemplate.postForObject(url, buildingTools, MiddleGroundDataRespose::class.java)
                if (postObject.result == "success") {
                    return true
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return false
    }

    /**
     * 中台删除建筑
     *
     * @param projectId     项目 id
     * @param building      建筑对象
     * @return 是否成功
     */
    fun  deleteObj(groupCode: String,projectId: String, buildingId: String): Boolean {
        try {
            val listId = ArrayList<String>()
            listId.add(buildingId)
            /** 直接 http 请求还是通过注册中心 */
            if (!Configure.customProperties.eurekaorhttp) {
                val url = "${Configure.opts.data_rwd}/rwd/instance/object/delete?groupCode=$groupCode&projectId=${projectId}&appId=datacenter"
                val postObject = SHttpUtil.postObject<MiddleGroundDataRespose>(url, listId)
                if (postObject.result == "success") {
                    return true
                }
            } else {
                val url = "http://${Configure.serviceNameConfig.DMPRWD}/rwd/instance/object/delete?groupCode=$groupCode&projectId=${projectId}&appId=datacenter"
                val postObject = Configure.restTemplate.postForObject(url, listId, MiddleGroundDataRespose::class.java)
                if (postObject.result == "success") {
                    return true
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return false
    }

    /**
     * 根据 id 查询对象
     *
     * @param projectId     项目 id
     * @param id             对象 id
     * @return 返回结果
     */
    fun queryId(groupCode: String, projectId: String,id: String): ArrayList<HashMap<String, Any?>> {
        try {
            val jsonObject = JSONObject()
            val jsonObject2 = JSONObject()
            jsonObject2["id"] = id
            jsonObject["criteria"] = jsonObject2
            /** 直接 http 请求还是通过注册中心 */
            if (!Configure.customProperties.eurekaorhttp) {
                val url = "${Configure.opts.data_rwd}/rwd/instance/object/query?groupCode=$groupCode&projectId=${projectId}"
                val postObject = SHttpUtil.postObject<MiddleGroundDataRespose>(url, jsonObject)
                if (postObject.result == "success"&&!postObject.data.isNullOrEmpty()) {
                    return postObject.data!!
                }
            } else {
                val url = "http://${Configure.serviceNameConfig.DMPRWD}/rwd/instance/object/query?groupCode=$groupCode&projectId=${projectId}"
                val postObject = Configure.restTemplate.postForObject(url, jsonObject, MiddleGroundDataRespose::class.java)
                if (postObject.result == "success"&&!postObject.data.isNullOrEmpty()) {
                    return postObject.data!!
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return ArrayList<HashMap<String, Any?>>()
    }

}