/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.syn.models.api.obj

import com.alibaba.fastjson.JSONObject
import com.persagy.base.utils.SHttpUtil
import com.persagy.server.syn.Configure
import com.persagy.server.syn.Opts
import com.persagy.server.syn.models.obj.MiddleGroundDataRespose
import com.persagy.service.utils.SSpringContextUtil

/**
 * 空间 API
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/10/20 9:18
 */
object SpaceApi {

    /** 配置实例 */
    private val opts: Opts by lazy {
        SSpringContextUtil.getBean(Opts::class.java) as Opts
    } // opts

    /**
     * 空间 - 获取中台空间数据
     *
     * @param projectId     项目 id
     * @return 返回结果
     */
    fun spaceSynApi(groupCode: String,projectId: String): ArrayList<HashMap<String, Any?>>{
        val jsonObject = JSONObject()
        val jsonObject2 = JSONObject()
        jsonObject2["objType"] = "space"
        jsonObject2["valid"] = 1
        jsonObject["criteria"] = jsonObject2
        /** 直接 http 请求还是通过注册中心 */
        if (!Configure.customProperties.eurekaorhttp) {
            val url = "${opts.data_rwd}/rwd/instance/object/query?groupCode=$groupCode&projectId=${projectId}"
            val postObject = SHttpUtil.postObject<MiddleGroundDataRespose>(url, jsonObject)
            if (postObject.result == "success"&&!postObject.data.isNullOrEmpty()) {
                return postObject.data!!
            }
        } else {
            val url = "http://${Configure.serviceNameConfig.DMPRWD}/rwd/instance/object/query?groupCode=$groupCode&projectId=${projectId}"
            val postObject = Configure.restTemplate.postForObject(url, jsonObject, MiddleGroundDataRespose::class.java)
            if (postObject.result == "success"&&!postObject.data.isNullOrEmpty()) {
                return postObject.data!!
            }
        }

        return ArrayList<HashMap<String, Any?>>()
    }

    /**
     * 获取设备和建筑的关系
     *
     * @param projectId     项目 id
     * @param spaceId       设备 id
     * @return 返回结果
     */
    fun spaceInBuilding(groupCode: String,projectId: String,spaceId: String): ArrayList<HashMap<String, Any?>> {
        val jsonObject = JSONObject()
        val jsonObject2 =  JSONObject()
        jsonObject2["graphCode"] = "ArchSubset"
        jsonObject2["relCode"] = "Bd2Sp"
        jsonObject2["objTo"] = spaceId
        jsonObject["criteria"] = jsonObject2
        /** 直接 http 请求还是通过注册中心 */
        if (!Configure.customProperties.eurekaorhttp) {
            val url = "${opts.data_rwd}/rwd/instance/relation/query?groupCode=$groupCode&projectId=${projectId}"
            val postObject = SHttpUtil.postObject<MiddleGroundDataRespose>(url, jsonObject)
            if (postObject.result == "success"&&!postObject.data.isNullOrEmpty()) {
                return postObject.data!!
            }
        } else {
            val url = "http://${Configure.serviceNameConfig.DMPRWD}/rwd/instance/relation/query?groupCode=$groupCode&projectId=${projectId}"
            val postObject = Configure.restTemplate.postForObject(url, jsonObject, MiddleGroundDataRespose::class.java)
            if (postObject.result == "success"&&!postObject.data.isNullOrEmpty()) {
                return postObject.data!!
            }
        }

        return ArrayList<HashMap<String, Any?>>()
    }

    /**
     * 获取设备和建筑的关系
     *
     * @param projectId     项目 id
     * @param spaceId       设备 id
     * @return 返回结果
     */
    fun spaceInFloor(groupCode: String,projectId: String,spaceId: String): ArrayList<HashMap<String, Any?>> {
        val jsonObject = JSONObject()
        val jsonObject2 =  JSONObject()
        jsonObject2["graphCode"] = "ArchSubset"
        jsonObject2["relCode"] = "Fl2Sp"
        jsonObject2["objTo"] = spaceId
        jsonObject["criteria"] = jsonObject2
        /** 直接 http 请求还是通过注册中心 */
        if (!Configure.customProperties.eurekaorhttp) {
            val url = "${opts.data_rwd}/rwd/instance/relation/query?groupCode=$groupCode&projectId=${projectId}"

            val postObject = SHttpUtil.postObject<MiddleGroundDataRespose>(url, jsonObject)
            if (postObject.result == "success"&&!postObject.data.isNullOrEmpty()) {
                return postObject.data!!
            }
        } else {
            val url = "http://${Configure.serviceNameConfig.DMPRWD}/rwd/instance/relation/query?groupCode=$groupCode&projectId=${projectId}"
            val postObject = Configure.restTemplate.postForObject(url, jsonObject, MiddleGroundDataRespose::class.java)
            if (postObject.result == "success"&&!postObject.data.isNullOrEmpty()) {
                return postObject.data!!
            }
        }
        return ArrayList<HashMap<String, Any?>>()
    }

}