/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.syn.models.tools

import com.persagy.base.extensions.toInt
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.dictnew.DefFuncId
import com.persagy.server.datacenter.models.entities.dictnew.Group
import com.persagy.server.datacenter.wanda.obj.Project
import com.persagy.server.syn.Opts
import com.persagy.service.SBaseService
import com.persagy.service.SObjectService
import com.persagy.service.utils.SSpringContextUtil
import org.slf4j.LoggerFactory

/**
 * 中台数据转换为数据中心，项目
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/10/18 19:03
 */
object ProjectUtil {
    /** 日志 */
    private val logger = LoggerFactory.getLogger(ProjectUtil::class.java)

    /** 集团服务 */
    private val groupService = SBaseService(SMybatisDao(Group::class.java))
    /** 信息点 */
    private val dictDefFuncIdService = SObjectService(SMybatisDao(DefFuncId::class.java))
    /** 配置 */
    private val opts: Opts by lazy {
        SSpringContextUtil.getBean(Opts::class.java) as Opts
    }


    /**
     * 项目转换工具
     *
     * @param groupCode     集团编码
     * @param mapObj        中台信息点
     * @return 返回项目对象
     */
    fun projectDataConversions(groupCode: String, mapObj: HashMap<String, Any?>): Project {

        /** 查询项目类型字典 */
        var dictDefFuncIdList = dictDefFuncIdService.select(SFilter.eq("groupCode", groupCode), SFilter.eq("projectId", mapObj["id"].toString())
        , SFilter.eq("classCode","project") ).exec()
        if (dictDefFuncIdList.isNullOrEmpty()) {
            dictDefFuncIdList = dictDefFuncIdService.select(SFilter.eq("type", "commo"),
                SFilter.eq("classCode","project")).exec()
        }

        val mapEx = HashMap<String, Any?>()
        val mapCode = HashMap<String, Any?>()
        /** 遍历项目类型字典 */
        for (explainProject in dictDefFuncIdList) {
            /** 输入类型 */
            mapEx[explainProject.code!!] = explainProject.dataType
            mapCode[explainProject.code!!] = explainProject.category
        }

        val project = Project()
        for (key in mapObj.keys) {
            val containsKey = mapEx.containsKey(key)
            if (containsKey){
                if (key != "id"&& key!="name"&&key!="localId"&&key!="localName"&&key!="introduction"
                    &&key!="businessFloorNum"&&key!="officeFloorNum"){
                    val containsKey1 = mapCode.containsKey(key)
                    if (containsKey1){
                        val any = mapCode[key].toString()
                        if (any == "STATIC"){
                            when(mapEx[key].toString()) {
                                "INTEGER" -> {
                                    if (project.infos.isNullOrEmpty()){
                                        project.infos = HashMap<String,Any?>()
                                    }
                                    project.infos!![key] = mapObj[key]!!.toInt()
                                }
                                "DOUBLE" -> {
                                    if (project.infos.isNullOrEmpty()){
                                        project.infos = HashMap<String,Any?>()
                                    }
                                    project.infos!![key] = mapObj[key]!!.toString().toDoubleOrNull()
                                }
                                "BOOLEAN" -> {
                                    if (project.infos.isNullOrEmpty()){
                                        project.infos = HashMap<String,Any?>()
                                    }
                                    project.infos!![key] = mapObj[key]
                                }
                                "STRING" -> {
                                    if (project.infos.isNullOrEmpty()){
                                        project.infos = HashMap<String,Any?>()
                                    }
                                    project.infos!![key] = mapObj[key]
                                }
                                "DATETIME" -> {
                                    if (project.infos.isNullOrEmpty()){
                                        project.infos = HashMap<String,Any?>()
                                    }
                                    project.infos!![key] = mapObj[key]
                                }
                                "ATTACHMENT" -> {
                                    if (project.infos.isNullOrEmpty()){
                                        project.infos = HashMap<String,Any?>()
                                    }
                                    project.infos!![key] = mapObj[key]
                                }
                                "OBJECT" -> {
                                    if (project.infos.isNullOrEmpty()){
                                        project.infos = HashMap<String,Any?>()
                                    }
                                    project.infos!![key] = mapObj[key]
                                }
                                "ENUM" -> {
                                    if (project.infos.isNullOrEmpty()){
                                        project.infos = HashMap<String,Any?>()
                                    }
                                    project.infos!![key] = mapObj[key]
                                }
                                "MENUM" -> {
                                    if (project.infos.isNullOrEmpty()){
                                        project.infos = HashMap<String,Any?>()
                                    }
                                    project.infos!![key] = mapObj[key]
                                }
                                else -> {
                                    if (project.infos.isNullOrEmpty()){
                                        project.infos = HashMap<String,Any?>()
                                    }
                                    project.infos!![key] = mapObj[key]!!.toString()
                                }
                            }
                        }else{
                            if (project.infos.isNullOrEmpty()){
                                project.infos = HashMap<String,Any?>()
                            }
                            project.infos!![key] = mapObj[key]!!.toString()
                        }
                    }
                } else {
                  when(key) {
                      "id" -> project.id =  mapObj[key]!!.toString()
                      "name" -> project.name =  mapObj[key]!!.toString()
                      "localId" -> project.localId =  mapObj[key]!!.toString()
                      "localName" -> project.localName =  mapObj[key]!!.toString()
                      "introduction" -> project.introduction =  mapObj[key]!!.toString()
                      "businessFloorNum" -> project.businessFloorNum =  mapObj[key]!!.toString()
                      "officeFloorNum" -> project.officeFloorNum =  mapObj[key]!!.toString()
                  }
                }

            } else {
//                if (project.infos.isNullOrEmpty()){
//                    project.infos = HashMap<String,Any?>()
//                }
//                project.infos!![key] =  mapObj[key]!!.toString()
            }
        }

        return project
    }




}