/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.syn.models.tools

import com.persagy.base.extensions.toInt
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.dictnew.DefFuncId
import com.persagy.server.datacenter.wanda.obj.Shaft
import com.persagy.server.syn.Opts
import com.persagy.server.syn.controllers.obj.ObjectFromToControllers
import com.persagy.service.SObjectService
import com.persagy.service.utils.SSpringContextUtil
import org.slf4j.LoggerFactory

/**
 * 竖井转换工具
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/10/21 20:22
 */
object ShaftUtil {

    /** 日志 */
    private val logger = LoggerFactory.getLogger(ObjectFromToControllers::class.java)
    /** 信息点 */
    private val dictDefFuncIdService = SObjectService(SMybatisDao(DefFuncId::class.java))
    /** 配置 */
    private val opts: Opts by lazy {
        SSpringContextUtil.getBean(Opts::class.java) as Opts
    }



    /**
     * 竖井
     *
     * @param projectId     项目id
     * @param groupCode     集团编码
     * @param mapObj        对象信息
     * @return 竖井对象
     */
    fun shaftDataConversions(projectId: String,groupCode: String, mapObj: HashMap<String, Any?>): Shaft {
        /** 查建楼层型字典 */
        var dictDefFuncIdList = dictDefFuncIdService.select(SFilter.eq("groupCode", groupCode),SFilter.eq("projectId",projectId), SFilter.eq("classCode","shaft")).exec()
        if (dictDefFuncIdList.isNullOrEmpty()) {
            dictDefFuncIdList = dictDefFuncIdService.select(SFilter.eq("type", "commo"),
                SFilter.eq("classCode","shaft")).exec()
        }
        val mapEx = HashMap<String, Any?>()
        val mapCode = HashMap<String, Any?>()
        /** 遍历项目类型字典 */
        for (explainProject in dictDefFuncIdList) {
            /** 输入类型 */
            mapEx[explainProject.code!!] = explainProject.dataType
            mapCode[explainProject.code!!] = explainProject.category
        }

        val shaft = Shaft()
        shaft.projectId = projectId

        for (key in mapObj.keys) {
            val containsKey = mapEx.containsKey(key)
            if (containsKey){
                if (key != "id"&& key!="name"&&key!="localId"&&key!="localName"&&key!="defaultQRCode"
                    &&key!="qRCodePic"&&key!="cadId"&&key!="bimId"){
                    val containsKey1 = mapCode.containsKey(key)
                    if (containsKey1){
                        val any = mapCode[key].toString()
                        if (any == "STATIC"){
                            when(mapEx[key].toString()) {
                                "INTEGER" -> {
                                    if (shaft.infos.isNullOrEmpty()){
                                        shaft.infos = HashMap<String,Any?>()
                                    }
                                    shaft.infos!![key] = mapObj[key]!!.toInt()
                                }
                                "DOUBLE" -> {
                                    if (shaft.infos.isNullOrEmpty()){
                                        shaft.infos = HashMap<String,Any?>()
                                    }
                                    shaft.infos!![key] = mapObj[key]!!.toString().toDoubleOrNull()
                                }
                                "BOOLEAN" -> {
                                    if (shaft.infos.isNullOrEmpty()){
                                        shaft.infos = HashMap<String,Any?>()
                                    }
                                    shaft.infos!![key] = mapObj[key]
                                }
                                "STRING" -> {
                                    if (shaft.infos.isNullOrEmpty()){
                                        shaft.infos = HashMap<String,Any?>()
                                    }
                                    shaft.infos!![key] = mapObj[key]!!.toString()
                                }
                                "DATETIME" -> {
                                    if (shaft.infos.isNullOrEmpty()){
                                        shaft.infos = HashMap<String,Any?>()
                                    }
                                    shaft.infos!![key] = mapObj[key]
                                }
                                "ATTACHMENT" -> {
                                    if (shaft.infos.isNullOrEmpty()){
                                        shaft.infos = HashMap<String,Any?>()
                                    }
                                    shaft.infos!![key] = mapObj[key]
                                }
                                "OBJECT" -> {
                                    if (shaft.infos.isNullOrEmpty()){
                                        shaft.infos = HashMap<String,Any?>()
                                    }
                                    shaft.infos!![key] = mapObj[key]
                                }
                                "ENUM" -> {
                                    if (shaft.infos.isNullOrEmpty()){
                                        shaft.infos = HashMap<String,Any?>()
                                    }
                                    shaft.infos!![key] = mapObj[key]
                                }
                                "MENUM" -> {
                                    if (shaft.infos.isNullOrEmpty()){
                                        shaft.infos = HashMap<String,Any?>()
                                    }
                                    shaft.infos!![key] = mapObj[key]
                                }
                                else -> {
                                    if (shaft.infos.isNullOrEmpty()){
                                        shaft.infos = HashMap<String,Any?>()
                                    }
                                    shaft.infos!![key] = mapObj[key]!!.toString()
                                }
                            }
                        }else{
                            if (shaft.infos.isNullOrEmpty()){
                                shaft.infos = HashMap<String,Any?>()
                            }
                            shaft.infos!![key] = mapObj[key]!!.toString()
                        }
                    }

                } else {
                    when(key) {
                        "id" -> shaft.id =  mapObj[key]!!.toString()
                        "name" -> shaft.name =  mapObj[key]!!.toString()
                        "localId" -> shaft.localId =  mapObj[key]!!.toString()
                        "localName" -> shaft.localName =  mapObj[key]!!.toString()
                        "defaultQRCode" -> shaft.defaultQRCode =  mapObj[key]!!.toString()
                        "qRCodePic" -> shaft.qRCodePic =  mapObj[key]!!.toString()
                        "cadId" -> shaft.cadId =  mapObj[key]!!.toString()
                        "bimId" -> shaft.bimId =  mapObj[key]!!.toString()
                    }
                }

            } else {
                //                if (zoneSpace.infos.isNullOrEmpty()){
//                    zoneSpace.infos = HashMap<String,Any?>()
//                }
//                zoneSpace.infos!![key] =  mapObj[key]!!.toString()
            }
        }

        return shaft
    }

    /**
     * 竖井工具
     *
     * @param projectId     项目 id
     * @param shaft         竖井对象
     * @return 返回结果
     */
    fun shaftTools(projectId: String,shaft: Shaft): ArrayList<HashMap<String,Any?>> {
        var list = ArrayList<HashMap<String,Any?>>()
        val hashMap = HashMap<String, Any?>()
        hashMap["id"] = shaft.id
        hashMap["name"] = shaft.name
        hashMap["projectId"] = projectId
        hashMap["objType"] = "shaft"
        hashMap["classCode"] = "shaft"
        if (!shaft.localName.isNullOrEmpty()){
            hashMap["localName"] = shaft.localName
        }
        if (!shaft.localId.isNullOrEmpty()){
            hashMap["localId"] = shaft.localId
        }
        if (!shaft.defaultQRCode.isNullOrEmpty()){
            hashMap["defaultQRCode"] = shaft.defaultQRCode
        }
        if (!shaft.qRCodePic.isNullOrEmpty()){
            hashMap["qRCodePic"] = shaft.qRCodePic
        }
        if (!shaft.bimId.isNullOrEmpty()){
            hashMap["bimId"] = shaft.bimId
        }
        if (!shaft.bimLocation.isNullOrEmpty()){
            hashMap["bimLocation"] = shaft.bimLocation
        }
        if (!shaft.outline.isNullOrEmpty()){
            hashMap["outline"] = shaft.outline.toString()
        }
        if (shaft.shaftSequenceId!=null&&shaft.shaftSequenceId!=0){
            hashMap["shaftSequenceId"] = shaft.shaftSequenceId
        }
        if (!shaft.infos.isNullOrEmpty()){
            for (key in shaft.infos!!.keys){
                hashMap[key] = shaft.infos!![key]
            }
        }

        list.add(hashMap)
        return list
    }


}