/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.syn.services.jms

import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.dictnew.DictProject
import com.persagy.server.datacenter.models.jms.SynchronousMessage
import com.persagy.server.datacenter.wanda.obj.Building
import com.persagy.server.syn.Configure
import com.persagy.server.syn.models.api.obj.BuildingApi
import com.persagy.service.SBaseService
import com.persagy.service.SObjectService
import org.slf4j.LoggerFactory
import org.springframework.stereotype.Service

/**
 * 建筑
 *
 * @author wx  <zhangweixin></zhangweixin>@sagacloud.com>
 * @date 2020/11/2 8:43
 */
@Service
class DataCenterBuilding {
    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(DataCenterBuilding::class.java)
    } // Companion object
    /** 建筑服务 */
    private val buildingService = SBaseService(SMybatisDao(Building::class.java))
    /** 缓存对象数据服务 - 数据中心到数据中台  */
    var synchronousMessageObjService = SObjectService(SMybatisDao(SynchronousMessage::class.java))
    /** 字典项目服务 */
    private val dictProjectService = SBaseService(SMybatisDao(DictProject::class.java))
    /**
     * 处理 数据中心到 - 数据中台的建筑
     */
    fun buildingType(message: SynchronousMessage) {
        val dictProject = dictProjectService.select(SFilter.eq("id", message.projectId!!)).entity()
        logger.debug(" 处理建筑***********************************")
        when (message.type) {
            Configure.CREATE -> {
                /**查询建筑信息 */
                val building = buildingService.select(
                    SFilter.eq("projectId", message.projectId!!),
                    SFilter.eq("id", message.objId!!)
                ).entity()
                logger.debug(" 处理建筑对象${building?.toJson()}***********************************")
                /** 建筑不为空 */
                if (building!=null) {
                    val sign = BuildingApi.createObj(dictProject!!.groupCode!!,message.projectId!!, building)
                    if (sign){
                        /** 成功 */
                        message.sign = 0
                    }else {
                        /** 失败 */
                        message.sign = 2
                        message.count+1
                    }
                }else {
                    message.sign = 0
                }
                /** 更改标记 */
                synchronousMessageObjService.update(message)
            }
            Configure.UPDATE -> {
                /** 修改 */
                /**查询建筑信息 */
                val building = buildingService.select(
                    SFilter.eq("projectId", message.projectId!!),
                    SFilter.eq("id", message.objId!!)
                ).entity()
                /** 建筑不为空 */
                if (building!=null) {
                    val sign = BuildingApi.updateObj(dictProject!!.groupCode!!,message.projectId!!, building)
                    if (sign){
                        /** 成功 */
                        message.sign = 0
                    } else {
                        val sign = BuildingApi.createObj(dictProject!!.groupCode!!,message.projectId!!, building)
                        if (sign){
                            /** 成功 */
                            message.sign = 0
                        }else {
                            /** 失败 */
                            message.sign = 2
                            message.count = message.count +1
                        }
                    }

                } else {
                    /** 成功 */
                    message.sign = 0
                }
                /** 更改标记 */
                synchronousMessageObjService.update(message)
            }
            Configure.DELETE -> {
                /** 删除 */
               val sign = BuildingApi.deleteObj(dictProject!!.groupCode!!,message.projectId!!, message.objId!!)
                if (sign){
                    /** 成功 */
                    message.sign = 0
                }else {
                    /** 失败 */
                    message.sign = 2
                    message.count = message.count +1
                }
                /** 更改标记 */
                synchronousMessageObjService.update(message)
            }
        }
    }

}