/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.syn.services.jms

import com.alibaba.fastjson.JSONArray
import com.alibaba.fastjson.JSONObject
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.dictnew.DictProject
import com.persagy.server.datacenter.models.entities.rel.RSpinBd
import com.persagy.server.datacenter.models.entities.rel.RSpinFl
import com.persagy.server.datacenter.models.jms.SynchronousMessage
import com.persagy.server.syn.Configure
import com.persagy.server.syn.models.api.obj.EquipApi
import com.persagy.server.datacenter.syn.models.tools.SpaceUtil
import com.persagy.server.datacenter.wanda.obj.ZoneSpace
import com.persagy.service.SBaseService
import com.persagy.service.SObjectService
import org.springframework.stereotype.Service

/**
 * 同步数据处理
 *
 * @author wx  <zhangweixin></zhangweixin>@sagacloud.com>
 * @date 2020/11/2 8:43
 */
@Service
class DataCenterSpace {

    /** 服务 */
    private val spaceService = SBaseService(SMybatisDao(ZoneSpace::class.java))
    /** 缓存对象数据服务 - 数据中心到数据中台  */
    var synchronousMessageObjService = SObjectService(SMybatisDao(SynchronousMessage::class.java))
    /** 项目所属的集团 */
    private val dictProjectService = SObjectService(SMybatisDao(DictProject::class.java))
    /** 空间和建筑的关系 */
    private val rSpinBdService = SObjectService(SMybatisDao(RSpinBd::class.java))
    /** 空间和楼层的关系 */
    private val rSpinFlService = SObjectService(SMybatisDao(RSpinFl::class.java))

    /**
     * 处理 数据中心到 - 数据中台的数据
     *
     * @param message   消息对象
     */
    fun spaceType(message: SynchronousMessage) {
        val dictProject = dictProjectService.select(SFilter.eq("id", message.projectId!!)).entity()
        when (message.type) {
            Configure.CREATE -> {
                /**查询信息 */
                val space = spaceService.select(
                    SFilter.eq("projectId", message.projectId!!),
                    SFilter.eq("id", message.objId!!)
                ).entity()
                /** 不为空 */
                if (space!=null) {
                    val spaceList = SpaceUtil.spaceTools(message.projectId!!, space)
                    val sign = EquipApi.createObj(dictProject!!.groupCode!!,message.projectId!!, spaceList)
                    if (sign){
                        /** 成功 */
                        message.sign = 0

                        if (dictProject!=null){
                            val rSpinBdList = rSpinBdService.select(
                                SFilter.eq("projectId", message.projectId!!),
                                SFilter.eq("spaceId", space.id!!)
                            ).exec()
                            val rSpinFlList = rSpinFlService.select(
                                SFilter.eq("projectId", message.projectId!!),
                                SFilter.eq("spaceId", space.id!!)
                            ).exec()
                            val listRel = ArrayList<JSONObject>()
                            val jsonArray = JSONArray()
                            /** 业务空间的建筑 */
                            if (!space.buildingId.isNullOrEmpty()) {
                                val toJsonObj =
                                    toJsonObjArray("ArchSubset", "Bd2Sp", space.id!!, space.buildingId!!)
                                jsonArray.add(toJsonObj)
                                if (!space.floorId.isNullOrEmpty()) {
                                    val toJsonObjFloor =
                                        toJsonObjArray("ArchSubset", "Fl2Sp", space.id!!, space.floorId!!)
                                    jsonArray.add(toJsonObj)
                                }
                                val createRel = EquipApi.createRel(dictProject.groupCode!!, message.projectId!!, jsonArray)
                                if (!createRel){
                                    message.sign = 2
                                    message.count = message.count +1
                                }
                            }
                        } else {
                            message.sign = 2
                            message.count = message.count +1
                        }
                    }else {
                        /** 失败 */
                        message.sign = 2
                        message.count = message.count +1
                    }

                }else {
                    message.sign = 0
                }
                /** 更改标记 */
                synchronousMessageObjService.update(message)
            }
            Configure.UPDATE -> {
                /** 修改 */
                /**查询信息 */
                val space = spaceService.select(
                    SFilter.eq("projectId", message.projectId!!),
                    SFilter.eq("id", message.objId!!)
                ).entity()
                /** 不为空 */
                if (space!=null) {
                    val spaceList = SpaceUtil.spaceTools(message.projectId!!, space)
                    val sign = EquipApi.updateObj(dictProject!!.groupCode!!,message.projectId!!, spaceList)
                    if (sign){
                        /** 成功 */
                        message.sign = 0
                    }else {
                        /** 失败 */
                        message.sign = 2
                        message.count = message.count +1
                    }
                }else {
                    message.sign = 0
                }
                /** 更改标记 */
                synchronousMessageObjService.update(message)
            }
            Configure.DELETE -> {
                /** 删除 */
               val sign = EquipApi.deleteObj(dictProject!!.groupCode!!,message.projectId!!, message.objId!!)
                if (sign){
                    /** 成功 */
                    message.sign = 0
                }else {
                    /** 失败 */
                    message.sign = 2
                    message.count = message.count +1
                }
                /** 更改标记 */
                synchronousMessageObjService.update(message)
            }
        }
    }

    /**
     * 创建关系列表
     *
     * @param graphCode     图类型编码
     * @param relCode       边类型编码
     * @param objFrom       主对象 id
     * @param objTo         从对象 id
     */
    fun toJsonObjArray(graphCode: String, relCode: String, objFrom: String, objTo: String): JSONObject {
//        /** 列表 */
//        val jsonArray = JSONArray()
        /** 对象 */
        val jsonObject = JSONObject()
        /** 图类型编码 */
        jsonObject["graphCode"] = graphCode
        /** 边类型编码 */
        jsonObject["relCode"] = relCode
        /** 主对象 id */
        jsonObject["objFrom"] = objFrom
        /** 从对象 id */
        jsonObject["objTo"] = objTo
//        /** 添加到列表 */
//        jsonArray.plus(jsonObject)
        return jsonObject
    }

}