/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.datacenter.syn.services.jms

import com.alibaba.fastjson.JSONObject
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.dictnew.DictProject
import com.persagy.server.syn.Configure
import com.persagy.server.syn.jms.bean.SyncRwdDatacenterMessage
import com.persagy.server.syn.models.api.obj.BuildingApi
import com.persagy.server.syn.models.api.obj.EquipApi
import com.persagy.server.datacenter.syn.models.tools.EquipUtil
import com.persagy.server.datacenter.wanda.obj.Equipment
import com.persagy.service.SBaseService
import com.persagy.service.SObjectService
import org.springframework.stereotype.Service

/**
 * 数据中台 - 部件
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/11/2 22:48
 */
@Service
class RwdComponent {
    /** 建筑服务 */
    private val equipService = SBaseService(SMybatisDao(Equipment::class.java))
    /** 缓存对象数据 - 数据中台到数据中心 */
    var syncRwdMessageService = SObjectService(SMybatisDao(SyncRwdDatacenterMessage::class.java))
    /** 项目所属的集团 */
    private val dictProjectService = SObjectService(SMybatisDao(DictProject::class.java))

    /**
     * 部件
     *
     * @param message   消息
     */
    fun componentType(message: SyncRwdDatacenterMessage) {
        /** 类型 */
        when(message.type){
            Configure.RWDCREATE -> {
                /** 创建 */
                val dictProject = dictProjectService.select(SFilter.eq("id", message.projectId!!)).entity()
                if (dictProject!=null) {
                    val queryIdList = BuildingApi.queryId(dictProject.groupCode!!, message.projectId!!, message.targetId!!)
                    if (queryIdList.size>0){
                        val equipment = EquipUtil.equipSyns(
                            message.projectId!!,
                            dictProject!!.groupCode!!,
                            queryIdList[0]
                        )
                        /** 部件和设备的关系 */
                        val jsonObject2 =  JSONObject()
                        jsonObject2["graphCode"] = "MechSubset"
                        jsonObject2["relCode"] = "Eq2Ec"
                        jsonObject2["objTo"] = equipment.id
                        val queryRelationList =
                            EquipApi.queryRelation(dictProject.groupCode!!, message.projectId!!, jsonObject2)
                        equipService.replace(equipment)
                        if (queryRelationList.size>0){
                            val hashMap = queryRelationList[0]
                            if (hashMap.containsKey("objTo")){
                                equipment.parentId = hashMap["objTo"].toString()
                            }
                        }
                        message.state = 0
                    } else {
                        message.state = 2
                        message.count+1
                    }
                } else {
                    message.state = 2
                    message.count+1
                }
                /** 更改标记 */
                syncRwdMessageService.update(message, arrayListOf("state","count"))
            }
            Configure.RWDUPDATE -> {
                /** 修改 */
                val dictProject = dictProjectService.select(SFilter.eq("id", message.projectId!!)).entity()
                if (dictProject!=null) {
                    val queryIdList = BuildingApi.queryId(dictProject.groupCode!!, message.projectId!!, message.targetId!!)
                    if (queryIdList.size>0){
                        val floorDataConversions = EquipUtil.equipSyns(
                            message.projectId!!,
                            dictProject!!.groupCode!!,
                            queryIdList[0]
                        )
                        equipService.replace(floorDataConversions)
                        message.state = 0
                    } else {
                        message.state = 2
                        message.count+1
                    }
                } else {
                    message.state = 2
                    message.count+1
                }
                /** 更改标记 */
                syncRwdMessageService.update(message, arrayListOf("state","count"))
            }
            Configure.RWDDELETE -> {
                /** 删除 */
                val delete = equipService.delete(
                    SFilter.eq("projectId", message.projectId!!),
                    SFilter.eq("id", message.targetId!!)
                )
                if (delete) {
                    message.state = 0
                } else {
                    message.state = 2
                    message.count+1
                }
                /** 更改标记 */
                syncRwdMessageService.update(message, arrayListOf("state","count"))
            }
        }
    }

}