/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.controllers.dict

import com.persagy.server.datacenter.datacenter.models.entities.dictnew.DefClass
import com.persagy.server.datacenter.datacenter.models.entities.dictnew.DictMajor
import com.persagy.server.datacenter.wanda.response.DictInfoResponse
import com.persagy.server.service.dict.DefFuncIdService
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.springframework.web.bind.annotation.*

/**
 * 项目化字典
 *
 * @author 张维新
 */
@Tag(name = "项目化字典")
@RestController
@RequestMapping("/dict")
class DictCustomController {

    /**
     * 设备类型
     */
    @Operation(summary = "所有设备类型", description = "")
    @GetMapping(value = ["/equip/type"])
    fun dictTypeEquipQuery(): SQueryResponse<DefClass>{
        return DefFuncIdService.dictTypeEquipQuery()
    }

    /**
     * 类型下信息点
     */
    @Operation(summary = "类型下信息点", description = "")
    @GetMapping(value = ["/equip/info"])
    fun dictEquipInfoQuery(@RequestParam category: String): DictInfoResponse {
        return DefFuncIdService.dictEquipInfoQuery(category)
    }

    /**
     * 统计项目已有的设备类型
     */
    @Operation(summary = "统计项目已有的设备类型", description = "系统-设备树形结构")
    @GetMapping(value = ["/project/equip/type"])
    fun projectEquipType(): SQueryResponse<DictMajor>{
       return  DefFuncIdService.dictProjectTypeEquipQuery()
    }

    /**
     * 统计项目下已有的系统类型
     */
    @Operation(summary = "统计项目下已有的系统类型", description = "专业-系统树形结构")
    @GetMapping(value = ["/project/system/type"])
    fun projectSystemType(): SQueryResponse<DefClass>{
        return  DefFuncIdService.dictProjectTypeSystemQuery()
    }

    /**
     * 统计所有的系统类型
     */
    @Operation(summary = "统计所有的系统类型", description = "专业-系统树形结构")
    @GetMapping(value = ["/system/type"])
    fun systemType(): SQueryResponse<DefClass>{
        return  DefFuncIdService.dictTypeSystemQuery()
    }


}