/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.models

import com.persagy.server.datacenter.models.entities.base.BaseInfo
import com.persagy.server.datacenter.models.entities.dict.EquipCategory
import com.persagy.server.datacenter.datacenter.models.entities.objects.Property
import com.persagy.server.datacenter.models.entities.task.base.ScanTaskBase
import com.persagy.service.models.annotations.SCascade
import io.swagger.v3.oas.annotations.media.Schema
import javax.persistence.Column
import javax.persistence.Id
import javax.persistence.Table

/**
 * 设备信息实体类、部件实体类
 *
 * @author  张维新
 */
@Schema(description = "设备信息实体类")
@Table(name = "equipment")
open class EquipmentDel : BaseInfo() {

    /** 设备id */
    @Schema(description = "设备id",required = true)
    @Id
    @Column(name = "id")
    var id: String? = null

    /** 设备名称 */
    @Schema(description = "设备名称",required = true)
    @Column(name = "name")
    var name: String? = null

    /** 设备本地编码 */
    @Schema(description = "设备本地编码")
    @Column(name = "local_id")
    var localId: String? = null

    /** 设备本地名称 */
    @Schema(description = "设备本地名称")
    @Column(name = "local_name")
    var localName: String? = null

    /** 设备二维码照片 */
//    @Schema(description = "设备二维码照片")
//    @Column(name = "qr_code")
//    @JSONField(name = "EquipQRCode")
//    var qrCode: String? = null

    /** 设计图纸中编码 */
    @Schema(description = "设计图纸中编码")
    @Column(name = "cad_id")
    var cadId: String? = null

    /** BIM模型中编码 */
    @Schema(description = "BIM模型中编码")
    @Column(name = "bim_id")
    var bimId: String? = null

    /** BIM模型中坐标 */
    @Schema(description = "BIM模型中坐标")
    @Column(name = "bim_location")
    var bimLocation: String? = null

    /** 项目id */
    @Schema(description = "项目id",required = true)
    @Column(name = "project_id")
    var projectId: String? = null

    /** 设备类型code */
    @Schema(description = "设备类型code")
    @Column(name = "class_code")
    var classCode: String? = null

    /** 设备所在建筑id */
    @Schema(description = "设备所在建筑id")
    @Column(name = "building_id")
    var buildingId: String? = null

    /** 设备所在楼层id */
    @Schema(description = "设备所在楼层id")
    @Column(name = "floor_id")
    var floorId: String? = null

//    /** 设备关联的资产id */
    @Schema(description = "设备关联的资产id")
    @Column(name = "property_id",insertable = false,updatable = false )
    var propertyId: Boolean? = false

    /** 品牌型号id */
    @Schema(description = "品牌型号id")
    @Column(name = "dp_manufacturer_id")
    var dpManufacturerID: String? = null

//    /** Revit族 */
//    @Schema(description = "Revit族 ")
//    @Column(name = "family_name")
//    var familyName: String? = null
//
//    /** Revit族类型 */
//    @Schema(description = "Revit族类型 ")
//    @Column(name = "family_symbol")
//    var familySymbol: String? = null

    /** 供应商信息id */
    @Schema(description = "供应商信息id")
    @Column(name = "dp_supplier_id")
    var dpSupplierID: String? = null

    /** 维修商信息id */
    @Schema(description = "维修商信息id")
    @Column(name = "dp_maintainer_id")
    var dpMaintainerID: String? = null

    /** 保险公司信息id */
    @Schema(description = "保险公司信息id")
    @Column(name = "dp_insurer_id")
    var dpInsurerID: String? = null

    /** 台账参数 */
    @Schema(description = "台账参数")
    @Column(name = "ledger_param")
    var ledgerParam: HashMap<String,Any?>? = null

    /** 通讯参数 */
    @Schema(description = "通讯参数")
    @Column(name = "comm_param")
    var commParam: HashMap<String,Any?>? = null

    /** 技术参数 */
    @Schema(description = "技术参数")
    @Column(name = "tech_param")
    var techParam: HashMap<String,Any?>? = null

    /** 运行参数 */
    @Schema(description = "运行参数")
    @Column(name = "run_param")
    var runParam: HashMap<String,Any?>? = null

    /** 设定参数 */
    @Schema(description = "设定参数")
    @Column(name = "setting_param")
    var settingParam: HashMap<String,Any?>? = null

    /** 报警信息 */
    @Schema(description = "报警信息")
    @Column(name = "alram")
    var alram: HashMap<String,Any?>? = null

    /** 自定义 */
    @Schema(description = "自定义")
    @Column(name = "custom_param")
    var customParam: HashMap<String,Any?>? = null

    /** 部件关联设备id */
    @Schema(description = "部件关联设备id")
    @Column(name = "parent_id")
    var parentId: String? = null

    /** 温度传感器 */
    @Schema(description = "温度传感器")
    @Column(name = "temp_sensor")
    var tempSensor: HashMap<String,Any?>? = null

    /** 技术参数-通用 */
    @Schema(description = "技术参数-通用")
    @Column(name = "tech_param_general")
    var techParamGeneral: HashMap<String,Any?>? = null

    /** 运行参数-通用 */
    @Schema(description = "运行参数-通用")
    @Column(name = "run_param_general")
    var runParamGeneral: HashMap<String,Any?>? = null

    /** 速度传感器 */
    @Schema(description = "速度传感器")
    @Column(name = "speed_sensor")
    var speedSensor: HashMap<String,Any?>? = null

    /** 声音传感器 */
    @Schema(description = "声音传感器")
    @Column(name = "sound_sensor")
    var soundSensor: HashMap<String,Any?>? = null

    /** 湿度传感器 */
    @Schema(description = "湿度传感器")
    @Column(name = "humidity_sensor")
    var humiditySensor: HashMap<String,Any?>? = null

    /** 压力传感器 */
    @Schema(description = "压力传感器")
    @Column(name = "press_transd")
    var pressTransd: HashMap<String,Any?>? = null

    /** 热量传感器 */
    @Schema(description = "热量传感器")
    @Column(name = "heat_sensor")
    var heatSensor: HashMap<String,Any?>? = null

    /** 流量传感器 */
    @Schema(description = "流量传感器")
    @Column(name = "flow_sensor")
    var flowSensor: HashMap<String,Any?>? = null

    /** 电度传感器 */
    @Schema(description = "电度传感器")
    @Column(name = "electricity_sensor")
    var electricitySensor: HashMap<String,Any?>? = null

    /** 位移传感器 */
    @Schema(description = "位移传感器")
    @Column(name = "displ_sensor")
    var displSensor: HashMap<String,Any?>? = null

    /** 浓度传感器 */
    @Schema(description = "浓度传感器")
    @Column(name = "concent_sensor")
    var concentSensor: HashMap<String,Any?>? = null

    /** 图像传感器 */
    @Schema(description = "图像传感器")
    @Column(name = "image_sensor")
    var imageSensor: HashMap<String,Any?>? = null

    /** 对象类型 */
    @Schema(description = "对象类型")
    @Column(name = "object_type")
    override var objectType: String? = "Equipment"

    /** 反馈参数 */
    @Schema(description = "反馈参数")
    @Column(name = "feedback_param")
    var feedbackParam: HashMap<String,Any?>? = null

    /** 系统id集合 */
    @Schema(description = "系统id集合")
    var systemList: List<String>? = null

    /** 任务状态 待验证1、已验证0、未找到-1  */
    @Schema(description = "任务状态-待验证1、已验证0、未找到-1")
    @Column(name = "task_state")
    var taskState: Int? = null

    /** 模型文件id */
    @Schema(description = "模型文件id")
    @Column(name = "model_id")
    var modelId: String? = null

    /** 撕码贴码状态 */
    @Schema(description = "撕码贴码状态")
    @Column(name = "code_type")
    var codeType: Int? = null

    /** 楼层名称 */
    @Schema(description = "楼层名称")
    @Column(name = "floor_name")
    var floorName: String? = null

    /** 建筑名称 */
    @Schema(description = "建筑名称")
    @Column(name = "building_name")
    var buildingName: String? = null

    /** 级联-部件   table  对应关系表   idColumn 对应 本类的id    childIdColumn 级联对象的关系id */
    @Schema(description = "级联-部件")
    @SCascade(table="scantask.scan_task_base", idColumn="equip_id", childIdColumn="id")
    var scanTaskBase: ArrayList<ScanTaskBase>? = null

    /** 资产 */
    @Schema(description = "级联-资产")
    @SCascade(table="property", idColumn="equip_id", childIdColumn="id")
    var property: Property? = null

    /** 级联-设备分类 */
    @Schema(description = "级联-设备分类")
    @SCascade(table="equipment", idColumn="id", childIdColumn="category")
    var equipCategory: EquipCategory? = null


} // Class Equipment