/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.models.mappers

import com.persagy.server.models.labsl.EquipClassCode
import com.persagy.server.models.responses.EquipQueryCount
import org.apache.ibatis.annotations.Mapper
import org.apache.ibatis.annotations.Param
import org.apache.ibatis.annotations.Select
import org.apache.ibatis.annotations.SelectProvider
import org.springframework.stereotype.Repository

/**
 *
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/10/20 22:01
 */
@Mapper
@Repository
interface CustomSqlLabsl {
    /** 统计项目下设备类型 */
    @Select("SELECT class_code AS classCode  FROM equipment WHERE project_id = #{projectId} and CHAR_LENGTH(class_code) = 6 GROUP BY class_code")
    fun equipType(@Param("projectId") projectId: String):ArrayList<EquipClassCode>

    /** 统计项目下空间类型 */
    @Select("SELECT class_code AS classCode   FROM zone_space_base WHERE project_id = #{projectId}  GROUP BY class_code")
    fun spaceType(@Param("projectId") projectId: String):ArrayList<EquipClassCode>


    /** 平面图统计设备类型以及数量 */
    @Select("SELECT e.class_code AS code , (SELECT name FROM dict.def_class WHERE project_id = #{projectId} AND code = e.class_code ) ,  count('class_code') FROM equipment e WHERE e.project_id = #{projectId} AND e.building_id = #{buildingId} AND e.floor_id = #{floorId} AND e.class_code in ('FFFSSN') GROUP BY e.class_code")
    fun equipTypeCount(@Param("projectId") projectId: String,@Param("buildingId") buildingId: String,@Param("floorId") floorId: String)

    /** 平面图统计设备类型以及数量 */
    @SelectProvider(type = StatisticalQuantity::class, method = "equipTypeCount")
    fun equipTypeCounts(projectId: String, buildingId: String,floorId: String,types: ArrayList<String>): ArrayList<EquipQueryCount>?

}