/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.service

import com.persagy.database.SFilter
import com.persagy.database.extensions.keyValue
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.datacenter.models.entities.dictnew.DefClass
import com.persagy.server.datacenter.datacenter.models.entities.objects.Property
import com.persagy.server.datacenter.models.entities.rel.RSyEq
import com.persagy.server.models.EquipmentProSy
import com.persagy.server.service.base.Service
import com.persagy.server.service.objects.PropertyService
import com.persagy.server.synchronize.MessageHandler
import com.persagy.server.utils.IdUtils
import com.persagy.server.utils.ShaftNameUtil
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.utils.SSpringContextUtil
import org.json.JSONObject
import org.slf4j.LoggerFactory

/**
 * 设备系统服务
 *
 * @author wx
 */
object EquipmentProSyService : Service<EquipmentProSy>(SMybatisDao(EquipmentProSy::class.java)) {

    /** 日志 */
    private val logger = LoggerFactory.getLogger(Service::class.java)

    /** MQ消息对象 */
    private val messageHandler by lazy {
        SSpringContextUtil.getBean(MessageHandler::class.java) as MessageHandler
    }
    /** 平台集团对象类型 */
    val defClassService = SObjectService(SMybatisDao(DefClass::class.java))

    override fun onCreateBefore(equip: EquipmentProSy): Boolean {
        try {
            /** 查询准备类型 */
            val projectId = SPageContext.getHeader("projectId")
            if (!equip.id.isNullOrEmpty()) {
                val isStartsWith = equip.id.toString().startsWith("Eq")
                if (!isStartsWith) {
                    /** 资产id */
                    equip.id = IdUtils.equipmentIdCreate(projectId!!)
                }
            } else {
                /** 资产id */
                equip.id = IdUtils.equipmentIdCreate(projectId!!)
            }
            /** 项目id */
            equip.projectId = projectId
            /** 设备名字 */
            if (equip.name.isNullOrEmpty()) {
                val entity = defClassService.select(
                    SFilter.eq("projectId", equip.projectId!!),
                    SFilter.eq("code", equip.classCode!!)
                ).entity()
                if (entity!=null) {
                    equip.name = ShaftNameUtil.shaftName(entity.name+"-")
                }else {
                    equip.name = ShaftNameUtil.shaftName("设备-")
                }
            }

        } catch (e: Exception) {
            e.printStackTrace()
        }
        return true
    }

    /**
     * 创建成功回调函数
     *
     * @param   entityList          创建成功对象
     */
    override fun onCreateSuccess(entityList: ArrayList<EquipmentProSy>) {
        try {
            entityList.forEach {
                val jsonObject = JSONObject()
                jsonObject.put("type", "1")
                jsonObject.put("projectId",it.keyValue("projectId"))
                jsonObject.put("id", it.keyValue("id"))
                /** 发送消息 */
                messageHandler!!.sendMessage(jsonObject.toString())
            }
            logger.debug("设备创建完成 =******************************************************************")
            for (entity in entityList){
                if (entity.propertyId!!){
                    if (entity.classCode.isNullOrEmpty()){
                        continue
                    }
                    val property = Property()
                    property.projectId = entity.projectId
                    property.buildingId = entity.buildingId
                    property.floorId = entity.floorId
                    val equipFamily = EquipFamilyService.select(SFilter.eq("equipCode", entity.classCode!!)).entity()
                    property.family = equipFamily!!.family!!

                    val sCreateRequest = SCreateRequest<Property>()
                    var list = ArrayList<Property>()
                    property.equipId = entity.id
                    list.add(property)
                    sCreateRequest.content = list
                    PropertyService.createList(sCreateRequest)
                }
                if (!entity.systemList.isNullOrEmpty()){
                    for (syId in entity.systemList!!){
                        val rSyEq = RSyEq()
                        rSyEq.equipId = entity.id
                        rSyEq.sysId = syId
                        rSyEq.projectId = entity.projectId
                        rSyEq.sign = 1
                        RSyEqService.create(rSyEq)
                    }
                }
            }
            logger.debug("资产创建完成 =******************************************************************")
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return
    } // Function onCreateSuccess()




} // Object EquipmentProSyService