/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.service


import com.persagy.database.SPageQueryInfo
import com.persagy.database.SQueryBuilder
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.models.PointPorperty
import com.persagy.server.models.PropertyFamily
import com.persagy.server.service.base.Service
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.responses.SQueryResponse
import org.slf4j.LoggerFactory

/**
 * 资产关联设备信息服务
 *
 * @author 张维新
 */
object PropertyFamilyService : Service<PropertyFamily>(SMybatisDao(PropertyFamily::class.java)) {
    /** 日志 */
    private val logger = LoggerFactory.getLogger(PropertyFamilyService::class.java)


   val pointPorpertyService = SObjectService(SMybatisDao(PointPorperty::class.java))

    /**
     * 查询之前的操作
     */
    override fun onQueryBefore(queryBuilder: SQueryBuilder<PropertyFamily>) {
        val projectId = SPageContext.getHeader("projectId").toString()
        queryBuilder.tableName = "f_project_family('$projectId')"
        return
    } // Function onQueryBefore()

    /**
     * 查询项目下的设备族分类
     *
     * @param   buildingId      建筑id
     * @param   floorId         楼层id
     * @param   startTime       开始时间
     * @param   endingTime      结束时间
     * @param   pointId         点位标签id
     * @param   checked         是否修订
     */
    fun queryFamily(buildingId: String?, floorId: String?,startTime: String?, endingTime: String?,pointId: String?, checked: String?): SQueryResponse<PropertyFamily> {
        val pageQuery: SPageQueryInfo<PropertyFamily>
        return try {
            var request = SQueryRequest()
            val projectId = SPageContext.getHeader("projectId").toString()
            val queryBuilder = select(request.filters)
            queryBuilder.tableName = "f_project_family('$projectId','$buildingId','$floorId','$startTime','$endingTime','$pointId','$checked')"
            pageQuery = queryBuilder.pageQuery(1, 1000)
            SQueryResponse(pageQuery)
        } catch (e: Exception) {
            e.printStackTrace()
            SQueryResponse(SResponseType.failure, e.message!!)
        }
    } // Function queryFamily()

    /**
     * 查询项目下的设备族分类
     *
     * @param   buildingId      建筑id
     * @param   floorId         楼层id
     * @param   startTime       开始时间
     * @param   endingTime      结束时间
     */
    fun queryPointPorperty(buildingId: String?, floorId: String?,startTime: String?, endingTime: String?): SQueryResponse<PointPorperty>{
        val pageQuery: SPageQueryInfo<PointPorperty>
        return try {
            var request = SQueryRequest()
            val projectId = SPageContext.getHeader("projectId").toString()
            val queryBuilder = pointPorpertyService.select(request.filters)
            queryBuilder.tableName = "f_point_poroerty('$projectId','$buildingId','$floorId','$startTime','$endingTime')"
            pageQuery = queryBuilder.pageQuery(1, 1000)
            SQueryResponse(pageQuery)
        } catch (e: Exception) {
            e.printStackTrace()
            SQueryResponse(SResponseType.failure, e.message!!)
        }
    } // Function queryPointPorperty()


} // Object PropertyService
