/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.service

import com.persagy.base.extensions.toJson
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.rel.REqSyList
import com.persagy.server.datacenter.models.entities.rel.RSyEq
import com.persagy.server.datacenter.models.entities.rel.RSyEqList
import com.persagy.server.service.base.RService
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse


/**
 * 系统下的设备设施关系服务
 *
 * @author 张维新
 */
object RSyEqService : RService<RSyEq>(SMybatisDao(RSyEq::class.java)) {

    /**
     * 创建系统下设备关系
     *
     * @param   rSyEq   系统和设备的关系对象
     */
    fun create(rSyEq: RSyEq): SCreateResponse<RSyEq> {
        return try {
//            rSyEq.id = IdUtils.uuidCreate()
            val isInsert = replace(rSyEq)
            if (isInsert){
                SCreateResponse(SResponseType.success)
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function create()

    /**
     * 根据对象删除
     *
     * @param   rSyEq      关系对象实例
     * @return
     */
     fun deletes(rSyEq: RSyEq): SBaseResponse {
        return try {
                val isDelete = delete(rSyEq)
                /** 删除成功 */
                if (isDelete) {
                    val sBaseResponse = SBaseResponse(SResponseType.success)
                    sBaseResponse.message = rSyEq.toJson()
                    sBaseResponse
                } else {
                    SBaseResponse(SResponseType.failure)
                }
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Function deletes()

    /**
     * 创建系统下设备关系
     *
     * @param   rSyEqList   系统和设备的关系对象列表
     */
    fun createSy(rSyEqList: RSyEqList): SCreateResponse<RSyEq> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("sysID = '${rSyEqList.sysId}'")
            for (eqId in rSyEqList.equipIdList!!){
                val rSyEq = RSyEq()
//                rSyEq.id = IdUtils.uuidCreate()
                rSyEq.equipId = eqId
                rSyEq.sysId =  rSyEqList.sysId
                val replace = replace(rSyEq)
                if (replace){
                    successList.add(rSyEq)
                }else{
                    failureList.add(rSyEq)
                }
            }
            if (successList.size>0){
                val sCreateResponse = SCreateResponse<RSyEq>(SResponseType.success)
                if (failureList.size>0){
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                }
                sCreateResponse
            }else{
               SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createSy()

    /**
     * 创建系统下设备关系
     *
     * @param   rSyEqList   系统和设备的关系对象列表
     */
    fun createsEq(rSyEqList: REqSyList): SCreateResponse<RSyEq> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("equipId = '${rSyEqList.equipId}'")
            for (eqId in rSyEqList.sysIdList!!){
                val rSyEq = RSyEq()
//                rSyEq.id = IdUtils.uuidCreate()
                rSyEq.equipId = rSyEqList.equipId
                rSyEq.sysId = eqId
                val replace = replace(rSyEq)
                if (replace){
                    successList.add(rSyEq)
                }else{
                    failureList.add(rSyEq)
                }
            }
            if (successList.size>0){
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<RSyEq>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<RSyEq>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsEq()

} // Object RSyEqService