/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.service.labsl

import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.datacenter.models.entities.dictnew.DefClass
import com.persagy.server.datacenter.datacenter.models.entities.dictnew.DictMajor
import com.persagy.server.datacenter.models.entities.dictnew.DictProject
import com.persagy.server.datacenter.wanda.obj.Building
import com.persagy.server.datacenter.wanda.obj.Floor
import com.persagy.server.models.mappers.CustomSqlLabsl
import com.persagy.server.models.responses.EquipQueryCount
import com.persagy.service.SBaseService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SQueryResponse
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service

/**
 * 自定义 sql 服务
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/10/20 23:01
 */
@Service
class CustomSqlLabslService {
    // 日志
    private val logger = LoggerFactory.getLogger(CustomSqlLabslService::class.java)
    /** 对象类型 */
    private val defClassService = SBaseService(SMybatisDao(DefClass::class.java))
    /** 字典项目服务 */
    private val dictProjectService = SBaseService(SMybatisDao(DictProject::class.java))
    /** 字典项目服务 */
    private val dictMajorService = SBaseService(SMybatisDao(DictMajor::class.java))
    /** 建筑服务 */
    private val buildingService = SBaseService(SMybatisDao(Building::class.java))
    /** 楼层服务 */
    private val floorService = SBaseService(SMybatisDao(Floor::class.java))

    /** 实例化 mapper 对象 */
    @Autowired
    lateinit var mapper: CustomSqlLabsl

    /**
     * 统计项目下的设备类型
     */
    fun equipTypeStatistics(): SQueryResponse<DefClass> {
        try {
            /** 项目 id */
            val projectId = SPageContext.getHeader("projectId")
            /** 返回对象 */
            val sQueryResponse = SQueryResponse<DefClass>()
            /** 返回标记 */
            sQueryResponse.result = SResponseType.success
            /** 查询项目对应的集团 */
            val dictProject = dictProjectService.select(SFilter.eq("id", projectId!!)).entity()
            /** 项目下设备类型列表 */
            val equipTypeList = mapper.equipType(projectId)
            /** 列表不可以为空 */
            if (equipTypeList.size > 0) {
                /** 类型列表 */
                val list = ArrayList<String>()
                /** 遍历类型 */
                for (equipType in equipTypeList) {
                    /** 添加类型 */
                    list.add(equipType.classCode!!)
                }
                /** 返回字典列表 */
//                val listCode = ArrayList<String>()
//                listCode.add("code")
//                listCode.add("name")
                /** 查询类型信息 */
                val defClassList =
                    defClassService.select(
                        SFilter.eq("type", "group"),
                        SFilter.eq("groupCode", dictProject!!.groupCode!!),
                        SFilter.inList("code", list)
                    ).exec()
                /** 系统类型 */
                val setSystem = HashSet<String>()
                for (defClass in defClassList){
                    setSystem.add(defClass.systemCode!!)
                }
                /** 系统类型列表 */
                val systemCodeList =
                    defClassService.select(
                        SFilter.eq("type", "group"),
                        SFilter.eq("groupCode", dictProject!!.groupCode!!),
                        SFilter.inList("code", setSystem.toList())
                    ).exec()
                for(systemCode in systemCodeList) {
                    for (defClass in defClassList) {
                        if (defClass.systemCode == systemCode.code) {
                            if (systemCode.children.isNullOrEmpty()) {
                                systemCode.children = ArrayList<DefClass>()
                            }
                            systemCode.children!!.add(defClass)
                        }
                    }
                }


                /** 专业 */
                val majorSet = HashSet<String>()
                for (systemCode in systemCodeList) {
                    majorSet.add(systemCode.majorCode!!)
                }
                /** 专业类型列表 */
                val majorCodeList =
                    dictMajorService.select(
                        SFilter.inList("code", majorSet.toList())
                    ).exec()
                val majorList = ArrayList<DefClass>()
                for (majorCode in majorCodeList) {
                    val defClass = DefClass()
                    defClass.code = majorCode.code
                    defClass.name = majorCode.name
                    defClass.aliasCode = majorCode.aliasCode
                    defClass.aliasName = majorCode.aliasName
                    for (systemCode in systemCodeList) {
                        if (systemCode.majorCode == majorCode.code ) {
                            if (defClass.children.isNullOrEmpty()) {
                                defClass.children = ArrayList<DefClass>()
                            }
                            defClass.children!!.add(systemCode)
                        }
                    }
                    majorList.add(defClass)
                }
//                        .projection(listCode).exec()
                /** 不需要的值置空 */
//                for (defClass in defClassList) {
//                    defClass.groupCode = null
//                    defClass.projectId = null
//                }
                /** 返回的对象列表 */
                sQueryResponse.content = majorList
                /** 总数量 */
                sQueryResponse.total = majorList.size.toLong()
            } else {
                sQueryResponse.result = SResponseType.failure
                sQueryResponse.message = "项目下没有设备类型"
            }

            return sQueryResponse
        } catch (e: Exception) {
            e.printStackTrace()
            return SQueryResponse(SResponseType.failure, e.message!!)
        }
    }

    /**
     * 项目下空间类型
     */
    fun spaceTypeStatistics(): SQueryResponse<DefClass> {
        try {
            /** 项目 id */
            val projectId = SPageContext.getHeader("projectId")
            /** 返回对象 */
            val sQueryResponse = SQueryResponse<DefClass>()
            /** 返回标记 */
            sQueryResponse.result = SResponseType.success
            /** 查询项目对应的集团 */
            val dictProject = dictProjectService.select(SFilter.eq("id", projectId!!)).entity()
            /** 项目下设备类型列表 */
            val spaceTypeList = mapper.spaceType(projectId)
            /** 列表不可以为空 */
            if (spaceTypeList.size > 0) {
                /** 类型列表 */
                val list = ArrayList<String>()
                /** 遍历类型 */
                for (spaceType in spaceTypeList) {
                    /** 添加类型 */
                    list.add(spaceType.classCode!!)
                }

                logger.debug("列表= ${list.toJson()}")
                /** 返回字典列表 */
                val listCode = ArrayList<String>()
                listCode.add("code")
                listCode.add("name")
                /** 查询类型信息 */
                val defClassList =
                    defClassService.select(
                        SFilter.eq("type", "group"),
                        SFilter.eq("groupCode", dictProject!!.groupCode!!),
                        SFilter.inList("code", list)
                    ).projection(listCode).exec()
                logger.debug("列表= ${defClassList.toJson()}")
                /** 不需要的值置空 */
                for (defClass in defClassList) {
                    defClass.groupCode = null
                    defClass.projectId = null
                }
                /** 返回的对象列表 */
                sQueryResponse.content = defClassList
                /** 总数量 */
                sQueryResponse.total = defClassList.size.toLong()
            } else {
                sQueryResponse.result = SResponseType.failure
                sQueryResponse.message = "项目下没有空间类型"
            }
            return sQueryResponse
        } catch (e: Exception) {
            e.printStackTrace()
            return SQueryResponse(SResponseType.failure, e.message!!)
        }
    }


    /**
     * 项目下的建筑楼层
     */
    fun buildingFloorStatistics(): SQueryResponse<Building> {
        try {
            /** 返回结果 */
            val sQueryResponse = SQueryResponse<Building>()
            /** 返回标记 */
            sQueryResponse.result = SResponseType.success
            /** 项目 id */
            val projectId = SPageContext.getHeader("projectId")
            /** 建筑列表 */
            val buildingList = buildingService.select(SFilter.eq("projectId", projectId!!)).exec()
            /** 遍历建筑 */
            for (building in buildingList) {
                /** 查询楼层 */
                val floorList =
                    floorService.select(SFilter.eq("projectId", projectId), SFilter.eq("buildingId", building.id!!)).order("floorSequenceId desc").exec()
                /** 建筑下楼层为空赋值 */
                if (building.floor.isNullOrEmpty()){
                    building.floor = ArrayList<Floor>()
                }

                /** 赋值 */
                building.floor = floorList
                /** 未明确楼层对象 */
                val floor = Floor()
                floor.id = "fisnull"
                floor.name = "未明确楼层"
                floor.localName = "未明确楼层"
                /** 添加到第一条 */
                building.floor!!.add(0,floor)
            }

            /** 未明确建筑对象 */
            val building = Building()
            building.id = "bisnull"
            building.name = "未明确建筑"
            building.localName = "未明确建筑"
            /** 添加到第一条 */
            buildingList.add(0,building)
            /** 列表赋值 */
            sQueryResponse.content = buildingList
            /** 数量 */
            sQueryResponse.total = buildingList.size.toLong()
            /** 返回结果 */
            return  sQueryResponse
        } catch (e: Exception) {
            e.printStackTrace()
            return SQueryResponse(SResponseType.failure,e.message!!)
        }
    }

    /**
     * 统计设备类型函数
     *
     * @param projectId     项目 id
     * @param buildingId    建筑 id
     * @param floorId       楼层 id
     * @param types         类型列表
     * @return 对象列表
     */
     fun equipTypeCounts(projectId: String, buildingId: String,floorId: String,types: ArrayList<String>): ArrayList<EquipQueryCount>?{
        return mapper.equipTypeCounts(projectId,buildingId,floorId,types)
     }



}