/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.service.labsl

import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.datacenter.models.entities.dictnew.DefClass
import com.persagy.server.datacenter.datacenter.models.entities.dictnew.DictMajor
import com.persagy.server.datacenter.models.entities.dictnew.DictProject
import com.persagy.server.models.labsl.EquipTypeLabsl
import com.persagy.server.service.base.Service
import com.persagy.service.SBaseService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SQueryResponse
import org.slf4j.LoggerFactory

/**
 * 设备服务
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/10/21 14:41
 */
object EquipTypeLabslService: Service<EquipTypeLabsl>(SMybatisDao(EquipTypeLabsl::class.java)) {

    /** 日志 */
    private val logger = LoggerFactory.getLogger(Service::class.java)

    /** 字典项目服务 */
    private val dictProjectService = SBaseService(SMybatisDao(DictProject::class.java))
    /** 对象类型 */
    private val defClassService = SBaseService(SMybatisDao(DefClass::class.java))
    /** 字典专业*/
    private val dictMajorService = SBaseService(SMybatisDao(DictMajor::class.java))

    /**
     * 设备对应的专业
     *
     * @param idList    id 列表
     * @return 返回结果
     */
    fun  equipMajor(idList: ArrayList<String>): SQueryResponse<DefClass> {
        try {
            /** 返回对象 */
            val sQueryResponse = SQueryResponse<DefClass>()
            /** 返回标记 */
            sQueryResponse.result = SResponseType.success
            if (idList.size>0) {
                /** 项目 id */
                val projectId = SPageContext.getHeader("projectId")
                val equipList = select(SFilter.eq("projectId", projectId!!), SFilter.inList("id", idList)).exec()
                if (equipList.size > 0) {
                val setType = HashSet<String>()
                for (equip in equipList) {
                    setType.add(equip.classCode!!)
                }
                /** 查询项目对应的集团 */
                val dictProject = dictProjectService.select(SFilter.eq("id", projectId!!)).entity()

                /** 查询类型信息 */
                val defClassList =
                    defClassService.select(
                        SFilter.eq("type", "group"),
                        SFilter.eq("groupCode", dictProject!!.groupCode!!),
                        SFilter.inList("code", setType.toList())
                    ).exec()
                val setMajor = HashSet<String>()
                for (defClass in defClassList) {
                    setMajor.add(defClass.majorCode!!)
                }
                /** 专业类型列表 */
                val majorCodeList =
                    dictMajorService.select(
                        SFilter.inList("code", setMajor.toList())
                    ).exec()
                val majorList = ArrayList<DefClass>()
                for (majorCode in majorCodeList) {
                    val defClass = DefClass()
                    defClass.code = majorCode.code
                    defClass.name = majorCode.name
                    defClass.aliasCode = majorCode.aliasCode
                    defClass.aliasName = majorCode.aliasName
                    for (def in defClassList) {
                        if (def.majorCode == majorCode.code) {
                            if (defClass.children.isNullOrEmpty()) {
                                defClass.children = ArrayList<DefClass>()
                            }
                            defClass.children!!.add(def)
                        }
                    }
                    majorList.add(defClass)
                }
                /** 返回的对象列表 */
                sQueryResponse.content = majorList
                /** 总数量 */
                sQueryResponse.total = majorList.size.toLong()
            } else{
                    sQueryResponse.result = SResponseType.failure
                    sQueryResponse.message = "设备不存在"
            }
            } else {
                sQueryResponse.result = SResponseType.failure
                sQueryResponse.message = "项目下没有设备类型"
            }
            return  sQueryResponse
        } catch (e: Exception) {
            e.printStackTrace()
            return SQueryResponse(SResponseType.failure,e.message!!)
        }
    }



}