/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.service.objects

import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.wanda.obj.GeneralSystem
import com.persagy.server.datacenter.wanda.rel.RSy2Sp
import com.persagy.server.service.RSyInShService
import com.persagy.server.service.base.Service
import com.persagy.service.SBaseService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.responses.SQueryResponse


/**
 * 系统信息服务
 *
 * @author 张维新
 */
object GeneralSystemService : Service<GeneralSystem>(SMybatisDao(GeneralSystem::class.java)) {

    val rSy2SpService = SBaseService(SMybatisDao(RSy2Sp::class.java))
    /**
     * 查询没有和当前空间绑定的系统
     *
     * @param   type        空间类型
     * @param   spaceId     空间id
     */
    fun unSpaceSys(type: String, spaceId: String, request: SQueryRequest): SQueryResponse<GeneralSystem> {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            val rSyInSpBaseList = rSy2SpService.select(SFilter.eq("Id2", spaceId), SFilter.eq("projectId", projectId!!),SFilter.eq("type", "sy2sp")).exec()
            val list = ArrayList<String>()
            val listf = ArrayList<SFilter>()
            for (rSyInSpBase in rSyInSpBaseList){
                list.add(rSyInSpBase.id1!!)
            }

            if (list.size>0){
                listf.add(SFilter.eq("projectId", projectId!!))
                listf.add(SFilter.not(SFilter.inList("SysID", list)))
            }else{
                listf.add(SFilter.eq("projectId", projectId!!))
            }

            pageQuery(request,listf)


//            val queryBuilder = select(request.filters)
//            queryBuilder.tableName="f_unsys_in_space('$projectId','$type','$spaceId')"
//            if (null != request.projection) {
//                queryBuilder.projection(request.projection!!)
//            }
//            if (null != request.orders) {
//                queryBuilder.order(request.orders!!)
//            }
//            val pageInfo = queryBuilder.pageQuery(request.pageNumber, request.pageSize)
//
//            SQueryResponse(pageInfo)
        } catch (e: Exception) {
            e.printStackTrace()
            SQueryResponse(SResponseType.failure, e.message!!)
        }

    } // Function  unSpaceSys

    /**
     * 查询没有和当前竖井绑定的系统
     *
     * @param   shaftId     竖井id
     */
    fun unShaftSys(shaftId: String, request: SQueryRequest): SQueryResponse<GeneralSystem> {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            val rSyInShaftBaseList = RSyInShService.select(SFilter.eq("shaftId", shaftId), SFilter.eq("projectId", projectId!!)).exec()
            val list = ArrayList<String>()
            val listf = ArrayList<SFilter>()
            for (rSyInSpBase in rSyInShaftBaseList){
                list.add(rSyInSpBase.sysId!!)
            }

            if (list.size>0){
                listf.add(SFilter.eq("projectId", projectId!!))
                listf.add(SFilter.not(SFilter.inList("id", list)))
            }else{
                listf.add(SFilter.eq("projectId", projectId!!))
            }
            val pageQuery = pageQuery(request, listf)
            pageQuery

        } catch (e: Exception) {
            e.printStackTrace()
            SQueryResponse(SResponseType.failure, e.message!!)
        }

    } // Function  unSpaceSys






} // Object GeneralSystemService