/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.service.objects

import com.persagy.server.utils.ShaftNameUtil
import com.persagy.database.SFilter
import com.persagy.database.SQueryBuilder
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.datacenter.models.entities.objects.Property
import com.persagy.server.service.EquipFamilyService
import com.persagy.server.service.base.Service
import com.persagy.server.utils.IdUtils
import com.persagy.service.SPageContext
import org.slf4j.LoggerFactory

/**
 * 资产信息服务
 *
 * @author 张维新
 */
object PropertyService : Service<Property>(SMybatisDao(Property::class.java)) {
    /** 日志 */
    private val logger = LoggerFactory.getLogger(PropertyService::class.java)

    /**
     * 创建之前的操作
     *
     * @param   property    创建的对象
     */
    override fun onCreateBefore(property: Property): Boolean  {
        if (!property.id.isNullOrEmpty()) {
            val isStartsWith = property.id.toString().startsWith("Pe")
            if (!isStartsWith) {
                /** 资产id */
                property.id = IdUtils.peIdCreate(property.projectId!!)
            }
        } else {
            /** 资产id */
            property.id = IdUtils.peIdCreate(property.projectId!!)
        }
//        /** 设备名字 */
//        if (property.name.isNullOrEmpty()) {
//            property.name = ShaftNameUtil.shaftName("资产-")
//        }
        if (!property.family.isNullOrEmpty()){
            val family = EquipFamilyService.select(SFilter.eq("family", property.family!!)).entity()
            if (family!=null){
                /** 资产名字 */
                if (property.name.isNullOrEmpty()) {
                    property.name = ShaftNameUtil.shaftName(family.familyName+"-")
                }
            }else{
                /** 资产名字 */
                if (property.name.isNullOrEmpty()) {
                    property.name = ShaftNameUtil.shaftName("资产-")
                }
            }
            return true
        }else{
            return false
        }
    } // Function onCreateBefore()

    /**
     * 查询之前的操作
     *
     * @param   queryBuilder    查询条件
     */
    override fun onQueryBefore(queryBuilder: SQueryBuilder<Property>) {
        queryBuilder.tableName = "v_property_equip"
        return
    } // Function onQueryBefore()

    override fun onUpdateBefore(entity: Property): Boolean {
        val projectId = SPageContext.getHeader("projectId")
        entity.projectId = projectId
        return super.onUpdateBefore(entity)
    } // Function onUpdateBefore()

} // Object PropertyService
