/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.service.objects

import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.datacenter.models.entities.objects.Tenant
import com.persagy.server.models.objects.Building
import com.persagy.server.models.objects.FloorCompon
import com.persagy.server.service.CustomSqlService
import com.persagy.server.service.base.Service
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SQueryResponse
import com.persagy.service.utils.SSpringContextUtil


/**
 * 租户信息服务
 *
 * @author
 */
object TenantService : Service<Tenant>(SMybatisDao(Tenant::class.java)) {

    /** 自定义sql */
    val customSqlService by lazy {
        SSpringContextUtil.getBean(CustomSqlService::class.java) as CustomSqlService
    }
    val serviceBuilding = SObjectService(SMybatisDao(Building::class.java))
    val serviceFloor = SObjectService(SMybatisDao(FloorCompon::class.java))

    /**
     * 查询楼层空间的数据以及数量
     */
    fun tenantBuildingFloors(tenantId: String,objectType: String): SQueryResponse<Building> {

        return try {
            val projectId = SPageContext.getHeader("projectId")
            val buildingList = serviceBuilding.select(SFilter.eq("projectId", projectId!!)).exec()
            if (buildingList.size>0){

                val tenantBuildingFloorList = customSqlService.tenantBuildingFloor(projectId, objectType, tenantId)

                for (building in buildingList){
                    val floorList = serviceFloor.select(SFilter.eq("projectId", projectId!!), SFilter.eq("buildId", building.id!!)).order("sequenceId desc").exec()

                    if (tenantBuildingFloorList.size>0){
                        for (floor in floorList){
                            for (tenantBuildingFloor in tenantBuildingFloorList){
                                if (floor.id == tenantBuildingFloor.id){
                                    floor.count = tenantBuildingFloor.count
                                    building.count = 1
                                }
                            }

                        }
                    }

                    building.floor = floorList
                }
            }
            val sQueryResponse = SQueryResponse<Building>(SResponseType.success)
            sQueryResponse.content = buildingList
            sQueryResponse.total = buildingList.size.toLong()
            sQueryResponse
        } catch (e: Exception) {
            e.printStackTrace()
            SQueryResponse(SResponseType.failure, e.message!!)
        }

    } // Function tenantBuildingFloors()


} // Object TenantService