/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.sso

import com.persagy.server.models.requests.UserChangePasswordRequest
import com.persagy.server.models.requests.UserDeleteRequest
import com.persagy.server.models.requests.UserUpdateRequest
import com.persagy.server.models.responses.BaseResponse
import com.persagy.server.models.responses.TokenCheckResponse
import com.persagy.server.models.responses.UserCreateRequest
import com.persagy.server.models.responses.UserCreateResponse
import com.persagy.service.utils.SSpringContextUtil
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.client.RestTemplate

/**
 * 用户服务
 *
 * @author  庞利祥(sybotan@126.com)
 */
object SSoClient {

    // 日志
    private val logger = LoggerFactory.getLogger(SSoClient::class.java)
    private const val SSO = "http://SAGACLOUD-SSO"

    /**
     * 创建用户
     */
    fun create(@RequestBody request: UserCreateRequest): UserCreateResponse {
        val user = HashMap<String, Any?>()
        user["username"] = request.userName
        user["password"] = request.password
        user["realname"] = request.realName
        user["phone"]    = request.phone
        user["mail"]     = request.mail
        user["certno"]   = request.certno

        val restTemplate = SSpringContextUtil.getBean(RestTemplate::class.java) as RestTemplate
        return restTemplate.postForObject("$SSO/server/user/create", user, UserCreateResponse::class.java) as UserCreateResponse
    } // Function create()

    /**
     * 更新用户
     */
    fun update(@RequestBody request: UserUpdateRequest): BaseResponse {
        val user = HashMap<String, Any?>()
        user["id"]       = request.id
        user["username"] = request.userName
        user["realname"] = request.realName
        user["phone"]    = request.phone
        user["mail"]     = request.mail
        user["certno"]   = request.certno

        val restTemplate = SSpringContextUtil.getBean(RestTemplate::class.java) as RestTemplate
        return restTemplate.postForObject("$SSO/server/user/update", user, BaseResponse::class.java) as BaseResponse
    } // Function update()

    /**
     * 删除用户
     */
    fun delete(@RequestBody request: UserDeleteRequest): BaseResponse {
        val user = HashMap<String, Any?>()
        user["id"]       = request.id

        val restTemplate = SSpringContextUtil.getBean(RestTemplate::class.java) as RestTemplate
        return restTemplate.postForObject("$SSO/server/user/delete", user, BaseResponse::class.java) as BaseResponse
    } // Function delete()

    /**
     * 修改用户密码
     */
    fun changePwd(@RequestBody request: UserChangePasswordRequest): BaseResponse {
        val user = HashMap<String, Any?>()
        user["id"]       = request.id
        user["newPwd"]   = request.newPwd
        user["oldPwd"]   = request.oldPwd

        val restTemplate = SSpringContextUtil.getBean(RestTemplate::class.java) as RestTemplate
        return restTemplate.postForObject("$SSO/server/user/delete", user, BaseResponse::class.java) as BaseResponse
    } // Function changePwd()

    /**
     * 审核token
     *
     * @param token     被审核的token
     */
    fun tokenCheck(@RequestBody token: String): TokenCheckResponse {
        val restTemplate = SSpringContextUtil.getBean(RestTemplate::class.java) as RestTemplate
        return restTemplate.getForObject("$SSO/auth/token/check?token=$token", TokenCheckResponse::class.java) as TokenCheckResponse
    } // Function tokenCheck()
} // Object SSoClient()
