/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */
package com.persagy.server.sso

import com.persagy.base.extensions.toJson
import com.persagy.server.models.Project
import com.persagy.server.models.responses.TokenCheckResponse
import com.persagy.server.models.responses.UserInfoResponse
import com.persagy.service.SPageContext
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.responses.SQueryResponse
import com.persagy.service.utils.SJsonUtil
import com.persagy.service.utils.SSpringContextUtil
import org.slf4j.LoggerFactory
import org.springframework.core.ParameterizedTypeReference
import org.springframework.http.HttpEntity
import org.springframework.http.converter.StringHttpMessageConverter
import org.springframework.web.client.RestTemplate
import java.nio.charset.StandardCharsets

object UserService {
    // 日志
    private val logger = LoggerFactory.getLogger(UserService::class.java)
    private const val DATACENTER = "http://DATACENTER"
//    private const val DATACENTER = "http://GUANCHENG-DATACENTER"
//    val opts: Opts by lazy {
//        SSpringContextUtil.getBean(Opts::class.java) as Opts
//    } // opts

    /**
     * 获得用户信息
     */
    fun userInfo(): UserInfoResponse {
        val user = SPageContext.getAttribute("User") as TokenCheckResponse?
        val resp = UserInfoResponse()
        try {
            logger.debug("项目= ${user?.toJson()}")
            val projResp = projectList(user!!.projects!!)

            // 获得项目
            for (pj in projResp!!.content!!) {
                val proj = Project(pj.id!!, "", pj.localName)
                resp.projects.add(proj)
            }

            resp.userId = user.id
            resp.username = user.userName
            resp.permissions = arrayListOf("system:role:query", "system:role:create", "system:user:create",
                    "system:user:delete",
                    "database:hbase:query",
                    "system:role:query",
                    "system:user:query",
                    "database:redis:query",
                    "system:role:setOpts",
                    "system:role:update",
                    "system:user:update")
            logger.debug("userInfo = ${resp.toJson()}")
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return resp
    } // Function userInfo()

    /**
     * 项目列表
     *
     * @param   ids     项目ID列表
     * @return  从数据中心取得项目列表
     */
    fun projectList(ids: ArrayList<String>): SQueryResponse<Project>? {
        val restTemplate = SSpringContextUtil.getBean(RestTemplate::class.java) as RestTemplate
        restTemplate.messageConverters[1] = StringHttpMessageConverter(StandardCharsets.UTF_8)
        val req = SQueryRequest()
        req.filters = "id in ["
        for ((index, id) in ids.withIndex()) {
            if (index > 0) {
                req.filters += ", "
            }
            req.filters += "'$id'"
        }
        req.filters += "]"
        SQueryRequest.maxRow = 100000
        req.pageSize = 100000
        val request = HttpEntity(req)
        val pt = object : ParameterizedTypeReference<SQueryResponse<Project>>() {}
        val r = restTemplate.postForObject("$DATACENTER/datacenter/project/query", req, String::class.java)
        @Suppress("UNCHECKED_CAST")
        return SJsonUtil.fromJson(r!!, pt.type) as SQueryResponse<Project>
    } // Function projectList()

} // Object UserService
