/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.sso.users


import com.persagy.server.models.requests.UserChangePasswordRequest
import com.persagy.server.models.requests.UserDeleteRequest
import com.persagy.server.models.requests.UserUpdateRequest
import com.persagy.server.models.responses.BaseResponse
import com.persagy.server.models.responses.UserCreateRequest
import com.persagy.server.models.responses.UserCreateResponse
import com.persagy.server.models.responses.UserInfoResponse
import com.persagy.server.sso.SSoClient
import com.persagy.server.sso.UserService
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.*

/**
 * 用户接口
 *
 * @author 庞利祥
 */
@Tag(name = "用户接口")
@RestController
@RequestMapping("/user")
class UserController {
    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(UserController::class.java)
    } // Companion object

    /**
     * 创建用户
     */
    @Operation(summary = "创建用户", description = "")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody request: UserCreateRequest): UserCreateResponse {
        return SSoClient.create(request)
    } // Function create()

    /**
     * 更新用户
     */
    @Operation(summary = "更新用户", description = "")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: UserUpdateRequest): BaseResponse {
        return SSoClient.update(request)
    } // Function update()

    /**
     * 删除用户
     */
    @Operation(summary = "删除用户", description = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody request: UserDeleteRequest): BaseResponse {
        return SSoClient.delete(request)
    } // Function delete()

    /**
     * 修改用户密码
     */
    @Operation(summary = "修改用户密码", description = "")
    @PostMapping(value = ["/changePwd"])
    fun changePwd(@RequestBody request: UserChangePasswordRequest): BaseResponse {
        return SSoClient.changePwd(request)
    } // Function changePwd()

    @Operation(summary = "用户登录", description = "")
    @GetMapping(value = ["/userInfo"])
    // @RequiresPermissions("permission:retrieve")
    fun userInfo(): UserInfoResponse {
        return UserService.userInfo()
    } // Function userInfo

} // Class UserController
