/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.messagecenter.controllers

import com.persagy.base.extensions.toJson
import com.persagy.server.messagecenter.models.entities.Message
import com.persagy.server.messagecenter.models.request.MessageRequest
import com.persagy.server.messagecenter.services.MessageService
import com.persagy.server.messagecenter.services.activemq.JMSProducer
import com.persagy.service.models.requests.SCountRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCountResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.apache.activemq.command.ActiveMQTopic
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 * 消息中心接口
 *
 * @author  邓婷婷
 */
@Tag(name = "消息中心接口")
@RestController
@RequestMapping("/message")
open class MessageController {

    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(MessageController::class.java)
    } // Companion object

    @Autowired
    lateinit var producer: JMSProducer

    val destination = ActiveMQTopic("message.manage")

    /**
     * 查询消息列表
     */
    @Operation(summary = "查询消息列表", description = "级联： 项目(project)")
    @PostMapping(value = ["/page-query"])
    fun pageQuery(@RequestBody request: SQueryRequest): SQueryResponse<Message> {
        logger.debug("MessageQuery =${request.toJson()}")
//        val projectId = SPageContext.getHeader("projectId");
//
//        if(projectId != null){
//            val withFilterList = ArrayList<SFilter>()
//            withFilterList.add(SFilter.eq("projectId",projectId ))
//
//            return MessageService.pageQuery(request,withFilterList)
//        }
        return MessageService.pageQuery(request)

    }

    /**
     * 创建消息
     *
    {
    "Content": {
    "ButtonList": [
    {
    "FileName": "ca7de892f14411e9b20d17b4c79481be.rvt",
    "Name": "下载报告",
    "Url": "/image-service/common/file_get?systemId=revit&key=model/ca7de892f14411e9b20d17b4c79481be.rvt"
    }
    ],
    "Message": "3楼模型损坏，请重新上传"
    },
    "Module": "模型文件管理",
    "ProjectId": "Pj4201050001",
    "Title": "3楼模型损坏，请重新上传",
    "Type": "Error",
    "UserList": [
    "234567"
    ]
    }

     */
    @Operation(summary = "创建消息", description = "消息类型：提示(Info)、警告(Warning)、错误(Error)、成功(Success)；")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody request: MessageRequest): SCreateResponse<Message> {
        logger.debug("message =${request.toJson()}")
//        println("content: "+ request.toJson())
        producer.sendMessage(destination,request.toJson())
        return MessageService.createList(MessageService.createBefore(request))
    }

    /**
     * 更新消息状态
     */
    @Operation(summary = "更新消息状态", description = "是否读过：未读（false）、已读（true）")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<Message>): SBaseResponse {
        return MessageService.updateList(request)
    }

    /**
     * 删除消息
     */
    @Operation(summary = "删除消息", description = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<Message>): SBaseResponse {
        return MessageService.deleteByKeysList(idList)
    }

    /**
     * 统计数量
     */
    @Operation(summary = "统计数量", description = "查询总数量不传参数，查询 已读Read = true，未读 Read = false")
    @PostMapping(value = ["/count"])
    fun count(@RequestBody request: SCountRequest): SCountResponse {
//        val projectId = SPageContext.getHeader("projectId");
//        if(projectId != null){
//            val withFilterList = ArrayList<SFilter>()
//            withFilterList.add(SFilter.eq("projectId",projectId ))
//
//            return MessageService.count(request,withFilterList)
//        }
        return MessageService.count(request)
    }

    /**
     * 根据接收人将所有消息置为已读
     */
    @Operation(summary = "根据接收人将消息置为已读", description = "已读Read = true")
    @PostMapping(value = ["/update-state"])
    fun updateState(@RequestBody request: Message): SBaseResponse {
        return MessageService.updateMessageState(request)
    }

}