/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.controller

import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.entities.User
import com.persagy.server.entities.request.ProjectChangeRequest
import com.persagy.server.entities.request.UserLoginRequest
import com.persagy.server.entities.request.VerifyCodeRequest
import com.persagy.server.entities.response.BaseResponse
import com.persagy.server.entities.response.UserLoginResponse
import com.persagy.server.entities.response.UserResponse
import com.persagy.server.services.UserService
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext
import com.persagy.service.models.requests.SCountRequest
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCountResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.validation.annotation.Validated
import org.springframework.web.bind.annotation.*

/**
 * 用户信息接口
 *
 * @author 付国宝
 */
@Tag(name = "008、用户信息接口")
@RestController
@RequestMapping("/service/user")
open class UserController {
    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(UserController::class.java)
    } // Companion object

    private var baseService = SObjectService(SMybatisDao(User::class.java))

    /**
     * 扫楼用户登录
     *
     * @param   body        用户登录请求
     */
   @Operation(summary="扫楼用户登录", description ="")
    @PostMapping("login")
    @ResponseBody
    open fun login(@Validated @RequestBody body: UserLoginRequest): UserLoginResponse {
        //userLogService.write(BaseRequest.Comming.app, null, null, "用户登录", null, body.toLog())
        return UserService.login(body.phone!!, body.code!!)
    } // Function login()

    /**
     * 扫楼用户获得验证码
     *
     * @param   body        验证码请求
     */
   @Operation(summary="扫楼用户获得验证码", description ="")
    @PostMapping("verify_code")
    @ResponseBody
    open fun getVerifyCode(@Validated @RequestBody body: VerifyCodeRequest): BaseResponse {
        //userLogService.write(BaseRequest.Comming.app, null, null, "获得验证码", null, body.toLog())
        val userList = UserService.queryByPhone(body.phone!!)
        if (userList.isEmpty()) {
            return BaseResponse(BaseResponse.ResultCode.failure, "手机号‘${body.phone!!}’未注册！")
        }
        return UserService.getVerifyCode(body.phone!!)
    } // Function getVerifyCode()


    /**
     * 批量创建用户信息
     *
     * @param   request         用户信息对象列表
     * @return                  创建结果信息
     */
   @Operation(summary = "创建用户信息", description = "")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody request: SCreateRequest<User>): SCreateResponse<User> {
        logger.debug("user =${request.toJson()}")
        return UserService.createList(request)
    } // Function create()


    /**
     * 根据id删除用户信息
     *
     * @param   entityList  用户信息数组
     * @return              删除的结果信息
     */
   @Operation(summary = "根据id删除用户信息", description = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody entityList: ArrayList<User>): SBaseResponse {
        return UserService.deleteByKeysList(entityList)
    } // Function delete()

    /**
     * 更新用户信息
     *
     * @param   request     更新的内容对象
     * @return              更新的结果
     */
   @Operation(summary = "更新用户信息", description = "")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<User>): SBaseResponse {
        return UserService.updateList(request)
    } // Function update()

    /**
     * 扫楼用户切换项目
     */
   @Operation(summary="扫楼用户切换项目", description ="")
    @PostMapping("change_project")
    @ResponseBody
    open fun switchProject(@Validated @RequestBody body: ProjectChangeRequest): BaseResponse {
        try {
//            val user: cn.sagacloud.server.scanbuilding.entities.User? = userService.changeProject(body.projId!!, body.phone!!) ?: return BaseResponse()
            val user = UserService.select(SFilter.eq("projId", body.projId!!), SFilter.eq("phone", body.phone!!)).entity()?: return BaseResponse()
//            userLogService.write(body.comming!!, body.projId, user!!.userId,"用户登录", null, body.toLog())
            return UserResponse(user)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return BaseResponse()
    } // Function switchProject()


    /**
     * 查询用户信息
     *
     * @param   request     查询信息条件
     * @return              查询结果
     */
   @Operation(summary = "查询用户信息", description = "")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<User> {
        val projectId = SPageContext.getHeader("projectId")
        return UserService.pageQuery(request, arrayListOf(SFilter.eq("projId",projectId!!)))
    } // Function query()

    /**
     * 根据条件查询统计数量
     */
   @Operation(summary = "根据条件查询统计数量", description = "")
    @PostMapping(value = ["/count"])
    fun count(@RequestBody request: SCountRequest): SCountResponse {
        return UserService.count(request)
    } // Function count()


} // Class UserController