/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.controller.task

import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.server.datacenter.datacenter.models.entities.task.EquipScanTask
import com.persagy.server.datacenter.datacenter.models.entities.task.EquipScanTaskQuery
import com.persagy.server.datacenter.models.entities.assistant.TaskPic
import com.persagy.server.datacenter.models.entities.task.*
import com.persagy.server.services.EquipmentService
import com.persagy.server.services.task.EquipScanTaskQueryService
import com.persagy.server.services.task.EquipScanTaskService
import com.persagy.server.services.task.ScanTaskBaseService
import com.persagy.service.SPageContext
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController


/**
 * 设备台账任务接口
 *
 * @author  邓婷婷
 */
@Tag(name = "001、任务-设备台账任务接口")
@RestController
@RequestMapping("/task/equip")
open class EquipScanTaskController {

    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(EquipScanTaskController::class.java)
    } // Companion object

    /**
     * 根据条件查询设备台账任务
     *
     * @author 邓婷婷
     *
     * {
            "Cascade": [
                {
                "Name": "project"
                }
            ],
            "Filters": "TaskState = 1 || TaskState = -1",
            "PageNumber": 1,
            "PageSize": 50
        }
     */
    @Operation(summary = "查询设备台账任务", description = "级联查询对象：项目（project）、建筑（building）、楼层（floor）、方案（scanScheme）、部件（component）、设备分类（equipCategory）;" +
            "查询条件：任务类型（taskSource--int）、任务执行情况（taskState--int）、所属建筑（buildingId）、所属楼层（floorId）、设备类型（Category）")
    @PostMapping(value = ["/page-query"])
    fun pageQuery(@RequestBody request: SQueryRequest): SQueryResponse<EquipScanTaskQuery> {
        logger.debug("EquipScanTaskQuery =${request.toJson()}")
//        val equipScanTaskQueryService = SObjectService(SMybatisDao(EquipScanTaskQuery::class.java))
//        val withFilterList = ArrayList<SFilter>()
//        withFilterList.add(SFilter.eq("projectId", SPageContext.getHeader("projectId").toString()))
//        withFilterList.add(SFilter.lt(SLength("category"), 6))
        return EquipScanTaskQueryService.pageQuerys(request)
    } // Function pageQuery()

    @Operation(summary = "查询设备台账任务,包含部件", description = "级联查询对象：项目（project）、建筑（building）、楼层（floor）、方案（scanScheme）、部件（component）、设备分类（equipCategory）;" +
            "查询条件：任务类型（TaskSource--int）、任务执行情况（TaskState--int）、所属建筑（BuildingId）、所属楼层（FloorId）、设备类型（Category）")
    @PostMapping(value = ["/page-query-com"])
    fun pageQueryCom(@RequestBody request: SQueryRequest): SQueryResponse<EquipScanTaskQuery> {
        logger.debug("EquipScanTaskQuery =${request.toJson()}")
//        val equipScanTaskQueryService = SObjectService(SMybatisDao(EquipScanTaskQuery::class.java))
        val withFilterList = ArrayList<SFilter>()
        withFilterList.add(SFilter.eq("projectId", SPageContext.getHeader("projectId").toString()))
//        withFilterList.add(SFilter.lt(SLength("category"), 6))
        return EquipScanTaskQueryService.pageQuery(request, withFilterList)
    }

    /**
     * 创建设备台账任务
     *
     * @param   request        设备台账任务对象列表
     * @return                      创建结果信息
     * @author  邓婷婷
     *
     * {
            "Content": [
                {
                "EquipId": "Eq11022900026d3f63a95c1511e9a7ded32044b9b08d",
                "EquipName": "太阳能电池板-EQ3A3"
                }
            ]
        }
     */
    @Operation(summary = "创建设备台账", description = "所有属性名要以查询返回结果为准  例如：任务id应用TaskId；当任务为未找到任务时重新生成任务，需要将TaskState=-1带入;")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody request: SCreateRequest<EquipScanTask>): SCreateResponse<EquipScanTask> {
        logger.debug("equip =${request.toJson()}")
        return EquipScanTaskService.createList(EquipScanTaskService.createBefore(request))
    } // Function create()

    /**
     * 更新设备台账
     *
     * @param   request      设备台账对象列表
     * @return              结果信息
     * @author  邓婷婷
     *
     * {
    "Content": [
    {
    "TaskId": "1",
    "TaskState":1,
    }
    ]
    }
     *
     */
    @Operation(summary = "更新设备台账任务", description = "所有属性名要以查询返回结果为准  例如：任务id应用TaskId；")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<EquipScanTask>): SBaseResponse {
        return EquipScanTaskService.updateList(request)
    } // Function update()

    /**
     * 根据id删除设备台账
     *
     * @param   idList      id数组
     * @return              删除的结果信息
     *
     * @author  邓婷婷
     *
     * [
            {
            "TaskId": "3"
            }
        ]
     */
    @Operation(summary = "根据id删除设备台账", description = "所有属性名要以查询返回结果为准  例如：任务id应用TaskId；")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<EquipScanTask>): SBaseResponse {
        return EquipScanTaskService.deleteByKeysList(idList)
    } // Function delete()

    /**
     * 查询设备分类
     */
    @Operation(summary = "查询设备分类",description = "")
    @PostMapping(value = ["/category/query"])
    fun equipCategoryQuery(): SQueryResponse<EquipCategoryDic>{
        return ScanTaskBaseService.equipCategoryQuery("equip");
    } // Function EquipCategoryQuery()

    /**
     * 查询建筑和楼层
     */
    @Operation(summary = "查询建筑和楼层",description = "")
    @PostMapping(value = ["/equip-building-query"])
    fun buildingQuery(): SQueryResponse<BuildingDic>{
        return ScanTaskBaseService.buildingQuery("equip");
    } // Function BuildingQuery()

    /**
     * 查询设备任务下的部件类型及数量
     */
    @Operation(summary = "查询设备任务下的部件类型及数量",description = "TaskId、EquipId 必填  ")
    @PostMapping(value = ["/category-count"])
    fun categoryCount(@RequestBody equipScanTask: EquipScanTask): SQueryResponse<CategoryStatistics>{
        return ScanTaskBaseService.categoryCounts(equipScanTask)
    } // Fun

//    /**
//     * 获取建筑楼层
//     */
//    @Operation(summary = "查询设备任务的建筑楼层",description = "TaskId、EquipId 必填  ")
//    @PostMapping(value = ["/category-count"])
//    fun taskEquipBuildingFloor(){
//
//    } // Fun
    /**
     * 任务图片和安装位置同步到设备
     */
    @Operation(summary = "任务图片和安装位置同步到设备",description = "TaskId、EquipId 必填  ")
    @PostMapping(value = ["/equipinfo"])
    fun equipInfo(){
        val projectId = SPageContext.getHeader("projectId").toString()
        val equipScanTaskList = EquipScanTaskQueryService.select(SFilter.eq("projectId", projectId), SFilter.eq("taskState", 0),SFilter.not(SFilter.isNull("manufacturer"))).exec()
        if (equipScanTaskList.size>0){
            for (equipScanTask in  equipScanTaskList){
                val equipment = EquipmentService.select(SFilter.eq("id", equipScanTask.equipId!!)).entity()
                if (equipment!=null){
                    if (!equipScanTask.taskPicList.isNullOrEmpty()&&equipScanTask.taskPicList!!.size>0){
                        val photoDoc = HashMap<String,Any?>()
                        if (equipment.infos.isNullOrEmpty()) {
                            equipment.infos = HashMap<String,Any?>()

                            val map = HashMap<String,Any?>()
                            val picList = ArrayList<TaskPic>()
                            for (taskPic in equipScanTask.taskPicList!!){
                                if (!taskPic.name.isNullOrEmpty()&&taskPic.name == "铭牌照片"){
                                    val taskPic1 = TaskPic()
                                    taskPic1.key = taskPic.key
                                    taskPic1.createTime = taskPic.createTime
                                    taskPic1.name = taskPic.name
                                    taskPic1.type = taskPic.type
                                    val  nameplateList = ArrayList<TaskPic>()
                                    nameplateList.add(taskPic1)
                                    equipment.infos!!["nameplate"] = nameplateList
                                }else{
                                    picList.add(taskPic)
                                }
                            }
                            if (!equipScanTask.installLocation.isNullOrEmpty()){
                                equipment.infos!!["installLocation"] = equipScanTask.installLocation
                            }
                            equipment.infos!!["ic"] = picList
                            EquipmentService.update(equipment)
                        }else{
                            val picList = ArrayList<TaskPic>()
                            for (taskPic in equipScanTask.taskPicList!!){
                                if (!taskPic.name.isNullOrEmpty()&&taskPic.name == "铭牌照片"){
                                    val taskPic1 = TaskPic()
                                    taskPic1.key = taskPic.key
                                    taskPic1.createTime = taskPic.createTime
                                    taskPic1.name = taskPic.name
                                    taskPic1.type = taskPic.type
                                    val  nameplateList = ArrayList<TaskPic>()
                                    nameplateList.add(taskPic1)
                                    equipment.infos!!["nameplate"] = nameplateList
                                }else{
                                    picList.add(taskPic)
                                }
                            }
                            if (!equipScanTask.installLocation.isNullOrEmpty()){
                                equipment.infos!!["installLocation"] = equipScanTask.installLocation
                            }
                            equipment.infos!!["pic"] = picList
                            EquipmentService.update(equipment)

                        }
                    }
                }
            }
        }
    } //



}