/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.controller.task

import com.persagy.database.SFilter
import com.persagy.database.functions.SLength
import com.persagy.server.datacenter.models.entities.task.CategoryStatistics
import com.persagy.server.datacenter.datacenter.models.entities.task.ModelScanTask
import com.persagy.server.services.task.ScanTaskBaseService
import com.persagy.service.SPageContext
import com.persagy.service.models.requests.SCountRequest
import com.persagy.service.models.responses.SCountResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController


/**
 * 任务父表接口
 *
 * @author  邓婷婷
 */
@Tag(name = "004、任务-总任务接口")
@RestController
@RequestMapping("/task/aggregate")
open class ScanTaskBaseController {

    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(ScanTaskBaseController::class.java)
    } // Companion object

    /**
     * 统计数量
     */
   @Operation(summary = "统计数量", description = "查询总数量不传参数，查询已完成数量TaskState=0，查询未找到TaskState=-1，查询未验证TaskState=1")
    @PostMapping(value = ["/count"])
    fun count(@RequestBody request: SCountRequest): SCountResponse {
        val withFilterList = ArrayList<SFilter>()
        withFilterList.add(SFilter.eq("projectId", SPageContext.getHeader("projectId").toString()))
        withFilterList.add(SFilter.lt(SLength("classCode"), 6))
        return ScanTaskBaseService.count(request,withFilterList)
    } // Function count()

    /**
     * 查询设备任务下的部件类型及数量
     */
   @Operation(summary = "查询设备任务下的部件类型及数量",description = "type 为model  是查询模型设备下的部件任务，默认是查设备台账任务  TaskId、EquipId 必填")
    @PostMapping(value = ["/category-count"])
    fun categoryCount(type: String?,@RequestBody equipScanTask: ModelScanTask): SQueryResponse<CategoryStatistics> {
        return ScanTaskBaseService.categoryCountsModel(type,equipScanTask)
    } // Fun


} // Class ScanTaskBaseController