/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.base

import com.persagy.database.SAbstractDao
import com.persagy.database.extensions.keyValue
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.datacenter.models.entities.base.BaseInfo
import com.persagy.server.datacenter.models.entities.logs.DataCenterLogs
import com.persagy.server.services.task.SchemeService
import com.persagy.server.util.IdUtils
import com.persagy.service.SObjectService
import com.persagy.service.SPageContext
import com.persagy.service.utils.SSpringContextUtil
import org.json.JSONArray
import org.json.JSONObject
import org.slf4j.LoggerFactory


/**
 * 服务基类
 *
 * @author 张维新
 */
open class Service<ENTITY : BaseInfo>(daoClass: SAbstractDao<ENTITY>) : SObjectService<ENTITY>(daoClass) {
    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(Service::class.java)

//        /** MQ消息对象 */
//        val messageHandler by lazy {
//            SSpringContextUtil.getBean(MessageHandler::class.java) as MessageHandler
//        }

        /** 自定义sql */
         val schemeService by lazy {
            SSpringContextUtil.getBean(SchemeService::class.java) as SchemeService
        }

        /** kafka消息对象 */
//        val kafkaService by lazy {
//            SSpringContextUtil.getBean(KafkaService::class.java) as KafkaService
//        }

    } // Companion object
    /** 日志 */
    var logService = SObjectService(SMybatisDao(DataCenterLogs::class.java))

    /**
     * 创建成功回调函数
     *
     * @param   entityList          创建成功对象
     */
    override fun onCreateSuccess(entityList: ArrayList<ENTITY>) {

        entityList.forEach {
            val projectId = SPageContext.getHeader("projectId")
            val comming = SPageContext.getHeader("coming")
            val account = SPageContext.getHeader("account")

            try {
                val jsonObject = JSONObject()
                jsonObject.put("type", "1")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", it.keyValue("id"))
                jsonObject.put("obj_type", it.objectType)
                /** 发送消息 */
//                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenter", ObjectNotice(projectId!!, it.objectType!!, "create",IdUtils.uuidCreate(), hashSetOf(it.keyValue("id").toString())),false)

            } catch (e: Exception) {
                e.printStackTrace()
            }
            try {
                /** 日志 */
                val dataCenterLogs = DataCenterLogs()
                dataCenterLogs.id = IdUtils.uuidCreate()
                dataCenterLogs.author = "dataCenter"
                dataCenterLogs.projectId = projectId
                dataCenterLogs.objectType = it.objectType
                dataCenterLogs.type = "create"
                dataCenterLogs.coming = comming
                dataCenterLogs.account = account
                dataCenterLogs.objectId = it.keyValue("id").toString()
                val hashMap = HashMap<String, Any?>()
                hashMap[it.objectType!!] = it
                dataCenterLogs.dataSource = hashMap
                logService.insert(dataCenterLogs)
            } catch (e: Exception) {
                e.printStackTrace()
            }


        }
//        entityList.forEach {
//            if (it.objectType =="Equipment") {
//                logger.debug("设备数据id${it.keyValue("id")}*******************************")
//                logger.debug("资产数据id${it.keyValue("propertyId")}*******************************")
//            }
//        }

        return
    } // Function onCreateSuccess()

    /**
     * 删除成功回调函数
     *
     * @param   entityList      删除成功对象列表
     */
    override fun onDeleteSuccess(entityList: ArrayList<ENTITY>) {
        val projectId = SPageContext.getHeader("projectId")
        entityList.forEach {

            try {
                val jsonObject = JSONObject()
                jsonObject.put("type", "2")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", it.keyValue("id"))
                jsonObject.put("obj_type", it.objectType)
                /** 发送消息 */
//                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenter", ObjectNotice(projectId!!, it.objectType!!, "delete",IdUtils.uuidCreate(), hashSetOf(it.keyValue("id").toString())),false)
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
        return
    } // Function onDeleteSuccess()


    override fun onUpdateBefore(entity: ENTITY): Boolean {

        try {
            val projectId = SPageContext.getHeader("projectId")
            val comming = SPageContext.getHeader("coming")
            val account = SPageContext.getHeader("account")
            /** 日志 */
            val dataCenterLogs = DataCenterLogs()
            dataCenterLogs.id = IdUtils.uuidCreate()
            dataCenterLogs.author = "dataCenter"
            dataCenterLogs.projectId = projectId
            dataCenterLogs.objectType = entity.objectType
            dataCenterLogs.type = "update-before"
            dataCenterLogs.account = account
            dataCenterLogs.coming = comming
            dataCenterLogs.objectId = entity.keyValue("id").toString()
            val hashMap = HashMap<String, Any?>()
            hashMap[entity.objectType!!] = entity
            dataCenterLogs.dataSource = hashMap
            logService.insert(dataCenterLogs)
        } catch (e: Exception) {
            e.printStackTrace()
    }

        return super.onUpdateBefore(entity)
    }

    /**
     * 更新成功能回调函数
     *
     * @param   entityList      更新成功对象列表
     */
    override fun onUpdateSuccess(entityList: ArrayList<ENTITY>) {
        try {
            val projectId = SPageContext.getHeader("projectId")
            val comming = SPageContext.getHeader("coming")
            val account = SPageContext.getHeader("account")
            val jsonObject = JSONObject()
            jsonObject.put("type", "7")
            jsonObject.put("projectId", projectId)
            jsonObject.put("obj_type", entityList[0].objectType)
            val jsonArray = JSONArray()
            for (entity in entityList) {
                val jsonInfos = JSONObject()
                jsonInfos.put("id", entity.keyValue("id"))
                jsonInfos.put("obj_type", entity.objectType)
                jsonArray.put(jsonInfos)

                try {
                    /** 日志 */
                    val dataCenterLogs = DataCenterLogs()
                    dataCenterLogs.id = IdUtils.uuidCreate()
                    dataCenterLogs.author = "dataCenter"
                    dataCenterLogs.projectId = projectId
                    dataCenterLogs.objectType = entity.objectType
                    dataCenterLogs.type = "update-after"
                    dataCenterLogs.account = account
                    dataCenterLogs.coming = comming
                    dataCenterLogs.objectId = entity.keyValue("id").toString()
                    val hashMap = HashMap<String, Any?>()
                    hashMap[entity.objectType!!] = entity
                    dataCenterLogs.dataSource = hashMap
                    logService.insert(dataCenterLogs)
                } catch (e: Exception) {
                    e.printStackTrace()
                }

                try {
                    /** 发送消息 */
//                    kafkaService.sendNotice("datacenter", ObjectNotice(projectId!!, entity.objectType!!, "update",IdUtils.uuidCreate(), hashSetOf(entity.keyValue("id").toString())),false)
                } catch (e: Exception) {
                    e.toString()
                }

            }
            jsonObject.put("infos", jsonArray)
            logger.debug("修改发送消息$jsonObject")
            /** 发送消息 */
//            messageHandler.sendMessage(jsonObject.toString())
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return
    } // Function onUpdateSuccess()

} // Class Service
