/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.task

import com.persagy.server.datacenter.datacenter.dao.mappers.SchemeMapper
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service

/**
 * 数据中心 方案导入服务
 *
 * @author 张维新
 */
@Service
open class SchemeService {

    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(SchemeService::class.java)
    } // Companion object

    /** 实例化 mapper 对象 */
    @Autowired
    lateinit var mapper: SchemeMapper

    /**
     * 导入方案处理
     *
     * @param      projectId        项目id
     * @param      schemeId         方案id
     */
    fun setScheme(projectId: String, schemeId: String):Boolean{
        try {
            mapper.setSchemeDict(projectId,schemeId)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return true
    } // Function setScheme()

    /**
     * 导入自定义信息点
     *
     * @param      projectId        项目id
     * @param      schemeId         方案id
     */
    fun setSchemeCustomDict(projectId: String, schemeId: String):Boolean{
        try {
            mapper.setSchemecustomDict(projectId,schemeId)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return true
    } // Function setSchemeCustomDict()

    /**
     * 导入自定义信息点
     *
     * @param      projectId        项目id
     * @param      schemeId         方案id
     */
    fun setSchemeCustomDictFalse(projectId: String, schemeId: String):Boolean{
        try {
            mapper.setSchemecustomDictFalse(projectId,schemeId)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return true
    } // Function setSchemeCustomDict()

    /**
     * 导入原始字典信息
     *
     * @param      projectId        项目id
     */
    fun setControlDict(projectId: String):Boolean{
        try {
            mapper.setControlDict(projectId)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return true
    } // Function setControlDict()


    /**
     * 处理自定义自定数据
     */
    fun setDictCustom():Boolean{
        try {
            val dict = mapper.setDict()
            return dict
        } catch (e: Exception) {
            e.printStackTrace()
            return false
        }
    }

} // Class SchemeService